@@ -1,4 +1,5 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
+import logging
 from pathlib import Path
 import pytest
 from numpy.testing import assert_allclose
@@ -9,8 +10,8 @@ from pydantic.error_wrappers import ValidationError
 from gammapy.analysis import Analysis, AnalysisConfig
 from gammapy.datasets import MapDataset, SpectrumDatasetOnOff
 from gammapy.maps import WcsGeom, WcsNDMap
-from gammapy.modeling.models import Models
-from gammapy.utils.testing import requires_data, requires_dependency
+from gammapy.modeling.models import DatasetModels
+from gammapy.utils.testing import requires_data
 
 CONFIG_PATH = Path(__file__).resolve().parent / ".." / "config"
 MODEL_FILE = CONFIG_PATH / "model.yaml"
@@ -132,18 +133,24 @@ def test_get_observations_missing_irf():
 
 @requires_data()
 def test_set_models():
-    config = get_example_config("1d")
+    config = get_example_config("3d")
     analysis = Analysis(config)
     analysis.get_observations()
     analysis.get_datasets()
     models_str = Path(MODEL_FILE).read_text()
     analysis.set_models(models=models_str)
-    assert isinstance(analysis.models, Models)
+    assert isinstance(analysis.models, DatasetModels)
+    assert len(analysis.models) == 2
+    assert analysis.models.names == ["source", "stacked-bkg"]
     with pytest.raises(TypeError):
         analysis.set_models(0)
 
+    new_source = analysis.models["source"].copy(name="source2")
+    analysis.set_models(models=[new_source], extend=False)
+    assert len(analysis.models) == 2
+    assert analysis.models.names == ["source2", "stacked-bkg"]
+
 
-@requires_dependency("iminuit")
 @requires_data()
 def test_analysis_1d():
     cfg = """
@@ -265,7 +272,6 @@ def test_exclusion_region(tmp_path):
     assert len(analysis.datasets) == 1
 
 
-@requires_dependency("iminuit")
 @requires_data()
 def test_analysis_1d_stacked_no_fit_range():
     cfg = """
@@ -339,13 +345,13 @@ def test_analysis_ring_3d():
 def test_analysis_no_bkg_1d(caplog):
     config = get_example_config("1d")
     analysis = Analysis(config)
-    analysis.get_observations()
-    analysis.get_datasets()
-    assert not isinstance(analysis.datasets[0], SpectrumDatasetOnOff)
-    assert "WARNING" in [_.levelname for _ in caplog.records]
-    assert "No background maker set. Check configuration." in [
-        _.message for _ in caplog.records
-    ]
+    with caplog.at_level(logging.WARNING):
+        analysis.get_observations()
+        analysis.get_datasets()
+        assert not isinstance(analysis.datasets[0], SpectrumDatasetOnOff)
+        assert "No background maker set. Check configuration." in [
+            _.message for _ in caplog.records
+        ]
 
 
 @requires_data()
@@ -353,16 +359,15 @@ def test_analysis_no_bkg_3d(caplog):
     config = get_example_config("3d")
     config.datasets.background.method = None
     analysis = Analysis(config)
-    analysis.get_observations()
-    analysis.get_datasets()
-    assert isinstance(analysis.datasets[0], MapDataset)
-    assert "WARNING" in [_.levelname for _ in caplog.records]
-    assert "No background maker set. Check configuration." in [
-        _.message for _ in caplog.records
-    ]
+    with caplog.at_level(logging.WARNING):
+        analysis.get_observations()
+        analysis.get_datasets()
+        assert isinstance(analysis.datasets[0], MapDataset)
+        assert "No background maker set. Check configuration." in [
+            _.message for _ in caplog.records
+        ]
 
 
-@requires_dependency("iminuit")
 @requires_data()
 def test_analysis_3d():
     config = get_example_config("3d")
@@ -401,17 +406,20 @@ def test_analysis_3d_joint_datasets():
     assert_allclose(
         analysis.datasets[0].background_model.spectral_model.norm.value,
         1.031743694988066,
+        rtol=1e-6,
     )
     assert_allclose(
-        analysis.datasets[0].background_model.spectral_model.tilt.value, 0.0
+        analysis.datasets[0].background_model.spectral_model.tilt.value,
+        0.0,
+        rtol=1e-6,
     )
     assert_allclose(
         analysis.datasets[1].background_model.spectral_model.norm.value,
         0.9776349021876344,
+        rtol=1e-6,
     )
 
 
-@requires_dependency("iminuit")
 @requires_data()
 def test_usage_errors():
     config = get_example_config("1d")
@@ -419,10 +427,43 @@ def test_usage_errors():
     with pytest.raises(RuntimeError):
         analysis.get_datasets()
     with pytest.raises(RuntimeError):
-        analysis.read_models(MODEL_FILE)
+        analysis.read_datasets()
+    with pytest.raises(RuntimeError):
+        analysis.write_datasets()
+    with pytest.raises(TypeError):
+        analysis.read_models()
+    with pytest.raises(RuntimeError):
+        analysis.write_models()
     with pytest.raises(RuntimeError):
         analysis.run_fit()
     with pytest.raises(RuntimeError):
         analysis.get_flux_points()
     with pytest.raises(ValidationError):
         analysis.config.datasets.type = "None"
+
+
+@requires_data()
+def test_datasets_io(tmpdir):
+    config = get_example_config("3d")
+
+    analysis = Analysis(config)
+    analysis.get_observations()
+    analysis.get_datasets()
+    models_str = Path(MODEL_FILE).read_text()
+    analysis.models = models_str
+
+    config.general.datasets_file = tmpdir / "datasets.yaml"
+    config.general.models_file = tmpdir / "models.yaml"
+    analysis.write_datasets()
+    analysis = Analysis(config)
+    analysis.read_datasets()
+    assert len(analysis.datasets.models) == 2
+    assert analysis.models.names == ["source", "stacked-bkg"]
+
+    analysis.models[0].parameters["index"].value = 3
+    analysis.write_models()
+    analysis = Analysis(config)
+    analysis.read_datasets()
+    assert len(analysis.datasets.models) == 2
+    assert analysis.models.names == ["source", "stacked-bkg"]
+    assert analysis.models[0].parameters["index"].value == 3
