@@ -5,14 +5,17 @@ import numpy as np
 from astropy import units as u
 from astropy.coordinates import AltAz, Angle, SkyCoord
 from astropy.coordinates.angle_utilities import angular_separation
+from astropy.io import fits
 from astropy.table import Table
 from astropy.table import vstack as vstack_tables
 from astropy.visualization import quantity_support
+import matplotlib.pyplot as plt
 from gammapy.maps import MapAxis, MapCoord, RegionGeom, WcsNDMap
 from gammapy.utils.fits import earth_location_from_dict
 from gammapy.utils.scripts import make_path
 from gammapy.utils.testing import Checker
 from gammapy.utils.time import time_ref_from_dict
+from .gti import GTI
 
 __all__ = ["EventList"]
 
@@ -22,7 +25,7 @@ log = logging.getLogger(__name__)
 class EventList:
     """Event list.
 
-    Event list data is stored in ``table`` (`~astropy.table.Table`) data member.
+    Event list data is stored as ``table`` (`~astropy.table.Table`) data member.
 
     The most important reconstructed event parameters
     are available as the following columns:
@@ -53,6 +56,32 @@ class EventList:
     ----------
     table : `~astropy.table.Table`
         Event list table
+
+    Examples
+    --------
+    >>> from gammapy.data import EventList
+    >>> events = EventList.read("$GAMMAPY_DATA/cta-1dc/data/baseline/gps/gps_baseline_110380.fits")
+    >>> print(events)
+    EventList
+    ---------
+    <BLANKLINE>
+      Instrument       : None
+      Telescope        : CTA
+      Obs. ID          : 110380
+    <BLANKLINE>
+      Number of events : 106217
+      Event rate       : 59.273 1 / s
+    <BLANKLINE>
+      Time start       : 59235.5
+      Time stop        : 59235.52074074074
+    <BLANKLINE>
+      Min. energy      : 3.00e-02 TeV
+      Max. energy      : 1.46e+02 TeV
+      Median energy    : 1.02e-01 TeV
+    <BLANKLINE>
+      Max. offset      : 5.0 deg
+    <BLANKLINE>
+
     """
 
     def __init__(self, table):
@@ -74,6 +103,74 @@ class EventList:
         table = Table.read(filename, **kwargs)
         return cls(table=table)
 
+    def to_table_hdu(self, format="gadf"):
+        """
+        Convert event list to a `~astropy.io.fits.BinTableHDU`
+
+        Parameters
+        ----------
+        format: str
+            Output format, currently only "gadf" is supported
+
+        Returns
+        -------
+        hdu: `astropy.io.fits.BinTableHDU`
+            EventList converted to FITS representation
+        """
+        if format != "gadf":
+            raise ValueError(f"Only the 'gadf' format supported, got {format}")
+
+        return fits.BinTableHDU(self.table, name="EVENTS")
+
+    def write(self, filename, gti=None, overwrite=False, format="gadf"):
+        """Write the event list to a FITS file.
+
+        If a GTI object is provided, it is saved into
+        a second extension in the file.
+
+        Parameters
+        ----------
+        filename : `pathlib.Path`, str
+            Filename
+        gti : `~gammapy.data.GTI`
+            Good Time Intervals object to save to the same file.
+            Default is None.
+        overwrite : bool
+            Overwrite existing file?
+        format : str, optional
+            FITS format convention.  By default files will be written
+            to the gamma-astro-data-formats (GADF) format.
+        """
+
+        if format != "gadf":
+            raise ValueError(f"{format} is not a valid EventList format.")
+
+        meta_dict = self.table.meta
+
+        if "HDUCLAS1" in meta_dict.keys() and meta_dict["HDUCLAS1"] != "EVENTS":
+            raise ValueError("The HDUCLAS1 keyword must be 'EVENTS' for an EventList")
+        else:
+            meta_dict["HDUCLAS1"] = "EVENTS"
+
+        if "HDUCLASS" in meta_dict.keys() and meta_dict["HDUCLASS"] != "GADF":
+            raise ValueError("The HDUCLASS must be 'GADF' for format 'gadf'")
+        else:
+            meta_dict["HDUCLASS"] = "GADF"
+
+        filename = make_path(filename)
+
+        primary_hdu = fits.PrimaryHDU()
+        hdu_evt = self.to_table_hdu(format=format)
+        hdu_all = fits.HDUList([primary_hdu, hdu_evt])
+
+        if gti is not None:
+            if not isinstance(gti, GTI):
+                raise TypeError("gti must be an instance of GTI")
+            hdu_gti = gti.to_table_hdu(format=format)
+            hdu_all.append(hdu_gti)
+
+        hdu_all.writeto(filename, overwrite=overwrite)
+
     @classmethod
     def from_stack(cls, event_lists, **kwargs):
         """Stack (concatenate) list of event lists.
@@ -202,15 +299,20 @@ class EventList:
 
         Examples
         --------
-        Use a boolean mask as ``row_specifier``:
 
-            mask = events.table['FOO'] > 42
-            events2 = events.select_row_subset(mask)
-
-        Use row index array as ``row_specifier``:
-
-            idx = np.where(events.table['FOO'] > 42)[0]
-            events2 = events.select_row_subset(idx)
+        >>> from gammapy.data import EventList
+        >>> import numpy as np
+        >>> events = EventList.read("$GAMMAPY_DATA/cta-1dc/data/baseline/gps/gps_baseline_110380.fits")
+        >>> #Use a boolean mask as ``row_specifier``:
+        >>> mask = events.table['MC_ID'] == 1
+        >>> events2 = events.select_row_subset(mask)
+        >>> print(len(events2.table))
+        97978
+        >>> #Use row index array as ``row_specifier``:
+        >>> idx = np.where(events.table['MC_ID'] == 1)[0]
+        >>> events2 = events.select_row_subset(idx)
+        >>> print(len(events2.table))
+        97978
         """
         table = self.table[row_specifier]
         return self.__class__(table=table)
@@ -303,6 +405,8 @@ class EventList:
         >>> event_list = EventList.read('$GAMMAPY_DATA/fermi_3fhl/fermi_3fhl_events_selected.fits.gz')
         >>> zd = (0, 30) * u.deg
         >>> event_list = event_list.select_parameter(parameter='ZENITH_ANGLE', band=zd)
+        >>> print(len(event_list.table))
+        123944
         """
         mask = band[0] <= self.table[parameter].quantity
         mask &= self.table[parameter].quantity < band[1]
@@ -330,8 +434,6 @@ class EventList:
         ax : `~matplotlib.axes.Axes` or None
             Axes
         """
-        import matplotlib.pyplot as plt
-
         ax = plt.gca() if ax is None else ax
 
         energy_axis = self._default_plot_energy_axis
@@ -363,8 +465,6 @@ class EventList:
         ax : `~matplotlib.axes.Axes`
             Axes
         """
-        import matplotlib.pyplot as plt
-
         ax = plt.gca() if ax is None else ax
 
         # Note the events are not necessarily in time order
@@ -423,16 +523,11 @@ class EventList:
 
         >>> from gammapy.data import EventList
         >>> from astropy import units as u
-        >>> import matplotlib.pyplot as plt
         >>> events = EventList.read('$GAMMAPY_DATA/hess-dl3-dr1/data/hess_dl3_dr1_obs_id_023523.fits.gz')
 
-        Plot the offset^2 distribution wrt. the observation pointing position
-        (this is a commonly used plot to check the background spatial distribution):
-
-        >>> plt.cla()
-        >>> events.plot_offset2_distribution()
-        <AxesSubplot:xlabel='Offset^2 (deg2)', ylabel='Counts'>
-
+        >>> #Plot the offset^2 distribution wrt. the observation pointing position
+        >>> #(this is a commonly used plot to check the background spatial distribution):
+        >>> events.plot_offset2_distribution() # doctest: +SKIP
         Plot the offset^2 distribution wrt. the Crab pulsar position
         (this is commonly used to check both the gamma-ray signal and the background spatial distribution):
 
@@ -440,15 +535,11 @@ class EventList:
         >>> from astropy.coordinates import SkyCoord
         >>> center = SkyCoord(83.63307, 22.01449, unit='deg')
         >>> bins = np.linspace(start=0, stop=0.3 ** 2, num=30) * u.deg ** 2
-        >>> plt.cla()
-        >>> events.plot_offset2_distribution(center=center, bins=bins)
-        <AxesSubplot:xlabel='Offset^2 (deg2)', ylabel='Counts'>
+        >>> events.plot_offset2_distribution(center=center, bins=bins) # doctest: +SKIP
 
         Note how we passed the ``bins`` option of `matplotlib.pyplot.hist` to control the histogram binning,
         in this case 30 bins ranging from 0 to (0.3 deg)^2.
         """
-        import matplotlib.pyplot as plt
-
         ax = plt.gca() if ax is None else ax
 
         if center is None:
@@ -483,7 +574,6 @@ class EventList:
         ax : `~matplotlib.pyplot.Axis`
             Plot axis
         """
-        import matplotlib.pyplot as plt
         from matplotlib.colors import LogNorm
 
         ax = plt.gca() if ax is None else ax
@@ -554,6 +644,22 @@ class EventList:
         ----------
         mask : `~gammapy.maps.Map`
             Mask
+
+        Returns
+        -------
+        event_list : `EventList`
+            Copy of event list with selection applied.
+
+        Examples
+        --------
+        >>> from gammapy.data import EventList
+        >>> from gammapy.maps import WcsGeom, Map
+        >>> geom = WcsGeom.create(skydir=(0,0), width=(4, 4), frame="galactic")
+        >>> mask = geom.region_mask("galactic;circle(0, 0, 0.5)")
+        >>> events = EventList.read("$GAMMAPY_DATA/cta-1dc/data/baseline/gps/gps_baseline_110380.fits")
+        >>> masked_event = events.select_mask(mask)
+        >>> len(masked_event.table)
+        5594
         """
         coord = self.map_coord(mask.geom)
         values = mask.get_by_coord(coord)
@@ -657,12 +763,50 @@ class EventList:
         -------
         event_list : `EventList`
             Copy of event list with selection applied.
+
+        Examples
+        --------
+        >>> from gammapy.data import EventList
+        >>> import astropy.units as u
+        >>> events = EventList.read("$GAMMAPY_DATA/cta-1dc/data/baseline/gps/gps_baseline_110380.fits")
+        >>> selected_events = events.select_offset([0.3, 0.9]*u.deg)
+        >>> len(selected_events.table)
+        12688
+
         """
         offset = self.offset
         mask = offset_band[0] <= offset
         mask &= offset < offset_band[1]
         return self.select_row_subset(mask)
 
+    def select_rad_max(self, rad_max, position=None):
+        """Select energy dependent offset
+
+        Parameters
+        ----------
+        rad_max : `~gamapy.irf.RadMax2D`
+            Rad max definition
+        position : `~astropy.coordinates.SkyCoord`
+            Center position. By default the pointing position is used.
+
+        Returns
+        -------
+        event_list : `EventList`
+            Copy of event list with selection applied.
+        """
+        if position is None:
+            position = self.pointing_radec
+
+        offset = position.separation(self.pointing_radec)
+        separation = position.separation(self.radec)
+
+        rad_max_for_events = rad_max.evaluate(
+            method="nearest", energy=self.energy, offset=offset
+        )
+
+        selected = separation <= rad_max_for_events
+        return self.select_row_subset(selected)
+
     @property
     def is_pointed_observation(self):
         """Whether observation is pointed"""
@@ -677,7 +821,6 @@ class EventList:
             Whether to look at the events allsky
         """
         import matplotlib.gridspec as gridspec
-        import matplotlib.pyplot as plt
 
         if allsky:
             gs = gridspec.GridSpec(nrows=2, ncols=2)
@@ -748,10 +891,20 @@ class EventList:
         m = m.smooth(width=0.5)
         return m
 
-    def plot_image(self, allsky=False):
-        """Quick look counts map sky plot."""
+    def plot_image(self, ax=None, allsky=False):
+        """Quick look counts map sky plot.
+
+        Parameters
+        ----------
+        ax : `~matplotlib.pyplot.Axes`
+            Axes to plot on.
+        allsky :  bool,
+            Whether to plot on an all sky geom
+        """
+        if ax is None:
+            ax = plt.gca()
         m = self._counts_image(allsky=allsky)
-        m.plot(stretch="sqrt")
+        m.plot(ax=ax, stretch="sqrt")
 
 
 class EventListChecker(Checker):
