@@ -2,6 +2,8 @@
 import copy
 from operator import le, lt
 import numpy as np
+import astropy.units as u
+from astropy.io import fits
 from astropy.table import Table, vstack
 from astropy.time import Time
 from astropy.units import Quantity
@@ -69,16 +71,23 @@ class GTI:
 
         Parameters
         ----------
-        start : `~astropy.units.Quantity`
-            start times w.r.t. reference time
-        stop : `~astropy.units.Quantity`
-            stop times w.r.t. reference time
+        start : `~astropy.time.Time` or `~astropy.units.Quantity`
+            Start times, if a quantity then w.r.t. reference time
+        stop : `~astropy.time.Time` or `~astropy.units.Quantity`
+            Stop times, if a quantity then w.r.t. reference time
         reference_time : `~astropy.time.Time`
             the reference time to use in GTI definition
         """
+        reference_time = Time(reference_time)
+
+        if isinstance(start, Time):
+            start = (start - reference_time).to(u.s)
+
+        if isinstance(stop, Time):
+            stop = (stop - reference_time).to(u.s)
+
         start = Quantity(start, ndmin=1)
         stop = Quantity(stop, ndmin=1)
-        reference_time = Time(reference_time)
         meta = time_ref_to_dict(reference_time)
         table = Table({"START": start.to("s"), "STOP": stop.to("s")}, meta=meta)
         return cls(table)
@@ -98,9 +107,36 @@ class GTI:
         table = Table.read(filename, hdu=hdu)
         return cls(table)
 
+    def to_table_hdu(self, format="gadf"):
+        """
+        Convert this GTI instance to a `astropy.io.fits.BinTableHDU`.
+
+        Parameters
+        ----------
+        format: str
+            Output format, currently only "gadf" is supported
+
+        Returns
+        -------
+        hdu: `astropy.io.fits.BinTableHDU`
+            GTI table converted to FITS representation
+        """
+        if format != "gadf":
+            raise ValueError(f'Only the "gadf" format supported, got {format}')
+
+        return fits.BinTableHDU(self.table, name="GTI")
+
     def write(self, filename, **kwargs):
-        """Write to file."""
-        self.table.write(make_path(filename), **kwargs)
+        """Write to file.
+
+        Parameters
+        ----------
+        filename : str or `Path`
+            File name to write to.
+        """
+        hdu = self.to_table_hdu()
+        hdulist = fits.HDUList([fits.PrimaryHDU(), hdu])
+        hdulist.writeto(make_path(filename), **kwargs)
 
     def __str__(self):
         return (
@@ -223,6 +259,28 @@ class GTI:
         table = Table({"START": start, "STOP": end}, names=["START", "STOP"])
         self.table = vstack([self.table, table])
 
+    @classmethod
+    def from_stack(cls, gtis, **kwargs):
+        """Stack (concatenate) list of event lists.
+
+        Calls `~astropy.table.vstack`.
+
+        Parameters
+        ----------
+        gtis : list of `GTI`
+            List of good time intervals to stack
+        **kwargs : dict
+            Keywords passed on to `~astropy.table.vstack`
+
+        Returns
+        -------
+        gti : `GTI`
+            Stacked good time intervals.
+        """
+        tables = [_.table for _ in gtis]
+        stacked_table = vstack(tables, **kwargs)
+        return cls(stacked_table)
+
     def union(self, overlap_ok=True, merge_equal=True):
         """Union of overlapping time intervals.
 
