@@ -210,7 +210,7 @@ class Datasets(collections.abc.MutableSequence):
 
         for dataset in self:
             if dataset.mask is not None:
-                value = dataset.mask.data.any()
+                value = np.any(dataset.mask)
             else:
                 value = True
             contributions.append(value)
@@ -298,7 +298,7 @@ class Datasets(collections.abc.MutableSequence):
             Region definition.
 
         Returns
-        --------
+        -------
         datasets : `Datasets`
             List of `~gammapy.datasets.SpectrumDataset`
         """
@@ -484,7 +484,8 @@ class Datasets(collections.abc.MutableSequence):
         if not self.is_all_same_type:
             raise ValueError("Info table not supported for mixed dataset type.")
 
-        stacked = self[0].to_masked(name="stacked")
+        name = "stacked" if cumulative else self[0].name
+        stacked = self[0].to_masked(name=name)
 
         rows = [stacked.info_dict()]
 
