@@ -1,4 +1,5 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
+import json
 import pytest
 import numpy as np
 from numpy.testing import assert_allclose
@@ -6,20 +7,24 @@ import astropy.units as u
 from astropy.coordinates import SkyCoord
 from astropy.table import Table
 from regions import CircleSkyRegion
+from gammapy.catalog import SourceCatalog3FHL
 from gammapy.data import GTI
 from gammapy.datasets import Datasets, MapDataset, MapDatasetOnOff
+from gammapy.datasets.map import RAD_AXIS_DEFAULT
 from gammapy.irf import (
-    EDispKernel,
     EDispKernelMap,
     EDispMap,
     EffectiveAreaTable2D,
     EnergyDependentMultiGaussPSF,
+    EnergyDispersion2D,
     PSFMap,
 )
-from gammapy.makers.utils import make_map_exposure_true_energy
-from gammapy.maps import Map, MapAxis, WcsGeom, WcsNDMap
+from gammapy.makers.utils import make_map_exposure_true_energy, make_psf_map
+from gammapy.maps import HpxGeom, Map, MapAxis, RegionGeom, WcsGeom, WcsNDMap
+from gammapy.maps.io import JsonQuantityEncoder
 from gammapy.modeling import Fit
 from gammapy.modeling.models import (
+    DiskSpatialModel,
     FoVBackgroundModel,
     GaussianSpatialModel,
     Models,
@@ -30,6 +35,34 @@ from gammapy.modeling.models import (
 from gammapy.utils.testing import mpl_plot_check, requires_data, requires_dependency
 
 
+@pytest.fixture
+def geom_hpx():
+    axis = MapAxis.from_energy_bounds("1 TeV", "10 TeV", nbin=3)
+
+    energy_axis_true = MapAxis.from_energy_bounds(
+        "1 TeV", "10 TeV", nbin=4, name="energy_true"
+    )
+
+    geom = HpxGeom.create(nside=32, axes=[axis], frame="galactic")
+
+    return {"geom": geom, "energy_axis_true": energy_axis_true}
+
+
+@pytest.fixture
+def geom_hpx_partial():
+    axis = MapAxis.from_energy_bounds("1 TeV", "10 TeV", nbin=3)
+
+    energy_axis_true = MapAxis.from_energy_bounds(
+        "1 TeV", "10 TeV", nbin=4, name="energy_true"
+    )
+
+    geom = HpxGeom.create(
+        nside=32, axes=[axis], frame="galactic", region="DISK(110.,75.,10.)"
+    )
+
+    return {"geom": geom, "energy_axis_true": energy_axis_true}
+
+
 @pytest.fixture
 def geom():
     axis = MapAxis.from_energy_bounds("0.1 TeV", "10 TeV", nbin=2)
@@ -42,6 +75,19 @@ def geom():
     )
 
 
+@pytest.fixture
+def geom1():
+    e_axis = MapAxis.from_energy_bounds("0.1 TeV", "10 TeV", nbin=20)
+    t_axis = MapAxis.from_bounds(0, 10, 2, name="time", unit="s")
+    return WcsGeom.create(
+        skydir=(266.40498829, -28.93617776),
+        binsz=0.02,
+        width=(3, 2),
+        frame="icrs",
+        axes=[e_axis, t_axis],
+    )
+
+
 @pytest.fixture
 def geom_etrue():
     axis = MapAxis.from_energy_bounds("0.1 TeV", "10 TeV", nbin=3, name="energy_true")
@@ -84,9 +130,33 @@ def get_psf():
     )
     psf = EnergyDependentMultiGaussPSF.read(filename, hdu="POINT SPREAD FUNCTION")
 
-    table_psf = psf.to_energy_dependent_table_psf(theta=0.5 * u.deg)
-    psf_map = PSFMap.from_energy_dependent_table_psf(table_psf)
-    return psf_map
+    geom = WcsGeom.create(
+        skydir=(0, 0),
+        frame="galactic",
+        binsz=2,
+        width=(2, 2),
+        axes=[RAD_AXIS_DEFAULT, psf.axes["energy_true"]],
+    )
+
+    return make_psf_map(
+        psf=psf,
+        pointing=SkyCoord(0, 0.5, unit="deg", frame="galactic"),
+        geom=geom,
+        exposure_map=Map.from_geom(geom.squash("rad"), unit="cm2 s"),
+    )
+
+
+@requires_data()
+def get_edisp(geom, geom_etrue):
+    filename = "$GAMMAPY_DATA/hess-dl3-dr1/data/hess_dl3_dr1_obs_id_020136.fits.gz"
+    edisp2d = EnergyDispersion2D.read(filename, hdu="EDISP")
+    energy = geom.axes["energy"].edges
+    energy_true = geom_etrue.axes["energy_true"].edges
+    edisp_kernel = edisp2d.to_edisp_kernel(
+        offset="1.2 deg", energy=energy, energy_true=energy_true
+    )
+    edisp = EDispKernelMap.from_edisp_kernel(edisp_kernel)
+    return edisp
 
 
 @pytest.fixture
@@ -103,7 +173,7 @@ def sky_model():
 
 
 def get_map_dataset(geom, geom_etrue, edisp="edispmap", name="test", **kwargs):
-    """Returns a MapDatasets"""
+    """Returns a MapDataset"""
     # define background model
     background = Map.from_geom(geom)
     background.data += 0.2
@@ -116,14 +186,14 @@ def get_map_dataset(geom, geom_etrue, edisp="edispmap", name="test", **kwargs):
 
     if edisp == "edispmap":
         edisp = EDispMap.from_diagonal_response(energy_axis_true=e_true)
+        data = exposure.get_spectrum(geom.center_skydir).data
+        edisp.exposure_map.data = np.repeat(data, 2, axis=-1)
     elif edisp == "edispkernelmap":
         edisp = EDispKernelMap.from_diagonal_response(
             energy_axis=e_reco, energy_axis_true=e_true
         )
-    elif edisp == "edispkernel":
-        edisp = EDispKernel.from_diagonal_response(
-            energy_true=e_true.edges, energy=e_reco.edges
-        )
+        data = exposure.get_spectrum(geom.center_skydir).data
+        edisp.exposure_map.data = np.repeat(data, 2, axis=-1)
     else:
         edisp = None
 
@@ -131,7 +201,6 @@ def get_map_dataset(geom, geom_etrue, edisp="edispmap", name="test", **kwargs):
     center = SkyCoord("0.2 deg", "0.1 deg", frame="galactic")
     circle = CircleSkyRegion(center=center, radius=1 * u.deg)
     mask_fit = geom.region_mask([circle])
-    mask_fit = Map.from_geom(geom, data=mask_fit)
 
     models = FoVBackgroundModel(dataset_name=name)
 
@@ -164,6 +233,11 @@ def test_map_dataset_str(sky_model, geom, geom_etrue):
     assert "MapDataset" in str(dataset)
 
 
+def test_map_dataset_str_empty():
+    dataset = MapDataset()
+    assert "MapDataset" in str(dataset)
+
+
 @requires_data()
 def test_fake(sky_model, geom, geom_etrue):
     """Test the fake dataset"""
@@ -180,7 +254,7 @@ def test_fake(sky_model, geom, geom_etrue):
 
     assert real_dataset.counts.data.shape == dataset.counts.data.shape
     assert_allclose(real_dataset.counts.data.sum(), 9525.299054, rtol=1e-5)
-    assert_allclose(dataset.counts.data.sum(), 9723)
+    assert_allclose(dataset.counts.data.sum(), 9711)
 
 
 @requires_data()
@@ -197,10 +271,10 @@ def test_different_exposure_unit(sky_model, geom):
 
     npred = dataset.npred()
 
-    assert_allclose(npred.data[0, 50, 50], 6.086019)
+    assert_allclose(npred.data[0, 50, 50], 6.086019, rtol=1e-2)
 
 
-@pytest.mark.parametrize(("edisp_mode"), ["edispmap", "edispkernelmap", "edispkernel"])
+@pytest.mark.parametrize(("edisp_mode"), ["edispmap", "edispkernelmap"])
 @requires_data()
 def test_to_spectrum_dataset(sky_model, geom, geom_etrue, edisp_mode):
 
@@ -221,7 +295,7 @@ def test_to_spectrum_dataset(sky_model, geom, geom_etrue, edisp_mode):
         on_region, containment_correction=True
     )
     mask = np.ones_like(dataset_ref.counts, dtype="bool")
-    mask[1, 40:60, 40:60] = 0
+    mask[1, 40:60, 40:60] = False
     dataset_ref.mask_safe = Map.from_geom(dataset_ref.counts.geom, data=mask)
     spectrum_dataset_mask = dataset_ref.to_spectrum_dataset(on_region)
 
@@ -230,14 +304,72 @@ def test_to_spectrum_dataset(sky_model, geom, geom_etrue, edisp_mode):
     assert spectrum_dataset.background.geom.axes[0].nbin == 2
     assert spectrum_dataset.exposure.geom.axes[0].nbin == 3
     assert spectrum_dataset.exposure.unit == "m2s"
-    assert spectrum_dataset.edisp.get_edisp_kernel().energy_axis.nbin == 2
-    assert spectrum_dataset.edisp.get_edisp_kernel().energy_axis_true.nbin == 3
-    assert_allclose(spectrum_dataset.edisp.exposure_map.data[1], 3.070884e09, rtol=1e-5)
+
+    energy_axis = geom.axes["energy"]
+    assert (
+        spectrum_dataset.edisp.get_edisp_kernel(energy_axis=energy_axis)
+        .axes["energy"]
+        .nbin
+        == 2
+    )
+    assert (
+        spectrum_dataset.edisp.get_edisp_kernel(energy_axis=energy_axis)
+        .axes["energy_true"]
+        .nbin
+        == 3
+    )
+
+    assert_allclose(spectrum_dataset.edisp.exposure_map.data[1], 3.069227e09, rtol=1e-5)
     assert np.sum(spectrum_dataset_mask.counts.data) == 0
     assert spectrum_dataset_mask.data_shape == (2, 1, 1)
     assert spectrum_dataset_corrected.exposure.unit == "m2s"
+
     assert_allclose(spectrum_dataset.exposure.data[1], 3.070884e09, rtol=1e-5)
-    assert_allclose(spectrum_dataset_corrected.exposure.data[1], 2.035899e09, rtol=1e-5)
+    assert_allclose(spectrum_dataset_corrected.exposure.data[1], 2.05201e09, rtol=1e-5)
+
+
+@requires_data()
+def test_energy_range(sky_model, geom1, geom_etrue):
+    sky_coord1 = SkyCoord(266.5, -29.3, unit="deg")
+    region1 = CircleSkyRegion(sky_coord1, 0.5 * u.deg)
+    mask1 = geom1.region_mask([region1]) & geom1.energy_mask(1 * u.TeV, 7 * u.TeV)
+    sky_coord2 = SkyCoord(266.5, -28.7, unit="deg")
+    region2 = CircleSkyRegion(sky_coord2, 0.5 * u.deg)
+    mask2 = geom1.region_mask([region2]) & geom1.energy_mask(2 * u.TeV, 8 * u.TeV)
+    mask3 = geom1.energy_mask(3 * u.TeV, 6 * u.TeV)
+
+    mask_safe = Map.from_geom(geom1, data=(mask1 | mask2 | mask3).data)
+    dataset = get_map_dataset(geom1, geom_etrue, edisp=None, mask_safe=mask_safe)
+    energy = geom1.axes["energy"].edges.value
+
+    e_min, e_max = dataset.energy_range_safe
+    assert_allclose(e_min.get_by_coord((265, -28, 0)), energy[15])
+    assert_allclose(e_max.get_by_coord((265, -28, 5)), energy[17])
+    assert_allclose(e_min.get_by_coord((sky_coord1.ra, sky_coord1.dec, 6)), energy[10])
+    assert_allclose(e_max.get_by_coord((sky_coord1.ra, sky_coord1.dec, 1)), energy[18])
+    assert_allclose(e_min.get_by_coord((sky_coord2.ra, sky_coord2.dec, 2)), energy[14])
+    assert_allclose(e_max.get_by_coord((sky_coord2.ra, sky_coord2.dec, 7)), energy[19])
+    assert_allclose(e_min.get_by_coord((266.5, -29, 8)), energy[10])
+    assert_allclose(e_max.get_by_coord((266.5, -29, 3)), energy[19])
+
+    e_min, e_max = dataset.energy_range_fit
+    assert_allclose(e_min.get_by_coord((265, -28, 0)), np.nan)
+    assert_allclose(e_max.get_by_coord((265, -28, 5)), np.nan)
+    assert_allclose(e_min.get_by_coord((266, -29, 4)), energy[0])
+    assert_allclose(e_max.get_by_coord((266, -29, 9)), energy[20])
+
+    e_min, e_max = dataset.energy_range
+    assert_allclose(e_min.get_by_coord((266, -29, 4)), energy[15])
+    assert_allclose(e_max.get_by_coord((266, -29, 9)), energy[17])
+
+    mask_zeros = Map.from_geom(geom1, data=np.zeros_like(mask_safe))
+    e_min, e_max = dataset._energy_range(mask_zeros)
+    assert_allclose(e_min.get_by_coord((266.5, -29, 8)), np.nan)
+    assert_allclose(e_max.get_by_coord((266.5, -29, 3)), np.nan)
+
+    e_min, e_max = dataset._energy_range()
+    assert_allclose(e_min.get_by_coord((265, -28, 0)), energy[0])
+    assert_allclose(e_max.get_by_coord((265, -28, 5)), energy[20])
 
 
 @requires_data()
@@ -271,6 +403,10 @@ def test_info_dict(sky_model, geom, geom_etrue):
 
     assert info_dict["name"] == "test"
 
+    # try to dump as json
+    result = json.dumps(info_dict, cls=JsonQuantityEncoder)
+    assert "counts" in result
+
 
 def get_fermi_3fhl_gc_dataset():
     counts = Map.read("$GAMMAPY_DATA/fermi-3fhl-gc/fermi-3fhl-gc-counts-cube.fits.gz")
@@ -376,6 +512,31 @@ def test_downsample():
         dataset.downsample(2, axis_name="energy")
 
 
+def test_downsample_energy(geom, geom_etrue):
+    # This checks that downsample and resample_energy_axis give identical results
+    counts = Map.from_geom(geom, dtype="int")
+    counts += 1
+    mask = Map.from_geom(geom, dtype="bool")
+    mask.data[1:] = True
+    counts += 1
+    exposure = Map.from_geom(geom_etrue, unit="m2s")
+    edisp = EDispKernelMap.from_gauss(geom.axes[0], geom_etrue.axes[0], 0.1, 0.0)
+    dataset = MapDataset(
+        counts=counts,
+        exposure=exposure,
+        mask_safe=mask,
+        edisp=edisp,
+    )
+    dataset_downsampled = dataset.downsample(2, axis_name="energy")
+    dataset_resampled = dataset.resample_energy_axis(geom.axes[0].downsample(2))
+
+    assert dataset_downsampled.edisp.edisp_map.data.shape == (3, 1, 1, 2)
+    assert_allclose(
+        dataset_downsampled.edisp.edisp_map.data[:, :, 0, 0],
+        dataset_resampled.edisp.edisp_map.data[:, :, 0, 0],
+    )
+
+
 @requires_data()
 def test_map_dataset_fits_io(tmp_path, sky_model, geom, geom_etrue):
     dataset = get_map_dataset(geom, geom_etrue)
@@ -389,6 +550,7 @@ def test_map_dataset_fits_io(tmp_path, sky_model, geom, geom_etrue):
     dataset.gti = gti
 
     hdulist = dataset.to_hdulist()
+
     actual = [hdu.name for hdu in hdulist]
 
     desired = [
@@ -434,6 +596,8 @@ def test_map_dataset_fits_io(tmp_path, sky_model, geom, geom_etrue):
 
     assert dataset.counts.geom == dataset_new.counts.geom
     assert dataset.exposure.geom == dataset_new.exposure.geom
+
+    assert_allclose(dataset.exposure.meta["livetime"], 1 * u.h)
     assert dataset.npred_background().geom == dataset_new.npred_background().geom
     assert dataset.edisp.edisp_map.geom == dataset_new.edisp.edisp_map.geom
 
@@ -455,8 +619,6 @@ def test_map_dataset_fits_io(tmp_path, sky_model, geom, geom_etrue):
     assert_allclose(stacked1.edisp.edisp_map, stacked.edisp.edisp_map)
 
 
-@requires_dependency("iminuit")
-@requires_dependency("matplotlib")
 @requires_data()
 def test_map_fit(sky_model, geom, geom_etrue):
     dataset_1 = get_map_dataset(geom, geom_etrue, name="test-1")
@@ -476,25 +638,25 @@ def test_map_fit(sky_model, geom, geom_etrue):
     models["test-1-bkg"].spectral_model.norm.value = 0.49
     models["test-2-bkg"].spectral_model.norm.value = 0.99
 
-    fit = Fit(datasets)
-    result = fit.run()
+    fit = Fit()
+    result = fit.run(datasets=datasets)
 
     assert result.success
     assert "minuit" in repr(result)
 
     npred = dataset_1.npred().data.sum()
     assert_allclose(npred, 7525.790688, rtol=1e-3)
-    assert_allclose(result.total_stat, 21659.2139, rtol=1e-3)
+    assert_allclose(result.total_stat, 21625.845714, rtol=1e-3)
 
-    pars = result.parameters
+    pars = models.parameters
     assert_allclose(pars["lon_0"].value, 0.2, rtol=1e-2)
     assert_allclose(pars["lon_0"].error, 0.002244, rtol=1e-2)
 
     assert_allclose(pars["index"].value, 3, rtol=1e-2)
-    assert_allclose(pars["index"].error, 0.024277, rtol=1e-2)
+    assert_allclose(pars["index"].error, 0.0242, rtol=1e-2)
 
     assert_allclose(pars["amplitude"].value, 1e-11, rtol=1e-2)
-    assert_allclose(pars["amplitude"].error, 4.216154e-13, rtol=1e-2)
+    assert_allclose(pars["amplitude"].error, 4.216e-13, rtol=1e-2)
 
     # background norm 1
     assert_allclose(pars[8].value, 0.5, rtol=1e-2)
@@ -505,12 +667,11 @@ def test_map_fit(sky_model, geom, geom_etrue):
     assert_allclose(pars[11].error, 0.02147, rtol=1e-2)
 
     # test mask_safe evaluation
-    mask_safe = geom.energy_mask(energy_min=1 * u.TeV)
-    dataset_1.mask_safe = Map.from_geom(geom, data=mask_safe)
-    dataset_2.mask_safe = Map.from_geom(geom, data=mask_safe)
+    dataset_1.mask_safe = geom.energy_mask(energy_min=1 * u.TeV)
+    dataset_2.mask_safe = geom.energy_mask(energy_min=1 * u.TeV)
 
-    stat = fit.datasets.stat_sum()
-    assert_allclose(stat, 14823.579908, rtol=1e-5)
+    stat = datasets.stat_sum()
+    assert_allclose(stat, 14823.772744, rtol=1e-5)
 
     region = sky_model.spatial_model.to_region()
 
@@ -524,10 +685,46 @@ def test_map_fit(sky_model, geom, geom_etrue):
     # test model evaluation outside image
     dataset_1.models[0].spatial_model.lon_0.value = 150
     dataset_1.npred()
-    assert not dataset_1._evaluators["test-model"].contributes
+    assert not dataset_1.evaluators["test-model"].contributes
+
+
+@requires_data()
+def test_map_fit_linked(sky_model, geom, geom_etrue):
+    dataset_1 = get_map_dataset(geom, geom_etrue, name="test-1")
+    dataset_2 = get_map_dataset(geom, geom_etrue, name="test-2")
+    datasets = Datasets([dataset_1, dataset_2])
+
+    models = Models(datasets.models)
+    models.insert(0, sky_model)
+    sky_model2 = sky_model.copy(name="test-model-2")
+    sky_model2.spectral_model.index = sky_model.spectral_model.index
+    sky_model2.spectral_model.reference = sky_model.spectral_model.reference
+
+    models.insert(0, sky_model2)
+
+    models["test-1-bkg"].spectral_model.norm.value = 0.5
+    models["test-model"].spatial_model.sigma.frozen = True
+
+    datasets.models = models
+    dataset_2.counts = dataset_2.npred()
+    dataset_1.counts = dataset_1.npred()
+
+    models["test-1-bkg"].spectral_model.norm.value = 0.49
+    models["test-2-bkg"].spectral_model.norm.value = 0.99
+
+    fit = Fit()
+    result = fit.run(datasets=datasets)
+
+    assert result.success
+    assert "minuit" in repr(result)
+
+    assert sky_model2.parameters["index"] is sky_model.parameters["index"]
+    assert sky_model2.parameters["reference"] is sky_model.parameters["reference"]
+
+    assert len(datasets.models.parameters.unique_parameters) == 20
+    assert datasets.models.covariance.shape == (22, 22)
 
 
-@requires_dependency("iminuit")
 @requires_data()
 def test_map_fit_one_energy_bin(sky_model, geom_image):
     energy_axis = geom_image.axes["energy"]
@@ -548,8 +745,8 @@ def test_map_fit_one_energy_bin(sky_model, geom_image):
     sky_model.parameters["sigma"].value = 0.21
     dataset.models[f"{dataset.name}-bkg"].parameters["norm"].frozen = True
 
-    fit = Fit([dataset])
-    result = fit.run()
+    fit = Fit()
+    result = fit.run(datasets=[dataset])
 
     assert result.success
 
@@ -557,7 +754,7 @@ def test_map_fit_one_energy_bin(sky_model, geom_image):
     assert_allclose(npred, 16538.124036, rtol=1e-3)
     assert_allclose(result.total_stat, -34844.125047, rtol=1e-3)
 
-    pars = result.parameters
+    pars = sky_model.parameters
 
     assert_allclose(pars["lon_0"].value, 0.2, rtol=1e-2)
     assert_allclose(pars["lon_0"].error, 0.001689, rtol=1e-2)
@@ -620,7 +817,7 @@ def test_create_with_migra(tmp_path):
     assert isinstance(empty_dataset.edisp, EDispMap)
     assert empty_dataset.edisp.edisp_map.data.shape == (3, 50, 10, 10)
     assert empty_dataset.edisp.exposure_map.data.shape == (3, 1, 10, 10)
-    assert_allclose(empty_dataset.edisp.edisp_map.data.sum(), 300)
+    assert_allclose(empty_dataset.edisp.edisp_map.data.sum(), 5000)
 
     assert_allclose(empty_dataset.gti.time_delta, 0.0 * u.s)
 
@@ -652,7 +849,7 @@ def test_stack(sky_model):
         energy_axis=axis, energy_axis_true=axis_etrue, geom=geom
     )
     edisp.exposure_map.quantity = (
-        1e0 * u.m ** 2 * u.s * np.ones(edisp.exposure_map.data.shape)
+        1e0 * u.m**2 * u.s * np.ones(edisp.exposure_map.data.shape)
     )
 
     bkg1 = Map.from_geom(geom)
@@ -662,7 +859,7 @@ def test_stack(sky_model):
     cnt1.data = 1.0 * np.ones(cnt1.data.shape)
 
     exp1 = Map.from_geom(geom_etrue)
-    exp1.quantity = 1e7 * u.m ** 2 * u.s * np.ones(exp1.data.shape)
+    exp1.quantity = 1e7 * u.m**2 * u.s * np.ones(exp1.data.shape)
 
     mask1 = Map.from_geom(geom)
     mask1.data = np.ones(mask1.data.shape, dtype=bool)
@@ -684,7 +881,7 @@ def test_stack(sky_model):
     cnt2.data = 1.0 * np.ones(cnt2.data.shape)
 
     exp2 = Map.from_geom(geom_etrue)
-    exp2.quantity = 1e7 * u.m ** 2 * u.s * np.ones(exp2.data.shape)
+    exp2.quantity = 1e7 * u.m**2 * u.s * np.ones(exp2.data.shape)
 
     mask2 = Map.from_geom(geom)
     mask2.data = np.ones(mask2.data.shape, dtype=bool)
@@ -706,36 +903,55 @@ def test_stack(sky_model):
 
     dataset1.models = [background_model1, sky_model]
     dataset2.models = [background_model2, sky_model]
-    dataset1.stack(dataset2)
 
-    dataset1.models = [sky_model]
-    npred_b = dataset1.npred()
+    stacked = MapDataset.from_geoms(**dataset1.geoms)
+    stacked.stack(dataset1)
+    stacked.stack(dataset2)
+
+    stacked.models = [sky_model]
+    npred_b = stacked.npred()
 
     assert_allclose(npred_b.data.sum(), 1459.985035, 1e-5)
-    assert_allclose(dataset1.npred_background().data.sum(), 1360.00, 1e-5)
-    assert_allclose(dataset1.counts.data.sum(), 9000, 1e-5)
-    assert_allclose(dataset1.mask_safe.data.sum(), 4600)
-    assert_allclose(dataset1.exposure.data.sum(), 1.6e11)
+    assert_allclose(stacked.npred_background().data.sum(), 1360.00, 1e-5)
+    assert_allclose(stacked.counts.data.sum(), 9000, 1e-5)
+    assert_allclose(stacked.mask_safe.data.sum(), 4600)
+    assert_allclose(stacked.exposure.data.sum(), 1.6e11)
 
-    assert_allclose(dataset1.meta_table["OBS_ID"][0], [0, 1])
+    assert_allclose(stacked.meta_table["OBS_ID"][0], [0, 1])
 
 
 @requires_data()
-def test_npred_sig(sky_model, geom, geom_etrue):
+def test_npred(sky_model, geom, geom_etrue):
     dataset = get_map_dataset(geom, geom_etrue)
 
     pwl = PowerLawSpectralModel()
     gauss = GaussianSpatialModel(
         lon_0="0.0 deg", lat_0="0.0 deg", sigma="0.5 deg", frame="galactic"
     )
-    model1 = SkyModel(pwl, gauss)
+    model1 = SkyModel(pwl, gauss, name="m1")
 
     bkg = FoVBackgroundModel(dataset_name=dataset.name)
     dataset.models = [bkg, sky_model, model1]
 
+    assert_allclose(
+        dataset.npred_signal(model_name=model1.name).data.sum(), 150.7487, rtol=1e-3
+    )
+    assert dataset._background_cached is None
+    assert_allclose(dataset.npred_background().data.sum(), 4000.0, rtol=1e-3)
+    assert_allclose(dataset._background_cached.data.sum(), 4000.0, rtol=1e-3)
+
     assert_allclose(dataset.npred().data.sum(), 9676.047906, rtol=1e-3)
     assert_allclose(dataset.npred_signal().data.sum(), 5676.04790, rtol=1e-3)
-    assert_allclose(dataset.npred_signal(model=model1).data.sum(), 150.7487, rtol=1e-3)
+
+    bkg.spectral_model.norm.value = 1.1
+    assert_allclose(dataset.npred_background().data.sum(), 4400.0, rtol=1e-3)
+    assert_allclose(dataset._background_cached.data.sum(), 4400.0, rtol=1e-3)
+
+    with pytest.raises(
+        KeyError,
+        match="m2",
+    ):
+        dataset.npred_signal(model_name="m2")
 
 
 def test_stack_npred():
@@ -749,7 +965,11 @@ def test_stack_npred():
     )
 
     geom = WcsGeom.create(
-        skydir=(0, 0), binsz=0.05, width=(2, 2), frame="icrs", axes=[axis],
+        skydir=(0, 0),
+        binsz=0.05,
+        width=(2, 2),
+        frame="icrs",
+        axes=[axis],
     )
 
     dataset_1 = MapDataset.create(
@@ -760,7 +980,7 @@ def test_stack_npred():
     )
     dataset_1.psf = None
     dataset_1.exposure.data += 1
-    dataset_1.mask_safe.data = geom.energy_mask(energy_min=1 * u.TeV)
+    dataset_1.mask_safe = geom.energy_mask(energy_min=1 * u.TeV)
     dataset_1.background.data += 1
 
     bkg_model_1 = FoVBackgroundModel(dataset_name=dataset_1.name)
@@ -774,7 +994,7 @@ def test_stack_npred():
     )
     dataset_2.psf = None
     dataset_2.exposure.data += 1
-    dataset_2.mask_safe.data = geom.energy_mask(energy_min=0.2 * u.TeV)
+    dataset_2.mask_safe = geom.energy_mask(energy_min=0.2 * u.TeV)
     dataset_2.background.data += 1
 
     bkg_model_2 = FoVBackgroundModel(dataset_name=dataset_2.name)
@@ -799,7 +1019,7 @@ def test_stack_npred():
 
 
 def to_cube(image):
-    # introduce a fake enery axis for now
+    # introduce a fake energy axis for now
     axis = MapAxis.from_edges([1, 10] * u.TeV, name="energy")
     geom = image.geom.to_cube([axis])
     return WcsNDMap.from_geom(geom=geom, data=image.data)
@@ -855,6 +1075,16 @@ def get_map_dataset_onoff(images, **kwargs):
     mask_data = np.ones(images["counts"].data.shape, dtype=bool)
     mask_safe = Map.from_geom(mask_geom, data=mask_data)
     gti = GTI.create([0 * u.s], [1 * u.h], reference_time="2010-01-01T00:00:00")
+    energy_axis = mask_geom.axes["energy"]
+    energy_axis_true = energy_axis.copy(name="energy_true")
+
+    psf = PSFMap.from_gauss(
+        energy_axis_true=energy_axis_true, sigma=0.2 * u.deg, geom=mask_geom.to_image()
+    )
+
+    edisp = EDispKernelMap.from_diagonal_response(
+        energy_axis=energy_axis, energy_axis_true=energy_axis_true, geom=mask_geom
+    )
 
     return MapDatasetOnOff(
         counts=images["counts"],
@@ -863,14 +1093,18 @@ def get_map_dataset_onoff(images, **kwargs):
         acceptance_off=images["acceptance_off"],
         exposure=images["exposure"],
         mask_safe=mask_safe,
+        psf=psf,
+        edisp=edisp,
         gti=gti,
         **kwargs,
     )
 
 
 @requires_data()
-def test_map_dataset_on_off_fits_io(images, tmp_path):
+@pytest.mark.parametrize("lazy", [False, True])
+def test_map_dataset_on_off_fits_io(images, lazy, tmp_path):
     dataset = get_map_dataset_onoff(images)
+    dataset.meta_table = Table(data=[[1.0 * u.h], [111]], names=["livetime", "obs_id"])
     gti = GTI.create([0 * u.s], [1 * u.h], reference_time="2010-01-01T00:00:00")
     dataset.gti = gti
 
@@ -883,9 +1117,18 @@ def test_map_dataset_on_off_fits_io(images, tmp_path):
         "COUNTS_BANDS",
         "EXPOSURE",
         "EXPOSURE_BANDS",
+        "EDISP",
+        "EDISP_BANDS",
+        "EDISP_EXPOSURE",
+        "EDISP_EXPOSURE_BANDS",
+        "PSF",
+        "PSF_BANDS",
+        "PSF_EXPOSURE",
+        "PSF_EXPOSURE_BANDS",
         "MASK_SAFE",
         "MASK_SAFE_BANDS",
         "GTI",
+        "META_TABLE",
         "COUNTS_OFF",
         "COUNTS_OFF_BANDS",
         "ACCEPTANCE",
@@ -898,9 +1141,48 @@ def test_map_dataset_on_off_fits_io(images, tmp_path):
 
     dataset.write(tmp_path / "test.fits")
 
-    dataset_new = MapDatasetOnOff.read(tmp_path / "test.fits")
-    assert dataset_new.mask.data.dtype == bool
+    if lazy:
+        with pytest.raises(NotImplementedError):
+            dataset_new = MapDatasetOnOff.read(tmp_path / "test.fits", lazy=lazy)
+    else:
+        dataset_new = MapDatasetOnOff.read(tmp_path / "test.fits", lazy=lazy)
+        assert dataset_new.mask.data.dtype == bool
+        assert dataset_new.meta_table["livetime"] == 1.0 * u.h
+        assert dataset_new.meta_table["obs_id"] == 111
 
+        assert_allclose(dataset.counts.data, dataset_new.counts.data)
+        assert_allclose(dataset.counts_off.data, dataset_new.counts_off.data)
+        assert_allclose(dataset.acceptance.data, dataset_new.acceptance.data)
+        assert_allclose(dataset.acceptance_off.data, dataset_new.acceptance_off.data)
+        assert_allclose(dataset.exposure.data, dataset_new.exposure.data)
+        assert_allclose(dataset.mask_safe, dataset_new.mask_safe)
+
+        assert np.all(dataset.mask_safe.data == dataset_new.mask_safe.data)
+        assert dataset.mask_safe.geom == dataset_new.mask_safe.geom
+        assert dataset.counts.geom == dataset_new.counts.geom
+        assert dataset.exposure.geom == dataset_new.exposure.geom
+
+        assert_allclose(
+            dataset.gti.time_sum.to_value("s"), dataset_new.gti.time_sum.to_value("s")
+        )
+
+        assert dataset.psf.psf_map == dataset_new.psf.psf_map
+        assert dataset.psf.exposure_map == dataset_new.psf.exposure_map
+        assert dataset.edisp.edisp_map == dataset_new.edisp.edisp_map
+        assert dataset.edisp.exposure_map == dataset_new.edisp.exposure_map
+
+
+@requires_data()
+def test_map_datasets_on_off_fits_io(images, tmp_path):
+    dataset = get_map_dataset_onoff(images)
+    Datasets([dataset]).write(tmp_path / "test.yaml")
+    datasets = Datasets.read(tmp_path / "test.yaml", lazy=False)
+    with pytest.raises(NotImplementedError):
+        datasets = Datasets.read(tmp_path / "test.yaml", lazy=True)
+
+    dataset_new = datasets[0]
+
+    assert dataset.name == dataset_new.name
     assert_allclose(dataset.counts.data, dataset_new.counts.data)
     assert_allclose(dataset.counts_off.data, dataset_new.counts_off.data)
     assert_allclose(dataset.acceptance.data, dataset_new.acceptance.data)
@@ -908,15 +1190,6 @@ def test_map_dataset_on_off_fits_io(images, tmp_path):
     assert_allclose(dataset.exposure.data, dataset_new.exposure.data)
     assert_allclose(dataset.mask_safe, dataset_new.mask_safe)
 
-    assert np.all(dataset.mask_safe.data == dataset_new.mask_safe.data) == True
-    assert dataset.mask_safe.geom == dataset_new.mask_safe.geom
-    assert dataset.counts.geom == dataset_new.counts.geom
-    assert dataset.exposure.geom == dataset_new.exposure.geom
-
-    assert_allclose(
-        dataset.gti.time_sum.to_value("s"), dataset_new.gti.time_sum.to_value("s")
-    )
-
 
 def test_create_onoff(geom):
     # tests empty datasets created
@@ -938,7 +1211,7 @@ def test_create_onoff(geom):
     assert empty_dataset.edisp.edisp_map.data.shape == (2, 50, 10, 10)
     assert empty_dataset.edisp.exposure_map.data.shape == (2, 1, 10, 10)
 
-    assert_allclose(empty_dataset.edisp.edisp_map.data.sum(), 200)
+    assert_allclose(empty_dataset.edisp.edisp_map.data.sum(), 3333.333333)
 
     assert_allclose(empty_dataset.gti.time_delta, 0.0 * u.s)
 
@@ -953,7 +1226,6 @@ def test_map_dataset_onoff_str(images):
 def test_stack_onoff(images):
     dataset = get_map_dataset_onoff(images)
     stacked = dataset.copy()
-
     stacked.stack(dataset)
 
     assert_allclose(stacked.counts.data.sum(), 2 * dataset.counts.data.sum())
@@ -961,12 +1233,7 @@ def test_stack_onoff(images):
     assert_allclose(
         stacked.acceptance.data.sum(), dataset.data_shape[1] * dataset.data_shape[2]
     )
-    assert_allclose(
-        np.nansum(stacked.acceptance_off.data),
-        np.nansum(
-            dataset.counts_off.data / (dataset.counts_off.data * dataset.alpha.data)
-        ),
-    )
+    assert_allclose(np.nansum(stacked.acceptance_off.data), 2.925793e08, rtol=1e-5)
     assert_allclose(stacked.exposure.data, 2.0 * dataset.exposure.data)
 
 
@@ -1122,10 +1389,9 @@ def test_map_dataset_on_off_fake(geom):
     rad_axis = MapAxis(nodes=np.linspace(0.0, 1.0, 51), unit="deg", name="rad")
     energy_true_axis = geom.axes["energy"].copy(name="energy_true")
 
-    empty_dataset = MapDataset.create(geom, energy_true_axis, rad_axis=rad_axis)
-    empty_dataset = MapDatasetOnOff.from_map_dataset(
-        empty_dataset, acceptance=1, acceptance_off=10.0
-    )
+    empty_dataset = MapDatasetOnOff.create(geom, energy_true_axis, rad_axis=rad_axis)
+    empty_dataset.acceptance.data = 1.0
+    empty_dataset.acceptance_off.data = 10.0
 
     empty_dataset.acceptance_off.data[0, 50, 50] = 0
     background_map = Map.from_geom(geom, data=1)
@@ -1214,16 +1480,13 @@ def test_names(geom, geom_etrue, sky_model):
     )
 
     dataset2 = dataset1.copy()
-    print(dataset2.models)
     assert dataset2.name != dataset1.name
-    assert dataset2.models[f"{dataset2.name}-bkg"]
+    assert dataset2.models is None
+
     dataset2 = dataset1.copy(name="dataset2")
 
     assert dataset2.name == "dataset2"
-    assert dataset2.models["dataset2-bkg"].name == "dataset2-bkg"
-    assert dataset2.models["dataset2-bkg"] is not dataset1.models["test-bkg"]
-    assert dataset2.models.names == ["model1", "model2", "dataset2-bkg"]
-    assert dataset2.models is not dataset1.models
+    assert dataset2.models is None
 
 
 def test_stack_dataset_dataset_on_off():
@@ -1256,7 +1519,7 @@ def test_info_dict_on_off(images):
     assert_allclose(info_dict["counts_off"], 20407510.0, rtol=1e-3)
     assert_allclose(info_dict["acceptance"], 4272.7075, rtol=1e-3)
     assert_allclose(info_dict["acceptance_off"], 20175596.0, rtol=1e-3)
-    assert_allclose(info_dict["alpha"], 0.000169, rtol=1e-3)
+    assert_allclose(info_dict["alpha"], 0.00021176, rtol=1e-3)
     assert_allclose(info_dict["ontime"].value, 3600)
 
 
@@ -1267,7 +1530,11 @@ def test_slice_by_idx():
     )
 
     geom = WcsGeom.create(
-        skydir=(0, 0), binsz=0.5, width=(2, 2), frame="icrs", axes=[axis],
+        skydir=(0, 0),
+        binsz=0.5,
+        width=(2, 2),
+        frame="icrs",
+        axes=[axis],
     )
     dataset = MapDataset.create(geom=geom, energy_axis_true=axis_etrue, binsz_irf=0.5)
 
@@ -1301,7 +1568,6 @@ def test_slice_by_idx():
     assert_allclose(axis.edges[0].value, 0.210175, rtol=1e-5)
 
 
-@requires_dependency("matplotlib")
 def test_plot_residual_onoff():
     axis = MapAxis.from_energy_bounds(1, 10, 2, unit="TeV")
     geom = WcsGeom.create(npix=(10, 10), binsz=0.05, axes=[axis])
@@ -1370,3 +1636,203 @@ def test_downsample_onoff():
     assert downsampled.counts.data.sum() == dataset_onoff.counts.data.sum()
     assert downsampled.counts_off.data.sum() == dataset_onoff.counts_off.data.sum()
     assert_allclose(downsampled.alpha.data, 0.5)
+
+
+@requires_data()
+def test_source_outside_geom(sky_model, geom, geom_etrue):
+    dataset = get_map_dataset(geom, geom_etrue)
+    dataset.edisp = get_edisp(geom, geom_etrue)
+
+    models = dataset.models
+    model = SkyModel(
+        PowerLawSpectralModel(),
+        DiskSpatialModel(lon_0=276.4 * u.deg, lat_0=-28.9 * u.deg, r_0=10 * u.deg),
+    )
+
+    assert not geom.to_image().contains(model.position)[0]
+    dataset.models = models + [model]
+    dataset.npred()
+    model_npred = dataset.evaluators[model.name].compute_npred().data
+    assert np.sum(np.isnan(model_npred)) == 0
+    assert np.sum(~np.isfinite(model_npred)) == 0
+    assert np.sum(model_npred) > 0
+
+
+# this is a regression test for an issue found, where the model selection fails
+@requires_data()
+def test_source_outside_geom_fermi():
+    dataset = MapDataset.read(
+        "$GAMMAPY_DATA/fermi-3fhl-gc/fermi-3fhl-gc.fits.gz", format="gadf"
+    )
+
+    catalog = SourceCatalog3FHL()
+    source = catalog["3FHL J1637.8-3448"]
+
+    dataset.models = source.sky_model()
+    npred = dataset.npred()
+
+    assert_allclose(npred.data.sum(), 28548.63, rtol=1e-4)
+
+
+def test_region_geom_io(tmpdir):
+    axis = MapAxis.from_energy_bounds("1 TeV", "10 TeV", nbin=1)
+    geom = RegionGeom.create("icrs;circle(0, 0, 0.2)", axes=[axis])
+
+    dataset = MapDataset.create(geom)
+
+    filename = tmpdir / "test.fits"
+    dataset.write(filename)
+
+    dataset = MapDataset.read(filename, format="gadf")
+
+    assert isinstance(dataset.counts.geom, RegionGeom)
+    assert isinstance(dataset.edisp.edisp_map.geom, RegionGeom)
+    assert isinstance(dataset.psf.psf_map.geom, RegionGeom)
+
+
+def test_dataset_mixed_geom(tmpdir):
+    energy_axis = MapAxis.from_energy_bounds("1 TeV", "10 TeV", nbin=3)
+    energy_axis_true = MapAxis.from_energy_bounds(
+        "1 TeV", "10 TeV", nbin=7, name="energy_true"
+    )
+
+    rad_axis = MapAxis.from_bounds(0, 1, nbin=10, name="rad", unit="deg")
+
+    geom = WcsGeom.create(npix=5, axes=[energy_axis])
+    geom_exposure = WcsGeom.create(npix=5, axes=[energy_axis_true])
+
+    geom_psf = RegionGeom.create(
+        "icrs;circle(0, 0, 0.2)", axes=[rad_axis, energy_axis_true]
+    )
+
+    geom_edisp = RegionGeom.create(
+        "icrs;circle(0, 0, 0.2)", axes=[energy_axis, energy_axis_true]
+    )
+
+    dataset = MapDataset.from_geoms(
+        geom=geom, geom_exposure=geom_exposure, geom_psf=geom_psf, geom_edisp=geom_edisp
+    )
+
+    filename = tmpdir / "test.fits"
+    dataset.write(filename)
+
+    dataset = MapDataset.read(filename, format="gadf")
+
+    assert isinstance(dataset.counts.geom, WcsGeom)
+    assert isinstance(dataset.exposure.geom, WcsGeom)
+    assert isinstance(dataset.background.geom, WcsGeom)
+
+    assert isinstance(dataset.psf.psf_map.geom.region, CircleSkyRegion)
+    assert isinstance(dataset.edisp.edisp_map.geom.region, CircleSkyRegion)
+
+
+@requires_data()
+def test_map_dataset_region_geom_npred():
+    dataset = MapDataset.read("$GAMMAPY_DATA/cta-1dc-gc/cta-1dc-gc.fits.gz")
+
+    pwl = PowerLawSpectralModel()
+    point = PointSpatialModel(lon_0="0 deg", lat_0="0 deg", frame="galactic")
+    model_1 = SkyModel(pwl, point, name="model-1")
+
+    pwl = PowerLawSpectralModel(amplitude="1e-11 TeV-1 cm-2 s-1")
+    gauss = GaussianSpatialModel(
+        lon_0="0.3 deg", lat_0="0.3 deg", sigma="0.5 deg", frame="galactic"
+    )
+    model_2 = SkyModel(pwl, gauss, name="model-2")
+
+    dataset.models = [model_1, model_2]
+
+    region = RegionGeom.create("galactic;circle(0, 0, 0.4)").region
+    npred_ref = dataset.npred().to_region_nd_map(region)
+
+    dataset_spec = dataset.to_region_map_dataset(region)
+    dataset_spec.models = [model_1, model_2]
+
+    npred = dataset_spec.npred()
+
+    assert_allclose(npred_ref.data, npred.data, rtol=1e-2)
+
+
+@requires_dependency("healpy")
+def test_map_dataset_create_hpx_geom(geom_hpx):
+
+    dataset = MapDataset.create(**geom_hpx, binsz_irf=10 * u.deg)
+
+    assert isinstance(dataset.counts.geom, HpxGeom)
+    assert dataset.counts.data.shape == (3, 12288)
+
+    assert isinstance(dataset.background.geom, HpxGeom)
+    assert dataset.background.data.shape == (3, 12288)
+
+    assert isinstance(dataset.exposure.geom, HpxGeom)
+    assert dataset.exposure.data.shape == (4, 12288)
+
+    assert isinstance(dataset.edisp.edisp_map.geom, HpxGeom)
+    assert dataset.edisp.edisp_map.data.shape == (4, 3, 768)
+
+    assert isinstance(dataset.psf.psf_map.geom, HpxGeom)
+    assert dataset.psf.psf_map.data.shape == (4, 66, 768)
+
+
+@requires_dependency("healpy")
+def test_map_dataset_create_hpx_geom_partial(geom_hpx_partial):
+
+    dataset = MapDataset.create(**geom_hpx_partial, binsz_irf=2 * u.deg)
+
+    assert isinstance(dataset.counts.geom, HpxGeom)
+    assert dataset.counts.data.shape == (3, 90)
+
+    assert isinstance(dataset.background.geom, HpxGeom)
+    assert dataset.background.data.shape == (3, 90)
+
+    assert isinstance(dataset.exposure.geom, HpxGeom)
+    assert dataset.exposure.data.shape == (4, 90)
+
+    assert isinstance(dataset.edisp.edisp_map.geom, HpxGeom)
+    assert dataset.edisp.edisp_map.data.shape == (4, 3, 24)
+
+    assert isinstance(dataset.psf.psf_map.geom, HpxGeom)
+    assert dataset.psf.psf_map.data.shape == (4, 66, 24)
+
+
+@requires_dependency("healpy")
+def test_map_dataset_stack_hpx_geom(geom_hpx_partial, geom_hpx):
+
+    dataset_all = MapDataset.create(**geom_hpx, binsz_irf=5 * u.deg)
+
+    gti = GTI.create(start=0 * u.s, stop=30 * u.min)
+    dataset_cutout = MapDataset.create(**geom_hpx_partial, binsz_irf=5 * u.deg, gti=gti)
+    dataset_cutout.counts.data += 1
+    dataset_cutout.background.data += 1
+    dataset_cutout.exposure.data += 1
+    dataset_cutout.mask_safe.data[...] = True
+
+    dataset_all.stack(dataset_cutout)
+
+    assert_allclose(dataset_all.counts.data.sum(), 3 * 90)
+    assert_allclose(dataset_all.background.data.sum(), 3 * 90)
+    assert_allclose(dataset_all.exposure.data.sum(), 4 * 90)
+
+
+@requires_data()
+@requires_dependency("healpy")
+def test_map_dataset_hpx_geom_npred(geom_hpx_partial):
+    hpx_geom = geom_hpx_partial["geom"]
+    hpx_true = hpx_geom.to_image().to_cube([geom_hpx_partial["energy_axis_true"]])
+    dataset = get_map_dataset(hpx_geom, hpx_true, edisp="edispkernelmap")
+
+    pwl = PowerLawSpectralModel()
+    point = PointSpatialModel(lon_0="110 deg", lat_0="75 deg", frame="galactic")
+    sky_model = SkyModel(pwl, point)
+
+    dataset.models = [sky_model]
+
+    assert_allclose(dataset.npred().data.sum(), 54, rtol=1e-3)
+
+
+@requires_data()
+def test_peek(images):
+    dataset = get_map_dataset_onoff(images)
+
+    with mpl_plot_check():
+        dataset.peek()
