@@ -113,10 +113,14 @@ class FluxEstimator(ParameterEstimator):
         ref_model = models[self.source].spectral_model
         scale_model = ScaleSpectralModel(ref_model)
 
-        if hasattr(ref_model, "amplitude"):
-            scaled_parameter = ref_model.amplitude
+        for scaled_parameter in ref_model.parameters:
+            if scaled_parameter.is_norm:
+                break
         else:
-            scaled_parameter = ref_model.norm
+            raise ValueError(
+                f"{self.tag} requires a 'norm' or 'amplitude' parameter"
+                " in the model to run"
+            )
 
         scale_model.norm = self._set_norm_parameter(scale_model.norm, scaled_parameter)
         return scale_model
@@ -140,7 +144,7 @@ class FluxEstimator(ParameterEstimator):
         for dataset in datasets:
             name = datasets.models[self.source].name
             npred_signal = dataset.npred_signal(model_name=name)
-            npred = Map.from_geom(dataset.mask.geom)
+            npred = Map.from_geom(dataset.counts.geom)
             npred.stack(npred_signal)
             npred_excess.append(npred.data[dataset.mask].sum())
 
