@@ -1,14 +1,17 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 import logging
+from copy import deepcopy
 import numpy as np
 from scipy import stats
 from astropy.io.registry import IORegistryError
 from astropy.table import Table, vstack
 from astropy.visualization import quantity_support
+import matplotlib.pyplot as plt
 from gammapy.maps import MapAxis, Maps, RegionNDMap, TimeMapAxis
 from gammapy.maps.axes import flat_if_equal
 from gammapy.modeling.models import TemplateSpectralModel
 from gammapy.modeling.models.spectral import scale_plot_flux
+from gammapy.modeling.scipy import stat_profile_ul_scipy
 from gammapy.utils.scripts import make_path
 from gammapy.utils.table import table_standardise_units_copy
 from ..map.core import DEFAULT_UNIT, FluxMaps
@@ -45,58 +48,56 @@ class FluxPoints(FluxMaps):
     The `FluxPoints` object is most easily created by reading a file with
     flux points given in one of the formats documented above::
 
-        from gammapy.estimators import FluxPoints
-        filename = '$GAMMAPY_DATA/hawc_crab/HAWC19_flux_points.fits'
-        flux_points = FluxPoints.read(filename)
-        flux_points.plot()
+    >>> from gammapy.estimators import FluxPoints
+    >>> filename = '$GAMMAPY_DATA/hawc_crab/HAWC19_flux_points.fits'
+    >>> flux_points = FluxPoints.read(filename)
+    >>> flux_points.plot() #doctest: +SKIP
 
     An instance of `FluxPoints` can also be created by passing an instance of
     `astropy.table.Table`, which contains the required columns, such as `'e_ref'`
     and `'dnde'`. The corresponding `sed_type` has to be defined in the meta data
     of the table::
 
-        import numpy as np
-        from astropy import units as u
-        from astropy.table import Table
-        from gammapy.estimators import FluxPoints
-        from gammapy.modeling.models import PowerLawSpectralModel
-
-        table = Table()
-        pwl = PowerLawSpectralModel()
-        e_ref = np.geomspace(1, 100, 7) * u.TeV
-
-        table["e_ref"] = e_ref
-        table["dnde"] = pwl(e_ref)
-        table.meta["SED_TYPE"] = "dnde"
-
-        flux_points = FluxPoints.from_table(table)
-        flux_points.plot(sed_type="flux")
+    >>> import numpy as np
+    >>> from astropy import units as u
+    >>> from astropy.table import Table
+    >>> from gammapy.estimators import FluxPoints
+    >>> from gammapy.modeling.models import PowerLawSpectralModel
+    >>> table = Table()
+    >>> pwl = PowerLawSpectralModel()
+    >>> e_ref = np.geomspace(1, 100, 7) * u.TeV
+    >>> table["e_ref"] = e_ref
+    >>> table["dnde"] = pwl(e_ref)
+    >>> table["dnde_err"] = pwl.evaluate_error(e_ref)[0]
+    >>> table.meta["SED_TYPE"] = "dnde"
+    >>> flux_points = FluxPoints.from_table(table)
+    >>> flux_points.plot(sed_type="flux") #doctest: +SKIP
 
     If you have flux points in a different data format, the format can be changed
     by renaming the table columns and adding meta data::
 
 
-        from astropy import units as u
-        from astropy.table import Table
-        from gammapy.estimators import FluxPoints
-        from gammapy.utils.scripts import make_path
+    >>> from astropy import units as u
+    >>> from astropy.table import Table
+    >>> from gammapy.estimators import FluxPoints
+    >>> from gammapy.utils.scripts import make_path
 
-        table = Table.read(make_path('$GAMMAPY_DATA/tests/spectrum/flux_points/flux_points_ctb_37b.txt'),
-                           format='ascii.csv', delimiter=' ', comment='#')
-        table.rename_column('Differential_Flux', 'dnde')
-        table['dnde'].unit = 'cm-2 s-1 TeV-1'
+    >>> filename = make_path('$GAMMAPY_DATA/tests/spectrum/flux_points/flux_points_ctb_37b.txt')
+    >>> table = Table.read(filename ,format='ascii.csv', delimiter=' ', comment='#')
+    >>> table.rename_column('Differential_Flux', 'dnde')
+    >>> table['dnde'].unit = 'cm-2 s-1 TeV-1'
 
-        table.rename_column('lower_error', 'dnde_errn')
-        table['dnde_errn'].unit = 'cm-2 s-1 TeV-1'
+    >>> table.rename_column('lower_error', 'dnde_errn')
+    >>> table['dnde_errn'].unit = 'cm-2 s-1 TeV-1'
 
-        table.rename_column('upper_error', 'dnde_errp')
-        table['dnde_errp'].unit = 'cm-2 s-1 TeV-1'
+    >>> table.rename_column('upper_error', 'dnde_errp')
+    >>> table['dnde_errp'].unit = 'cm-2 s-1 TeV-1'
 
-        table.rename_column('E', 'e_ref')
-        table['e_ref'].unit = 'TeV'
+    >>> table.rename_column('E', 'e_ref')
+    >>> table['e_ref'].unit = 'TeV'
 
-        flux_points = FluxPoints.from_table(table, sed_type="dnde")
-        flux_points.plot(sed_type="e2dnde")
+    >>> flux_points = FluxPoints.from_table(table, sed_type="dnde")
+    >>> flux_points.plot(sed_type="e2dnde") #doctest: +SKIP
 
 
     Note: In order to reproduce the example you need the tests datasets folder.
@@ -108,7 +109,7 @@ class FluxPoints(FluxMaps):
     def read(
         cls, filename, sed_type=None, format="gadf-sed", reference_model=None, **kwargs
     ):
-        """Read flux points.
+        """Read precomputed flux points.
 
         Parameters
         ----------
@@ -143,7 +144,7 @@ class FluxPoints(FluxMaps):
             format=format,
         )
 
-    def write(self, filename, sed_type="likelihood", format="gadf-sed", **kwargs):
+    def write(self, filename, sed_type=None, format="gadf-sed", **kwargs):
         """Write flux points.
 
         Parameters
@@ -152,11 +153,25 @@ class FluxPoints(FluxMaps):
             Filename
         sed_type : {"dnde", "flux", "eflux", "e2dnde", "likelihood"}
             Sed type
-        format : {"gadf-sed", "lightcurve"}
-            Format string.
+        format : {"gadf-sed", "lightcurve", "binned-time-series", "profile"}
+            Format specification. The following formats are supported:
+
+            * "gadf-sed": format for sed flux points see :ref:`gadf:flux-points`
+                for details
+            * "lightcurve": Gammapy internal format to store energy dependent
+                lightcurves. Basically a generalisation of the "gadf" format, but
+                currently there is no detailed documentation available.
+            * "binned-time-series": table format support by Astropy's
+                `~astropy.timeseries.BinnedTimeSeries`.
+            * "profile": Gammapy internal format to store energy dependent
+                flux profiles. Basically a generalisation of the "gadf" format, but
+                currently there is no detailed documentation available.
         **kwargs : dict
             Keyword arguments passed to `astropy.table.Table.write`.
         """
+        if sed_type is None:
+            sed_type = self.sed_type_init
+
         filename = make_path(filename)
         table = self.to_table(sed_type=sed_type, format=format)
         table.write(filename, **kwargs)
@@ -184,7 +199,8 @@ class FluxPoints(FluxMaps):
     def from_table(
         cls, table, sed_type=None, format="gadf-sed", reference_model=None, gti=None
     ):
-        """Create flux points from table
+        """Create flux points from a table. The table column names must be consistent with the
+        sed_type
 
         Parameters
         ----------
@@ -267,7 +283,7 @@ class FluxPoints(FluxMaps):
 
         return table
 
-    def to_table(self, sed_type="likelihood", format="gadf-sed", formatted=False):
+    def to_table(self, sed_type=None, format="gadf-sed", formatted=False):
         """Create table for a given SED type.
 
         Parameters
@@ -292,26 +308,29 @@ class FluxPoints(FluxMaps):
             Formatted version with column formats applied. Numerical columns are
             formatted to .3f and .3e respectively.
 
+        Returns
+        -------
+        table : `~astropy.table.Table`
+            Flux points table
+
         Examples
         --------
 
         This is how to read and plot example flux points:
 
-            >>> from gammapy.estimators import FluxPoints
-            >>> fp = FluxPoints.read("$GAMMAPY_DATA/hawc_crab/HAWC19_flux_points.fits")
-            >>> table = fp.to_table(sed_type="flux", format="gadf-sed", formatted=True)
-            >>> print(table[:2])
-            e_ref e_min e_max     flux      flux_err    flux_ul      ts    sqrt_ts is_ul
-             TeV   TeV   TeV  1 / (cm2 s) 1 / (cm2 s) 1 / (cm2 s)
-            ----- ----- ----- ----------- ----------- ----------- -------- ------- -----
-            1.334 1.000 1.780   1.423e-11   3.135e-13         nan 2734.000  52.288 False
-            2.372 1.780 3.160   5.780e-12   1.082e-13         nan 4112.000  64.125 False
-
-        Returns
-        -------
-        table : `~astropy.table.Table`
-            Flux points table
+        >>> from gammapy.estimators import FluxPoints
+        >>> fp = FluxPoints.read("$GAMMAPY_DATA/hawc_crab/HAWC19_flux_points.fits")
+        >>> table = fp.to_table(sed_type="flux", format="gadf-sed", formatted=True)
+        >>> print(table[:2])
+        e_ref e_min e_max     flux      flux_err    flux_ul      ts    sqrt_ts is_ul
+         TeV   TeV   TeV  1 / (cm2 s) 1 / (cm2 s) 1 / (cm2 s)
+        ----- ----- ----- ----------- ----------- ----------- -------- ------- -----
+        1.334 1.000 1.780   1.423e-11   3.135e-13         nan 2734.000  52.288 False
+        2.372 1.780 3.160   5.780e-12   1.082e-13         nan 4112.000  64.125 False
         """
+        if sed_type is None:
+            sed_type = self.sed_type_init
+
         if format == "gadf-sed":
             # TODO: what to do with GTI info?
             if not self.geom.axes.names == ["energy"]:
@@ -324,6 +343,9 @@ class FluxPoints(FluxMaps):
             table = self.energy_axis.to_table(format="gadf-sed")
             table.meta["SED_TYPE"] = sed_type
 
+            if not self.is_convertible_to_flux_sed_type:
+                table.remove_columns(["e_min", "e_max"])
+
             if self.n_sigma_ul:
                 table.meta["UL_CONF"] = np.round(
                     1 - 2 * stats.norm.sf(self.n_sigma_ul), 7
@@ -441,7 +463,7 @@ class FluxPoints(FluxMaps):
 
         return y_errn, y_errp
 
-    def plot(self, ax=None, sed_type="dnde", energy_power=0, **kwargs):
+    def plot(self, ax=None, sed_type=None, energy_power=0, **kwargs):
         """Plot flux points.
 
         Parameters
@@ -460,7 +482,8 @@ class FluxPoints(FluxMaps):
         ax : `~matplotlib.axes.Axes`
             Axis object
         """
-        import matplotlib.pyplot as plt
+        if sed_type is None:
+            sed_type = self.sed_type_plot_default
 
         if not self.norm.geom.is_region:
             raise ValueError("Plotting only supported for region based flux points")
@@ -476,6 +499,7 @@ class FluxPoints(FluxMaps):
         y_errn, y_errp = self._plot_get_flux_err(sed_type=sed_type)
 
         is_ul = self.is_ul.data
+
         if self.has_ul and y_errn and is_ul.any():
             flux_ul = getattr(self, sed_type + "_ul").quantity
             y_errn.data[is_ul] = 0.5 * flux_ul[is_ul].to_value(y_errn.unit)
@@ -484,13 +508,12 @@ class FluxPoints(FluxMaps):
             kwargs.setdefault("uplims", is_ul)
 
         # set flux points plotting defaults
-        if y_errp:
+        if y_errp and y_errn:
             y_errp = scale_plot_flux(y_errp, energy_power=energy_power).quantity
-
-        if y_errn:
             y_errn = scale_plot_flux(y_errn, energy_power=energy_power).quantity
-
-        kwargs.setdefault("yerr", (y_errn, y_errp))
+            kwargs.setdefault("yerr", (y_errn, y_errp))
+        else:
+            kwargs.setdefault("yerr", None)
 
         flux = scale_plot_flux(flux=flux.to_unit(flux_unit), energy_power=energy_power)
         ax = flux.plot(ax=ax, **kwargs)
@@ -501,7 +524,7 @@ class FluxPoints(FluxMaps):
     def plot_ts_profiles(
         self,
         ax=None,
-        sed_type="dnde",
+        sed_type=None,
         add_cbar=True,
         **kwargs,
     ):
@@ -523,11 +546,12 @@ class FluxPoints(FluxMaps):
         ax : `~matplotlib.axes.Axes`
             Axis object
         """
-        import matplotlib.pyplot as plt
-
         if ax is None:
             ax = plt.gca()
 
+        if sed_type is None:
+            sed_type = self.sed_type_plot_default
+
         if not self.norm.geom.is_region:
             raise ValueError("Plotting only supported for region based flux points")
 
@@ -541,7 +565,12 @@ class FluxPoints(FluxMaps):
         if isinstance(axis, TimeMapAxis) and not axis.is_contiguous:
             axis = axis.to_contiguous()
 
-        yunits = kwargs.pop("yunits", DEFAULT_UNIT[sed_type])
+        if ax.yaxis.units is None:
+            yunits = DEFAULT_UNIT[sed_type]
+        else:
+            yunits = ax.yaxis.units
+
+        ax.yaxis.set_units(yunits)
 
         flux_ref = getattr(self, sed_type + "_ref").to(yunits)
 
@@ -588,3 +617,52 @@ class FluxPoints(FluxMaps):
             ax.figure.colorbar(caxes, ax=ax, label=label)
 
         return ax
+
+    def recompute_ul(self, n_sigma_ul=2, **kwargs):
+        """Recompute upper limits corresponding to the given value.
+        The pre-computed stat profiles must exist for the re-computation.
+
+        Parameters
+        ----------
+        n_sigma_ul : int
+            Number of sigma to use for upper limit computation. Default is 2.
+        **kwargs : dict
+            Keyword arguments passed to `~scipy.optimize.brentq`.
+
+        Returns
+        -------
+        flux_points : `FluxPoints`
+            A new FluxPoints object with modified upper limits
+
+        Examples
+        --------
+        >>> from gammapy.estimators import FluxPoints
+        >>> filename = '$GAMMAPY_DATA/tests/spectrum/flux_points/binlike.fits'
+        >>> flux_points = FluxPoints.read(filename)
+        >>> flux_points_recomputed = flux_points.recompute_ul(n_sigma_ul=3)
+        >>> print(flux_points.meta["n_sigma_ul"], flux_points.flux_ul.data[0])
+        2.0 [[3.95451985e-09]]
+        >>> print(flux_points_recomputed.meta["n_sigma_ul"], flux_points_recomputed.flux_ul.data[0])
+        3 [[6.22245374e-09]]
+        """
+
+        if not self.has_stat_profiles:
+            raise ValueError(
+                "Stat profiles not present. Upper limit computation is not possible"
+            )
+
+        delta_ts = n_sigma_ul**2
+
+        flux_points = deepcopy(self)
+
+        value_scan = self.stat_scan.geom.axes["norm"].center
+        shape_axes = self.stat_scan.geom._shape[slice(3, None)]
+        for idx in np.ndindex(shape_axes):
+            stat_scan = np.abs(
+                self.stat_scan.data[idx].squeeze() - self.stat.data[idx].squeeze()
+            )
+            flux_points.norm_ul.data[idx] = stat_profile_ul_scipy(
+                value_scan, stat_scan, delta_ts=delta_ts, **kwargs
+            )
+        flux_points.meta["n_sigma_ul"] = n_sigma_ul
+        return flux_points
