@@ -1,6 +1,7 @@
 import abc
 import logging
 from copy import deepcopy
+from enum import Enum
 import numpy as np
 from astropy import units as u
 from astropy.io import fits
@@ -13,11 +14,24 @@ from gammapy.utils.interpolation import (
     interpolation_scale,
 )
 from gammapy.utils.scripts import make_path
-from .io import IRF_DL3_HDU_SPECIFICATION, IRF_MAP_HDU_SPECIFICATION
+from .io import IRF_DL3_HDU_SPECIFICATION, IRF_MAP_HDU_SPECIFICATION, gadf_is_pointlike
 
 log = logging.getLogger(__name__)
 
 
+class FoVAlignment(str, Enum):
+    """
+    Orientation of the Field of View Coordinate System
+
+    Currently, only two possible alignments are supported: alignment with
+    the horizontal coordinate system (ALTAZ) and alignment with the equatorial
+    coordinate system (RADEC).
+    """
+
+    ALTAZ = "ALTAZ"
+    RADEC = "RADEC"
+
+
 class IRF(metaclass=abc.ABCMeta):
     """IRF base class for DL3 instrument response functions
 
@@ -29,6 +43,10 @@ class IRF(metaclass=abc.ABCMeta):
         Data
     unit : str or `~astropy.units.Unit`
         Unit, ignored if data is a Quantity.
+    is_pointlike: boolean
+        True for point-like IRFs, False for full-enclosure.
+    fov_alignment: `FoVAlignment`
+        The orientation of the field of view coordinate system.
     meta : dict
         Meta data
     """
@@ -38,16 +56,33 @@ class IRF(metaclass=abc.ABCMeta):
         fill_value=0.0,
     )
 
-    def __init__(self, axes, data=0, unit="", meta=None, interp_kwargs=None):
+    def __init__(
+        self,
+        axes,
+        data=0,
+        unit="",
+        is_pointlike=False,
+        fov_alignment=FoVAlignment.RADEC,
+        meta=None,
+        interp_kwargs=None,
+    ):
         axes = MapAxes(axes)
         axes.assert_names(self.required_axes)
         self._axes = axes
+        self._fov_alignment = FoVAlignment(fov_alignment)
+        self._is_pointlike = is_pointlike
+
         if isinstance(data, u.Quantity):
             self.data = data.value
-            self.unit = data.unit
+            if not self.default_unit.is_equivalent(data.unit):
+                raise ValueError(
+                    f"Error: {data.unit} is not an allowed unit. {self.tag} requires {self.default_unit} data quantities."
+                )
+            else:
+                self._unit = data.unit
         else:
             self.data = data
-            self.unit = unit
+            self._unit = unit
         self.meta = meta or {}
         if interp_kwargs is None:
             interp_kwargs = self.default_interp_kwargs.copy()
@@ -66,13 +101,18 @@ class IRF(metaclass=abc.ABCMeta):
     @property
     def is_pointlike(self):
         """Whether the IRF is pointlike of full containment."""
-        return self.meta.get("is_pointlike", False)
+        return self._is_pointlike
 
     @property
-    def is_offset_dependent(self):
-        """Whether the IRF depends on offset"""
+    def has_offset_axis(self):
+        """Whether the IRF explicitly depends on offset"""
         return "offset" in self.required_axes
 
+    @property
+    def fov_alignment(self):
+        """Alignment of the field of view coordinate axes, see `FoVAlignment`"""
+        return self._fov_alignment
+
     @property
     def data(self):
         return self._data
@@ -141,10 +181,6 @@ class IRF(metaclass=abc.ABCMeta):
         """Map unit (`~astropy.units.Unit`)"""
         return self._unit
 
-    @unit.setter
-    def unit(self, val):
-        self._unit = u.Unit(val)
-
     @lazyproperty
     def _interpolate(self):
         kwargs = self.interp_kwargs.copy()
@@ -175,7 +211,25 @@ class IRF(metaclass=abc.ABCMeta):
         """
         val = u.Quantity(val, copy=False)
         self.data = val.value
-        self.unit = val.unit
+        self._unit = val.unit
+
+    def to_unit(self, unit):
+        """Convert irf to different unit
+
+        Parameters
+        ----------
+        unit : `~astropy.unit.Unit` or str
+            New unit
+
+        Returns
+        -------
+        irf : `IRF`
+            IRF with new unit and converted data
+        """
+        data = self.quantity.to_value(unit)
+        return self.__class__(
+            self.axes, data=data, meta=self.meta, interp_kwargs=self.interp_kwargs
+        )
 
     @property
     def axes(self):
@@ -209,6 +263,7 @@ class IRF(metaclass=abc.ABCMeta):
         """
         # TODO: change to coord dict?
         non_valid_axis = set(kwargs).difference(self.axes.names)
+
         if non_valid_axis:
             raise ValueError(
                 f"Not a valid coordinate axis {non_valid_axis}"
@@ -221,6 +276,7 @@ class IRF(metaclass=abc.ABCMeta):
             coord = kwargs.get(key, value)
             if coord is not None:
                 coords_default[key] = u.Quantity(coord, copy=False)
+
         data = self._interpolate(coords_default.values(), method=method)
 
         if self.interp_kwargs["fill_value"] is not None:
@@ -396,9 +452,14 @@ class IRF(metaclass=abc.ABCMeta):
         column_name = IRF_DL3_HDU_SPECIFICATION[cls.tag]["column_name"]
         data = table[column_name].quantity[0].transpose()
 
-        if "HDUCLAS3" in table.meta and table.meta["HDUCLAS3"] == "POINT-LIKE":
-            table.meta["is_pointlike"] = True
-        return cls(axes=axes, data=data.value, meta=table.meta, unit=data.unit)
+        return cls(
+            axes=axes,
+            data=data.value,
+            meta=table.meta,
+            unit=data.unit,
+            is_pointlike=gadf_is_pointlike(table.meta),
+            fov_alignment=table.meta.get("FOVALIGN", "RADEC"),
+        )
 
     def to_table(self, format="gadf-dl3"):
         """Convert to table
@@ -418,10 +479,14 @@ class IRF(metaclass=abc.ABCMeta):
         if format == "gadf-dl3":
             table.meta = self.meta.copy()
             spec = IRF_DL3_HDU_SPECIFICATION[self.tag]
-            # TODO: add missing required meta data!
-            table.meta["HDUCLAS2"] = spec["hduclas2"]
+
+            table.meta.update(spec["mandatory_keywords"])
+
             if self.is_pointlike:
                 table.meta["HDUCLAS3"] = "POINT-LIKE"
+            else:
+                table.meta["HDUCLAS3"] = "FULL-ENCLOSURE"
+
             table[spec["column_name"]] = self.quantity.T[np.newaxis]
         else:
             raise ValueError(f"Not a valid supported format: '{format}'")
@@ -489,6 +554,41 @@ class IRF(metaclass=abc.ABCMeta):
             data=data, axes=axes, meta=self.meta.copy(), unit=self.unit
         )
 
+    def is_allclose(self, other, rtol_axes=1e-3, atol_axes=1e-6, **kwargs):
+        """Compare two data IRFs for equivalency
+
+        Parameters
+        ----------
+        other : `gammapy.irfs.IRF`
+            The irf to compare against
+        rtol_axes : float
+            Relative tolerance for the axes comparison.
+        atol_axes : float
+            Relative tolerance for the axes comparison.
+        **kwargs : dict
+                keywords passed to `numpy.allclose`
+
+        Returns
+        -------
+        is_allclose : bool
+            Whether the IRF is all close.
+        """
+        if not isinstance(other, self.__class__):
+            return TypeError(f"Cannot compare {type(self)} and {type(other)}")
+
+        if self.data.shape != other.data.shape:
+            return False
+
+        axes_eq = self.axes.is_allclose(other.axes, rtol=rtol_axes, atol=atol_axes)
+        data_eq = np.allclose(self.quantity, other.quantity, **kwargs)
+        return axes_eq and data_eq
+
+    def __eq__(self, other):
+        if not isinstance(other, self.__class__):
+            return False
+
+        return self.is_allclose(other=other, rtol=1e-3, rtol_axes=1e-6)
+
 
 class IRFMap:
     """IRF map base class for DL4 instrument response functions"""
@@ -523,7 +623,7 @@ class IRFMap:
 
         Parameters
         ----------
-        region : `SkyRegion` or `SkyCoord`
+        region : `~regions.SkyRegion` or `~astropy.coordinates.SkyCoord`
             Region or position where to get the map.
 
         Returns
@@ -553,7 +653,7 @@ class IRFMap:
         """Get nearest valid position"""
         is_valid = np.nan_to_num(self.mask_safe_image.get_by_coord(position))[0]
 
-        if not is_valid:
+        if not is_valid and np.any(self.mask_safe_image > 0):
             log.warning(
                 f"Position {position} is outside "
                 "valid IRF map range, using nearest IRF defined within"
