@@ -2,6 +2,7 @@
 import numpy as np
 import astropy.units as u
 from astropy.visualization import quantity_support
+import matplotlib.pyplot as plt
 from gammapy.maps import MapAxes, MapAxis
 from .core import IRF
 
@@ -64,6 +65,7 @@ class EffectiveAreaTable2D(IRF):
 
     tag = "aeff_2d"
     required_axes = ["energy_true", "offset"]
+    default_unit = u.m**2
 
     def plot_energy_dependence(self, ax=None, offset=None, **kwargs):
         """Plot effective area versus energy for a given offset.
@@ -82,8 +84,6 @@ class EffectiveAreaTable2D(IRF):
         ax : `~matplotlib.axes.Axes`
             Axis
         """
-        import matplotlib.pyplot as plt
-
         ax = plt.gca() if ax is None else ax
 
         if offset is None:
@@ -120,8 +120,6 @@ class EffectiveAreaTable2D(IRF):
         ax : `~matplotlib.axes.Axes`
             Axis
         """
-        import matplotlib.pyplot as plt
-
         ax = plt.gca() if ax is None else ax
 
         if energy is None:
@@ -148,8 +146,6 @@ class EffectiveAreaTable2D(IRF):
 
     def plot(self, ax=None, add_cbar=True, **kwargs):
         """Plot effective area image."""
-        import matplotlib.pyplot as plt
-
         ax = plt.gca() if ax is None else ax
 
         energy = self.axes["energy_true"]
@@ -178,9 +174,14 @@ class EffectiveAreaTable2D(IRF):
         return ax
 
     def peek(self, figsize=(15, 5)):
-        """Quick-look summary plots."""
-        import matplotlib.pyplot as plt
+        """Quick-look summary plots.
 
+        Parameters
+        ----------
+        figsize : tuple
+            Size of the figure.
+
+        """
         fig, axes = plt.subplots(nrows=1, ncols=3, figsize=figsize)
         self.plot(ax=axes[2])
         self.plot_energy_dependence(ax=axes[0])
