@@ -2,11 +2,12 @@
 import logging
 import astropy.units as u
 from astropy.table import Table
-from gammapy.datasets import MapDataset
-from gammapy.irf import EDispKernelMap, EnergyDependentMultiGaussPSF, PSFMap
+from regions import PointSkyRegion
+from gammapy.irf import EDispKernelMap, PSFMap
 from gammapy.maps import Map
 from .core import Maker
 from .utils import (
+    make_counts_rad_max,
     make_edisp_kernel_map,
     make_edisp_map,
     make_map_background_irf,
@@ -20,7 +21,7 @@ log = logging.getLogger(__name__)
 
 
 class MapDatasetMaker(Maker):
-    """Make maps for a single IACT observation.
+    """Make binned maps for a single IACT observation.
 
     Parameters
     ----------
@@ -30,14 +31,72 @@ class MapDatasetMaker(Maker):
         By default, all maps are made.
     background_oversampling : int
         Background evaluation oversampling factor in energy.
+    background_interp_missing_data: bool
+        Interpolate missing values in background 3d map.
+        Default is True, have to be set to True for CTA IRF.
+
+    Examples
+    --------
+    This example shows how to run the MapMaker for a single observation
+
+    >>> from gammapy.data import DataStore
+    >>> from gammapy.datasets import MapDataset
+    >>> from gammapy.maps import WcsGeom, MapAxis
+    >>> from gammapy.makers import MapDatasetMaker
+
+    >>>  #load an observation
+    >>> data_store = DataStore.from_dir("$GAMMAPY_DATA/hess-dl3-dr1")
+    >>> obs = data_store.obs(23523)
+
+    >>> #prepare the geom
+    >>> energy_axis = MapAxis.from_energy_bounds(1.0, 10.0, 4, unit="TeV")
+    >>> energy_axis_true = MapAxis.from_energy_bounds( 0.5, 20, 10, unit="TeV", name="energy_true")
+    >>> geom = WcsGeom.create(skydir=(83.633, 22.014), binsz=0.02, width=(2, 2), frame="icrs", proj="CAR", axes=[energy_axis])
+
+    >>> #Run the maker
+    >>> empty = MapDataset.create(geom=geom, energy_axis_true=energy_axis_true, name="empty")
+    >>> maker = MapDatasetMaker()
+    >>> dataset = maker.run(empty, obs)
+    >>> print(dataset)
+    MapDataset
+    ----------
+    <BLANKLINE>
+      Name                            : empty
+    <BLANKLINE>
+      Total counts                    : 787
+      Total background counts         : 684.52
+      Total excess counts             : 102.48
+    <BLANKLINE>
+      Predicted counts                : 684.52
+      Predicted background counts     : 684.52
+      Predicted excess counts         : nan
+    <BLANKLINE>
+      Exposure min                    : 7.01e+07 m2 s
+      Exposure max                    : 1.10e+09 m2 s
+    <BLANKLINE>
+      Number of total bins            : 40000
+      Number of fit bins              : 40000
+    <BLANKLINE>
+      Fit statistic type              : cash
+      Fit statistic value (-2 log(L)) : nan
+    <BLANKLINE>
+      Number of models                : 0
+      Number of parameters            : 0
+      Number of free parameters       : 0
+
     """
 
     tag = "MapDatasetMaker"
     available_selection = ["counts", "exposure", "background", "psf", "edisp"]
 
-    def __init__(self, selection=None, background_oversampling=None):
+    def __init__(
+        self,
+        selection=None,
+        background_oversampling=None,
+        background_interp_missing_data=True,
+    ):
         self.background_oversampling = background_oversampling
-
+        self.background_interp_missing_data = background_interp_missing_data
         if selection is None:
             selection = self.available_selection
 
@@ -53,6 +112,12 @@ class MapDatasetMaker(Maker):
     def make_counts(geom, observation):
         """Make counts map.
 
+        **NOTE for 1D analysis:** if the `~gammapy.maps.Geom` is built from a
+        `~regions.CircleSkyRegion`, the latter will be directly used to extract
+        the counts. If instead the `~gammapy.maps.Geom` is built from a
+        `~regions.PointSkyRegion`, the size of the ON region is taken from
+        the `RAD_MAX_2D` table containing energy-dependent theta2 cuts.
+
         Parameters
         ----------
         geom : `~gammapy.maps.Geom`
@@ -65,12 +130,15 @@ class MapDatasetMaker(Maker):
         counts : `~gammapy.maps.Map`
             Counts map.
         """
-        counts = Map.from_geom(geom)
-        counts.fill_events(observation.events)
+        if geom.is_region and isinstance(geom.region, PointSkyRegion):
+            counts = make_counts_rad_max(geom, observation.rad_max, observation.events)
+        else:
+            counts = Map.from_geom(geom)
+            counts.fill_events(observation.events)
         return counts
 
     @staticmethod
-    def make_exposure(geom, observation):
+    def make_exposure(geom, observation, use_region_center=True):
         """Make exposure map.
 
         Parameters
@@ -86,16 +154,19 @@ class MapDatasetMaker(Maker):
             Exposure map.
         """
         if isinstance(observation.aeff, Map):
-            return observation.aeff.interp_to_geom(geom=geom,)
+            return observation.aeff.interp_to_geom(
+                geom=geom,
+            )
         return make_map_exposure_true_energy(
             pointing=observation.pointing_radec,
             livetime=observation.observation_live_time_duration,
             aeff=observation.aeff,
             geom=geom,
+            use_region_center=use_region_center,
         )
 
     @staticmethod
-    def make_exposure_irf(geom, observation):
+    def make_exposure_irf(geom, observation, use_region_center=True):
         """Make exposure map with irf geometry.
 
         Parameters
@@ -115,6 +186,7 @@ class MapDatasetMaker(Maker):
             livetime=observation.observation_live_time_duration,
             aeff=observation.aeff,
             geom=geom,
+            use_region_center=use_region_center,
         )
 
     def make_background(self, geom, observation):
@@ -132,26 +204,24 @@ class MapDatasetMaker(Maker):
         background : `~gammapy.maps.Map`
             Background map.
         """
-        if isinstance(observation.bkg, Map):
-            return observation.bkg.interp_to_geom(geom=geom, preserve_counts=True)
-        bkg_coordsys = observation.bkg.meta.get("FOVALIGN", "RADEC")
 
-        if bkg_coordsys == "ALTAZ":
-            pointing = observation.fixed_pointing_info
-        elif bkg_coordsys == "RADEC":
-            pointing = observation.pointing_radec
-        else:
-            raise ValueError(
-                f"Invalid background coordinate system: {bkg_coordsys!r}\n"
-                "Options: ALTAZ, RADEC"
-            )
+        bkg = observation.bkg
+
+        if isinstance(bkg, Map):
+            return bkg.interp_to_geom(geom=geom, preserve_counts=True)
+
+        use_region_center = getattr(self, "use_region_center", True)
+
+        if self.background_interp_missing_data:
+            bkg.interp_missing_data(axis_name="energy")
 
         return make_map_background_irf(
-            pointing=pointing,
+            pointing=observation.fixed_pointing_info,
             ontime=observation.observation_time_duration,
-            bkg=observation.bkg,
+            bkg=bkg,
             geom=geom,
             oversampling=self.background_oversampling,
+            use_region_center=use_region_center,
         )
 
     def make_edisp(self, geom, observation):
@@ -171,11 +241,14 @@ class MapDatasetMaker(Maker):
         """
         exposure = self.make_exposure_irf(geom.squash(axis_name="migra"), observation)
 
+        use_region_center = getattr(self, "use_region_center", True)
+
         return make_edisp_map(
             edisp=observation.edisp,
             pointing=observation.pointing_radec,
             geom=geom,
             exposure_map=exposure,
+            use_region_center=use_region_center,
         )
 
     def make_edisp_kernel(self, geom, observation):
@@ -197,13 +270,17 @@ class MapDatasetMaker(Maker):
             exposure = None
             interp_map = observation.edisp.edisp_map.interp_to_geom(geom)
             return EDispKernelMap(edisp_kernel_map=interp_map, exposure_map=exposure)
+
         exposure = self.make_exposure_irf(geom.squash(axis_name="energy"), observation)
 
+        use_region_center = getattr(self, "use_region_center", True)
+
         return make_edisp_kernel_map(
             edisp=observation.edisp,
             pointing=observation.pointing_radec,
             geom=geom,
             exposure_map=exposure,
+            use_region_center=use_region_center,
         )
 
     def make_psf(self, geom, observation):
@@ -222,13 +299,10 @@ class MapDatasetMaker(Maker):
             Psf map.
         """
         psf = observation.psf
+
         if isinstance(psf, PSFMap):
             return PSFMap(psf.psf_map.interp_to_geom(geom))
 
-        if isinstance(psf, EnergyDependentMultiGaussPSF):
-            rad_axis = geom.axes["rad"]
-            psf = psf.to_psf3d(rad=rad_axis.center)
-
         exposure = self.make_exposure_irf(geom.squash(axis_name="rad"), observation)
 
         return make_psf_map(
@@ -252,7 +326,7 @@ class MapDatasetMaker(Maker):
         meta_table: `~astropy.table.Table`
         """
         meta_table = Table()
-        meta_table["TELESCOP"] = [observation.aeff.meta["TELESCOP"]]
+        meta_table["TELESCOP"] = [observation.aeff.meta.get("TELESCOP", "Unknown")]
         meta_table["OBS_ID"] = [observation.obs_id]
         meta_table["RA_PNT"] = [observation.pointing_radec.icrs.ra.deg] * u.deg
         meta_table["DEC_PNT"] = [observation.pointing_radec.icrs.dec.deg] * u.deg
@@ -276,14 +350,17 @@ class MapDatasetMaker(Maker):
         """
         kwargs = {"gti": observation.gti}
         kwargs["meta_table"] = self.make_meta_table(observation)
+
         mask_safe = Map.from_geom(dataset.counts.geom, dtype=bool)
-        mask_safe.data |= True
+        mask_safe.data[...] = True
 
         kwargs["mask_safe"] = mask_safe
 
         if "counts" in self.selection:
             counts = self.make_counts(dataset.counts.geom, observation)
-            kwargs["counts"] = counts
+        else:
+            counts = Map.from_geom(dataset.counts.geom, data=0)
+        kwargs["counts"] = counts
 
         if "exposure" in self.selection:
             exposure = self.make_exposure(dataset.exposure.geom, observation)
@@ -308,4 +385,4 @@ class MapDatasetMaker(Maker):
 
             kwargs["edisp"] = edisp
 
-        return MapDataset(name=dataset.name, **kwargs)
+        return dataset.__class__(name=dataset.name, **kwargs)
