@@ -9,6 +9,7 @@ from astropy.io import fits
 from astropy.table import Column, Table, hstack
 from astropy.time import Time
 from astropy.utils import lazyproperty
+import matplotlib.pyplot as plt
 from gammapy.utils.interpolation import interpolation_scale
 from gammapy.utils.time import time_ref_from_dict, time_ref_to_dict
 from .utils import INVALID_INDEX, edges_from_lo_hi
@@ -23,26 +24,47 @@ def flat_if_equal(array):
         return array
 
 
-def coord_to_pix(edges, coord, interp="lin"):
-    """Convert axis to pixel coordinates for given interpolation scheme."""
-    scale = interpolation_scale(interp)
+class AxisCoordInterpolator:
+    """Axis coord interpolator"""
 
-    interp_fn = scipy.interpolate.interp1d(
-        scale(edges), np.arange(len(edges), dtype=float), fill_value="extrapolate"
-    )
+    def __init__(self, edges, interp="lin"):
+        self.scale = interpolation_scale(interp)
+        self.x = self.scale(edges)
+        self.y = np.arange(len(edges), dtype=float)
+        self.fill_value = "extrapolate"
 
-    return interp_fn(scale(coord))
+        if len(edges) == 1:
+            self.kind = 0
+        else:
+            self.kind = 1
+
+    def coord_to_pix(self, coord):
+        """Pix to coord"""
+        interp_fn = scipy.interpolate.interp1d(
+            x=self.x, y=self.y, kind=self.kind, fill_value=self.fill_value
+        )
+        return interp_fn(self.scale(coord))
+
+    def pix_to_coord(self, pix):
+        """Coord to pix"""
+        interp_fn = scipy.interpolate.interp1d(
+            x=self.y, y=self.x, kind=self.kind, fill_value=self.fill_value
+        )
+        return self.scale.inverse(interp_fn(pix))
 
 
-def pix_to_coord(edges, pix, interp="lin"):
-    """Convert pixel to grid coordinates for given interpolation scheme."""
-    scale = interpolation_scale(interp)
+PLOT_AXIS_LABEL = {
+    "energy": "Energy",
+    "energy_true": "True Energy",
+    "offset": "FoV Offset",
+    "rad": "Source Offset",
+    "migra": "Energy / True Energy",
+    "fov_lon": "FoV Lon.",
+    "fov_lat": "FoV Lat.",
+    "time": "Time",
+}
 
-    interp_fn = scipy.interpolate.interp1d(
-        np.arange(len(edges), dtype=float), scale(edges), fill_value="extrapolate"
-    )
-
-    return scale.inverse(interp_fn(pix))
+DEFAULT_LABEL_TEMPLATE = "{quantity} [{unit}]"
 
 
 class MapAxis:
@@ -88,9 +110,6 @@ class MapAxis:
         if ~(np.all(nodes == np.sort(nodes)) or np.all(nodes[::-1] == np.sort(nodes))):
             raise ValueError("MapAxis: node values must be sorted")
 
-        if len(nodes) == 1 and node_type == "center":
-            raise ValueError("Single bins can only be used with node-type 'edges'")
-
         if isinstance(nodes, u.Quantity):
             unit = nodes.unit if nodes.unit is not None else ""
             nodes = nodes.value
@@ -118,6 +137,7 @@ class MapAxis:
             raise ValueError(f"Invalid node type: {node_type!r}")
 
         self._nbin = nbin
+        self._use_center_as_plot_labels = None
 
     def assert_name(self, required_name):
         """Assert axis name if a specific one is required.
@@ -157,30 +177,52 @@ class MapAxis:
         aligned = np.allclose(np.round(pix_all) - pix_all, 0, atol=atol)
         return aligned and self.interp == other.interp
 
-    def __eq__(self, other):
+    def is_allclose(self, other, **kwargs):
+        """Check if other map axis is all close.
+
+        Parameters
+        ----------
+        other : `MapAxis`
+            Other map axis
+        **kwargs : dict
+            Keyword arguments forwarded to `~numpy.allclose`
+
+        Returns
+        -------
+        is_allclose : bool
+            Whether other axis is allclose
+        """
         if not isinstance(other, self.__class__):
-            return NotImplemented
+            return TypeError(f"Cannot compare {type(self)} and {type(other)}")
 
-        # TODO: implement an allclose method for MapAxis and call it here
         if self.edges.shape != other.edges.shape:
             return False
         if not self.unit.is_equivalent(other.unit):
             return False
         return (
-            np.allclose(
-                self.edges.to(other.unit).value, other.edges.value, atol=1e-6, rtol=1e-6
-            )
+            np.allclose(self.edges, other.edges, **kwargs)
             and self._node_type == other._node_type
             and self._interp == other._interp
             and self.name.upper() == other.name.upper()
         )
 
+    def __eq__(self, other):
+        if not isinstance(other, self.__class__):
+            return False
+
+        return self.is_allclose(other, rtol=1e-6, atol=1e-6)
+
     def __ne__(self, other):
         return not self.__eq__(other)
 
     def __hash__(self):
         return id(self)
 
+    @lazyproperty
+    def _transform(self):
+        """Interpolate coordinates to pixel"""
+        return AxisCoordInterpolator(edges=self._nodes, interp=self.interp)
+
     @property
     def is_energy_axis(self):
         return self.name in ["energy", "energy_true"]
@@ -195,11 +237,6 @@ class MapAxis:
         """Name of the axis."""
         return self._name
 
-    @name.setter
-    def name(self, value):
-        """Name of the axis."""
-        self._name = value
-
     @lazyproperty
     def edges(self):
         """Return array of bin edges."""
@@ -233,17 +270,29 @@ class MapAxis:
             self.edges_max - self.center,
         )
 
+    @property
+    def use_center_as_plot_labels(self):
+        """Use center as plot labels"""
+        if self._use_center_as_plot_labels is not None:
+            return self._use_center_as_plot_labels
+
+        return self.node_type == "center"
+
+    @use_center_as_plot_labels.setter
+    def use_center_as_plot_labels(self, value):
+        """Use center as plot labels"""
+        self._use_center_as_plot_labels = bool(value)
+
     @property
     def as_plot_labels(self):
         """Return list of axis plot labels"""
-        if self.node_type == "edges":
+        if self.use_center_as_plot_labels:
+            labels = [f"{val:.2e}" for val in self.center]
+        else:
             labels = [
                 f"{val_min:.2e} - {val_max:.2e}"
                 for val_min, val_max in self.iter_by_edges
             ]
-        else:
-            labels = [f"{val:.2e}" for val in self.center]
-
         return labels
 
     @property
@@ -263,6 +312,34 @@ class MapAxis:
 
         return mpl_scale[self.interp]
 
+    def to_node_type(self, node_type):
+        """Return MapAxis copy chaning its node type to node_type.
+
+        Parameters
+        ----------
+        node_type : str 'edges' or 'center'
+            the target node type
+
+        Returns
+        -------
+        axis : `~gammapy.maps.MapAxis`
+            the new MapAxis
+        """
+        if node_type == self.node_type:
+            return self
+        else:
+            if node_type == "center":
+                nodes = self.center
+            else:
+                nodes = self.edges
+            return self.__class__(
+                nodes=nodes,
+                interp=self.interp,
+                name=self.name,
+                node_type=node_type,
+                unit=self.unit,
+            )
+
     def format_plot_xaxis(self, ax):
         """Format plot axis
 
@@ -278,9 +355,14 @@ class MapAxis:
         """
         ax.set_xscale(self.as_plot_scale)
 
-        xlabel = self.name.capitalize() + f" [{ax.xaxis.units}]"
+        xlabel = DEFAULT_LABEL_TEMPLATE.format(
+            quantity=PLOT_AXIS_LABEL.get(self.name, self.name.capitalize()),
+            unit=ax.xaxis.units,
+        )
         ax.set_xlabel(xlabel)
-        ax.set_xlim(self.bounds)
+        xmin, xmax = self.bounds
+        if not xmin == xmax:
+            ax.set_xlim(self.bounds)
         return ax
 
     def format_plot_yaxis(self, ax):
@@ -298,7 +380,10 @@ class MapAxis:
         """
         ax.set_yscale(self.as_plot_scale)
 
-        ylabel = self.name.capitalize() + f" [{ax.yaxis.units}]"
+        ylabel = DEFAULT_LABEL_TEMPLATE.format(
+            quantity=PLOT_AXIS_LABEL.get(self.name, self.name.capitalize()),
+            unit=ax.yaxis.units,
+        )
         ax.set_ylabel(ylabel)
         ax.set_ylim(self.bounds)
         return ax
@@ -386,7 +471,7 @@ class MapAxis:
         elif interp == "log":
             nodes = np.exp(np.linspace(np.log(lo_bnd), np.log(hi_bnd), nnode))
         elif interp == "sqrt":
-            nodes = np.linspace(lo_bnd ** 0.5, hi_bnd ** 0.5, nnode) ** 2.0
+            nodes = np.linspace(lo_bnd**0.5, hi_bnd**0.5, nnode) ** 2.0
         else:
             raise ValueError(f"Invalid interp: {interp}")
 
@@ -637,7 +722,7 @@ class MapAxis:
             Array of axis coordinate values.
         """
         pix = pix - self._pix_offset
-        values = pix_to_coord(self._nodes, pix, interp=self._interp)
+        values = self._transform.pix_to_coord(pix=pix)
         return u.Quantity(values, unit=self.unit, copy=False)
 
     def pix_to_idx(self, pix, clip=False):
@@ -679,7 +764,7 @@ class MapAxis:
             Array of pixel coordinate values.
         """
         coord = u.Quantity(coord, self.unit, copy=False).value
-        pix = coord_to_pix(self._nodes, coord, interp=self._interp)
+        pix = self._transform.coord_to_pix(coord=coord)
         return np.array(pix + self._pix_offset, ndmin=1)
 
     def coord_to_idx(self, coord, clip=False):
@@ -1309,13 +1394,15 @@ class MapAxes(Sequence):
     @property
     def is_flat(self):
         """Whether axes is flat"""
-        return np.all(self.shape == 1)
+        shape = np.array(self.shape)
+        return np.all(shape == 1)
 
     @property
     def is_unidimensional(self):
         """Whether axes is unidimensional"""
-        value = (np.array(self.shape) > 1).sum()
-        return value == 1
+        shape = np.array(self.shape)
+        non_zero = np.count_nonzero(shape > 1)
+        return self.is_flat or non_zero == 1
 
     @property
     def reverse(self):
@@ -1598,13 +1685,16 @@ class MapAxes(Sequence):
         return self.__class__(axes=axes)
 
     def __getitem__(self, idx):
-        if isinstance(idx, (int, slice)):
+        if isinstance(idx, int):
             return self._axes[idx]
         elif isinstance(idx, str):
             for ax in self._axes:
                 if ax.name == idx:
                     return ax
             raise KeyError(f"No axes: {idx!r}")
+        elif isinstance(idx, slice):
+            axes = self._axes[idx]
+            return self.__class__(axes=axes)
         elif isinstance(idx, list):
             axes = []
             for name in idx:
@@ -1925,7 +2015,7 @@ class MapAxes(Sequence):
                 ax = MapAxis(ax)
 
             if ax.name == "":
-                ax.name = f"axis{idx}"
+                ax._name = f"axis{idx}"
 
             axes_out.append(ax)
 
@@ -1959,6 +2049,39 @@ class MapAxes(Sequence):
         """Center coordinates"""
         return tuple([ax.pix_to_coord((float(ax.nbin) - 1.0) / 2.0) for ax in self])
 
+    def is_allclose(self, other, **kwargs):
+        """Check if other map axes are all close.
+
+        Parameters
+        ----------
+        other : `MapAxes`
+            Other map axes
+        **kwargs : dict
+            Keyword arguments forwarded to `~MapAxis.is_allclose`
+
+        Returns
+        -------
+        is_allclose : bool
+            Whether other axes are all close
+        """
+        if not isinstance(other, self.__class__):
+            return TypeError(f"Cannot compare {type(self)} and {type(other)}")
+
+        return np.all([ax0.is_allclose(ax1, **kwargs) for ax0, ax1 in zip(other, self)])
+
+    def __eq__(self, other):
+        if not isinstance(other, self.__class__):
+            return False
+
+        return self.is_allclose(other, rtol=1e-6, atol=1e-6)
+
+    def __ne__(self, other):
+        return not self.__eq__(other)
+
+    def copy(self):
+        """Init map axes instance by copying each axis."""
+        return self.__class__([_.copy() for _ in self])
+
 
 class TimeMapAxis:
     """Class representing a time axis.
@@ -2097,6 +2220,17 @@ class TimeMapAxis:
 
         return edges_from_lo_hi(self.edges_min, self.edges_max)
 
+    @property
+    def bounds(self):
+        """Bounds of the axis (~astropy.units.Quantity)"""
+        return self.edges_min[0], self.edges_max[-1]
+
+    @property
+    def time_bounds(self):
+        """Bounds of the axis (~astropy.units.Quantity)"""
+        t_min, t_max = self.bounds
+        return t_min + self.reference_time, t_max + self.reference_time
+
     @property
     def time_min(self):
         """Return axis lower edges as Time objects."""
@@ -2184,11 +2318,12 @@ class TimeMapAxis:
         ax : `~matplotlib.pyplot.Axis`
             Formatted plot axis
         """
-        import matplotlib.pyplot as plt
         from matplotlib.dates import DateFormatter
 
-        xlabel = self.name.capitalize() + f" [{self.time_format}]"
-
+        xlabel = DEFAULT_LABEL_TEMPLATE.format(
+            quantity=PLOT_AXIS_LABEL.get(self.name, self.name.capitalize()),
+            unit=self.time_format,
+        )
         ax.set_xlabel(xlabel)
 
         if self.time_format == "iso":
@@ -2216,9 +2351,23 @@ class TimeMapAxis:
                 f' expected "{required_name}", got: "{self.name}"'
             )
 
-    def __eq__(self, other):
+    def is_allclose(self, other, **kwargs):
+        """Check if other map axis is all close.
+
+        Parameters
+        ----------
+        other : `TimeMapAxis`
+            Other map axis
+        **kwargs : dict
+            Keyword arguments forwarded to `~numpy.allclose`
+
+        Returns
+        -------
+        is_allclose : bool
+            Whether other axis is allclose
+        """
         if not isinstance(other, self.__class__):
-            return NotImplemented
+            return TypeError(f"Cannot compare {type(self)} and {type(other)}")
 
         if self._edges_min.shape != other._edges_min.shape:
             return False
@@ -2228,12 +2377,18 @@ class TimeMapAxis:
         delta_max = self.time_max - other.time_max
 
         return (
-            np.allclose(delta_min.to_value("s"), 0.0, atol=1e-6)
-            and np.allclose(delta_max.to_value("s"), 0.0, atol=1e-6)
+            np.allclose(delta_min.to_value("s"), 0.0, **kwargs)
+            and np.allclose(delta_max.to_value("s"), 0.0, **kwargs)
             and self._interp == other._interp
             and self.name.upper() == other.name.upper()
         )
 
+    def __eq__(self, other):
+        if not isinstance(other, self.__class__):
+            return False
+
+        return self.is_allclose(other=other, atol=1e-6)
+
     def __ne__(self, other):
         return not self.__eq__(other)
 
@@ -2852,14 +3007,32 @@ class LabelMapAxis:
         str_ += fmt.format("labels", "{0}".format(list(self._labels)))
         return str_.expandtabs(tabsize=2)
 
-    def __eq__(self, other):
+    def is_allclose(self, other, **kwargs):
+        """Check if other map axis is all close.
+
+        Parameters
+        ----------
+        other : `LabelMapAxis`
+            Other map axis
+
+        Returns
+        -------
+        is_allclose : bool
+            Whether other axis is allclose
+        """
         if not isinstance(other, self.__class__):
-            return NotImplemented
+            return TypeError(f"Cannot compare {type(self)} and {type(other)}")
 
         name_equal = self.name.upper() == other.name.upper()
         labels_equal = np.all(self.center == other.center)
         return name_equal & labels_equal
 
+    def __eq__(self, other):
+        if not isinstance(other, self.__class__):
+            return False
+
+        return self.is_allclose(other=other)
+
     def __ne__(self, other):
         return not self.__eq__(other)
 
