@@ -6,10 +6,13 @@ import json
 import numpy as np
 from astropy import units as u
 from astropy.io import fits
-from gammapy.maps.hpx import HpxGeom
+import matplotlib.pyplot as plt
 from gammapy.utils.scripts import make_path
-from .geom import MapAxis, MapCoord, pix_tuple_to_idx
-from .utils import INVALID_VALUE
+from gammapy.utils.units import energy_unit_format
+from .axes import MapAxis
+from .coord import MapCoord
+from .geom import pix_tuple_to_idx
+from .io import JsonQuantityDecoder
 
 __all__ = ["Map"]
 
@@ -24,20 +27,25 @@ class Map(abc.ABC):
     ----------
     geom : `~gammapy.maps.Geom`
         Geometry
-    data : `~numpy.ndarray`
+    data : `~numpy.ndarray` or `~astropy.units.Quantity`
         Data array
     meta : `dict`
         Dictionary to store meta data
     unit : str or `~astropy.units.Unit`
-        Data unit
+        Data unit, ignored if data is a Quantity.
     """
 
     tag = "map"
 
     def __init__(self, geom, data, meta=None, unit=""):
         self._geom = geom
-        self.data = data
-        self.unit = unit
+
+        if isinstance(data, u.Quantity):
+            self._unit = u.Unit(unit)
+            self.quantity = data
+        else:
+            self.data = data
+            self._unit = u.Unit(unit)
 
         if meta is None:
             self.meta = {}
@@ -45,8 +53,7 @@ class Map(abc.ABC):
             self.meta = meta
 
     def _init_copy(self, **kwargs):
-        """Init map instance by copying missing init arguments from self.
-        """
+        """Init map instance by copying missing init arguments from self."""
         argnames = inspect.getfullargspec(self.__init__).args
         argnames.remove("self")
         argnames.remove("dtype")
@@ -57,6 +64,11 @@ class Map(abc.ABC):
 
         return self.from_geom(**kwargs)
 
+    @property
+    def is_mask(self):
+        """Whether map is mask with bool dtype"""
+        return self.data.dtype == bool
+
     @property
     def geom(self):
         """Map geometry (`~gammapy.maps.Geom`)"""
@@ -69,8 +81,15 @@ class Map(abc.ABC):
 
     @data.setter
     def data(self, value):
+        """Set data
+
+        Parameters
+        ----------
+        value : array-like
+            Data array
+        """
         if np.isscalar(value):
-            value = value * np.ones(self.geom.data_shape)
+            value = value * np.ones(self.geom.data_shape, dtype=type(value))
 
         if isinstance(value, u.Quantity):
             raise TypeError("Map data must be a Numpy array. Set unit separately")
@@ -85,10 +104,6 @@ class Map(abc.ABC):
         """Map unit (`~astropy.units.Unit`)"""
         return self._unit
 
-    @unit.setter
-    def unit(self, val):
-        self._unit = u.Unit(val)
-
     @property
     def meta(self):
         """Map meta (`dict`)"""
@@ -105,9 +120,17 @@ class Map(abc.ABC):
 
     @quantity.setter
     def quantity(self, val):
+        """Set data and unit
+
+        Parameters
+        ----------
+        value : `~astropy.units.Quantity`
+           Quantity
+        """
         val = u.Quantity(val, copy=False)
+
         self.data = val.value
-        self.unit = val.unit
+        self._unit = val.unit
 
     @staticmethod
     def create(**kwargs):
@@ -146,9 +169,9 @@ class Map(abc.ABC):
         map : `Map`
             Empty map object.
         """
-        from .hpxmap import HpxMap
-        from .wcsmap import WcsMap
-        from .regionnd import RegionNDMap
+        from .hpx import HpxMap
+        from .region import RegionNDMap
+        from .wcs import WcsMap
 
         map_type = kwargs.setdefault("map_type", "wcs")
         if "wcs" in map_type.lower():
@@ -162,7 +185,9 @@ class Map(abc.ABC):
             raise ValueError(f"Unrecognized map type: {map_type!r}")
 
     @staticmethod
-    def read(filename, hdu=None, hdu_bands=None, map_type="auto", format=None):
+    def read(
+        filename, hdu=None, hdu_bands=None, map_type="auto", format=None, colname=None
+    ):
         """Read a map from a FITS file.
 
         Parameters
@@ -175,12 +200,14 @@ class Map(abc.ABC):
             Name or index of the HDU with the BANDS table.  If not
             defined this will be inferred from the FITS header of the
             map HDU.
-        map_type : {'wcs', 'wcs-sparse', 'hpx', 'hpx-sparse', 'auto'}
+        map_type : {'wcs', 'wcs-sparse', 'hpx', 'hpx-sparse', 'auto', 'region'}
             Map type.  Selects the class that will be used to
             instantiate the map.  The map type should be consistent
             with the format of the input file.  If map_type is 'auto'
             then an appropriate map type will be inferred from the
             input file.
+        colname : str, optional
+            data column name to be used of healix map.
 
         Returns
         -------
@@ -188,7 +215,9 @@ class Map(abc.ABC):
             Map object
         """
         with fits.open(str(make_path(filename)), memmap=False) as hdulist:
-            return Map.from_hdulist(hdulist, hdu, hdu_bands, map_type, format=format)
+            return Map.from_hdulist(
+                hdulist, hdu, hdu_bands, map_type, format=format, colname=colname
+            )
 
     @staticmethod
     def from_geom(geom, meta=None, data=None, unit="", dtype="float32"):
@@ -212,8 +241,8 @@ class Map(abc.ABC):
 
         """
         from .hpx import HpxGeom
-        from .wcs import WcsGeom
         from .region import RegionGeom
+        from .wcs import WcsGeom
 
         if isinstance(geom, HpxGeom):
             map_type = "hpx"
@@ -228,20 +257,48 @@ class Map(abc.ABC):
         return cls_out(geom, data=data, meta=meta, unit=unit, dtype=dtype)
 
     @staticmethod
-    def from_hdulist(hdulist, hdu=None, hdu_bands=None, map_type="auto", format=None):
-        """Create from `astropy.io.fits.HDUList`."""
+    def from_hdulist(
+        hdulist, hdu=None, hdu_bands=None, map_type="auto", format=None, colname=None
+    ):
+        """Create from `astropy.io.fits.HDUList`.
+
+        Parameters
+        ----------
+        hdulist :  `~astropy.io.fits.HDUList`
+            HDU list containing HDUs for map data and bands.
+        hdu : str
+            Name or index of the HDU with the map data.
+        hdu_bands : str
+            Name or index of the HDU with the BANDS table.
+        map_type : {"auto", "wcs", "hpx", "region"}
+            Map type.
+        format : {'gadf', 'fgst-ccube', 'fgst-template'}
+            FITS format convention.
+        colname : str, optional
+            Data column name to be used for the HEALPix map.
+
+        Returns
+        -------
+        map_out : `Map`
+            Map object
+        """
         if map_type == "auto":
             map_type = Map._get_map_type(hdulist, hdu)
         cls_out = Map._get_map_cls(map_type)
-        return cls_out.from_hdulist(
-            hdulist, hdu=hdu, hdu_bands=hdu_bands, format=format
-        )
+        if map_type == "hpx":
+            return cls_out.from_hdulist(
+                hdulist, hdu=hdu, hdu_bands=hdu_bands, format=format, colname=colname
+            )
+        else:
+            return cls_out.from_hdulist(
+                hdulist, hdu=hdu, hdu_bands=hdu_bands, format=format
+            )
 
     @staticmethod
     def _get_meta_from_header(header):
         """Load meta data from a FITS header."""
         if "META" in header:
-            return json.loads(header["META"])
+            return json.loads(header["META"], cls=JsonQuantityDecoder)
         else:
             return {}
 
@@ -261,8 +318,10 @@ class Map(abc.ABC):
 
         if ("PIXTYPE" in header) and (header["PIXTYPE"] == "HEALPIX"):
             return "hpx"
-        else:
+        elif "CTYPE1" in header:
             return "wcs"
+        else:
+            return "region"
 
     @staticmethod
     def _get_map_cls(map_type):
@@ -274,19 +333,19 @@ class Map(abc.ABC):
         but that's non-trivial to implement without avoiding circular imports.
         """
         if map_type == "wcs":
-            from .wcsnd import WcsNDMap
+            from .wcs import WcsNDMap
 
             return WcsNDMap
         elif map_type == "wcs-sparse":
             raise NotImplementedError()
         elif map_type == "hpx":
-            from .hpxnd import HpxNDMap
+            from .hpx import HpxNDMap
 
             return HpxNDMap
         elif map_type == "hpx-sparse":
             raise NotImplementedError()
         elif map_type == "region":
-            from .regionnd import RegionNDMap
+            from .region import RegionNDMap
 
             return RegionNDMap
         else:
@@ -332,15 +391,64 @@ class Map(abc.ABC):
         hdulist = self.to_hdulist(**kwargs)
         hdulist.writeto(str(make_path(filename)), overwrite=overwrite)
 
-    def iter_by_image(self):
+    def iter_by_axis(self, axis_name, keepdims=False):
+        """ "Iterate over a given axis
+
+        Yields
+        ------
+        map : `Map`
+            Map iteration.
+
+        See also
+        --------
+        iter_by_image : iterate by image returning a map
+        """
+        axis = self.geom.axes[axis_name]
+        for idx in range(axis.nbin):
+            idx_axis = slice(idx, idx + 1) if keepdims else idx
+            slices = {axis_name: idx_axis}
+            yield self.slice_by_idx(slices=slices)
+
+    def iter_by_image(self, keepdims=False):
+        """Iterate over image planes of a map.
+
+        Parameters
+        ----------
+        keepdims : bool
+            Keep dimensions.
+
+        Yields
+        ------
+        map : `Map`
+            Map iteration.
+
+        See also
+        --------
+        iter_by_image_data : iterate by image returning data and index
+        """
+        for idx in np.ndindex(self.geom.shape_axes):
+            if keepdims:
+                names = self.geom.axes.names
+                slices = {name: slice(_, _ + 1) for name, _ in zip(names, idx)}
+                yield self.slice_by_idx(slices=slices)
+            else:
+                yield self.get_image_by_idx(idx=idx)
+
+    def iter_by_image_data(self):
         """Iterate over image planes of the map.
 
-        This is a generator yielding ``(data, idx)`` tuples,
-        where ``data`` is a `numpy.ndarray` view of the image plane data,
-        and ``idx`` is a tuple of int, the index of the image plane.
-
         The image plane index is in data order, so that the data array can be
-        indexed directly. See :ref:`mapiter` for further information.
+        indexed directly.
+
+        Yields
+        ------
+        (data, idx) : tuple
+            Where ``data`` is a `numpy.ndarray` view of the image plane data,
+            and ``idx`` is a tuple of int, the index of the image plane.
+
+        See also
+        --------
+        iter_by_image : iterate by image returning a map
         """
         for idx in np.ndindex(self.geom.shape_axes):
             yield self.data[idx[::-1]], idx[::-1]
@@ -370,23 +478,21 @@ class Map(abc.ABC):
         vals = u.Quantity(map_in.get_by_idx(idx), map_in.unit)
         self.fill_by_coord(coords, vals)
 
-    @abc.abstractmethod
-    def pad(self, pad_width, mode="constant", cval=0, order=1):
+    def pad(self, pad_width, axis_name=None, mode="constant", cval=0, method="linear"):
         """Pad the spatial dimensions of the map.
 
         Parameters
         ----------
         pad_width : {sequence, array_like, int}
             Number of pixels padded to the edges of each axis.
+        axis_name : str
+            Which axis to downsample. By default spatial axes are padded.
         mode : {'edge', 'constant', 'interp'}
             Padding mode.  'edge' pads with the closest edge value.
             'constant' pads with a constant value. 'interp' pads with
             an extrapolated value.
         cval : float
             Padding value when mode='consant'.
-        order : int
-            Order of interpolation when mode='constant' (0 =
-            nearest-neighbor, 1 = linear, 2 = quadratic, 3 = cubic).
 
         Returns
         -------
@@ -394,6 +500,28 @@ class Map(abc.ABC):
             Padded map.
 
         """
+        if axis_name:
+            if np.isscalar(pad_width):
+                pad_width = (pad_width, pad_width)
+
+            geom = self.geom.pad(pad_width=pad_width, axis_name=axis_name)
+            idx = self.geom.axes.index_data(axis_name)
+            pad_width_np = [(0, 0)] * self.data.ndim
+            pad_width_np[idx] = pad_width
+
+            kwargs = {}
+            if mode == "constant":
+                kwargs["constant_values"] = cval
+
+            data = np.pad(self.data, pad_width=pad_width_np, mode=mode, **kwargs)
+            return self.__class__(
+                geom=geom, data=data, unit=self.unit, meta=self.meta.copy()
+            )
+
+        return self._pad_spatial(pad_width, mode="constant", cval=cval)
+
+    @abc.abstractmethod
+    def _pad_spatial(self, pad_width, mode="constant", cval=0, order=1):
         pass
 
     @abc.abstractmethod
@@ -414,7 +542,7 @@ class Map(abc.ABC):
         pass
 
     @abc.abstractmethod
-    def downsample(self, factor, preserve_counts=True, axis=None):
+    def downsample(self, factor, preserve_counts=True, axis_name=None):
         """Downsample the spatial dimension by a given factor.
 
         Parameters
@@ -425,7 +553,7 @@ class Map(abc.ABC):
             Preserve the integral over each bin.  This should be true
             if the map is an integral quantity (e.g. counts) and false if
             the map is a differential quantity (e.g. intensity).
-        axis : str
+        axis_name : str
             Which axis to downsample. By default spatial axes are downsampled.
 
         Returns
@@ -436,7 +564,7 @@ class Map(abc.ABC):
         pass
 
     @abc.abstractmethod
-    def upsample(self, factor, order=0, preserve_counts=True, axis=None):
+    def upsample(self, factor, order=0, preserve_counts=True, axis_name=None):
         """Upsample the spatial dimension by a given factor.
 
         Parameters
@@ -449,7 +577,7 @@ class Map(abc.ABC):
             Preserve the integral over each bin.  This should be true
             if the map is an integral quantity (e.g. counts) and false if
             the map is a differential quantity (e.g. intensity).
-        axis : str
+        axis_name : str
             Which axis to upsample. By default spatial axes are upsampled.
 
 
@@ -482,6 +610,8 @@ class Map(abc.ABC):
         map : `Map`
             Map with resampled axis.
         """
+        from .hpx import HpxGeom
+
         geom = self.geom.resample_axis(axis)
 
         axis_self = self.geom.axes[axis.name]
@@ -510,12 +640,11 @@ class Map(abc.ABC):
         return self._init_copy(data=data, geom=geom)
 
     def slice_by_idx(
-        self, slices,
+        self,
+        slices,
     ):
         """Slice sub map from map object.
 
-        For usage examples, see :ref:`mapslicing`.
-
         Parameters
         ----------
         slices : dict
@@ -648,7 +777,7 @@ class Map(abc.ABC):
         data = self.data[idx[::-1]]
         return self.__class__(geom=geom, data=data, unit=self.unit, meta=self.meta)
 
-    def get_by_coord(self, coords):
+    def get_by_coord(self, coords, fill_value=np.nan):
         """Return map values at the given map coordinates.
 
         Parameters
@@ -657,6 +786,9 @@ class Map(abc.ABC):
             Coordinate arrays for each dimension of the map.  Tuple
             should be ordered as (lon, lat, x_0, ..., x_n) where x_i
             are coordinates for non-spatial dimensions of the map.
+        fill_value : float
+            Value which is returned if the position is outside of the projection
+            footprint
 
         Returns
         -------
@@ -668,10 +800,10 @@ class Map(abc.ABC):
             coords, frame=self.geom.frame, axis_names=self.geom.axes.names
         )
         pix = self.geom.coord_to_pix(coords)
-        vals = self.get_by_pix(pix)
+        vals = self.get_by_pix(pix, fill_value=fill_value)
         return vals
 
-    def get_by_pix(self, pix):
+    def get_by_pix(self, pix, fill_value=np.nan):
         """Return map values at the given pixel coordinates.
 
         Parameters
@@ -681,6 +813,9 @@ class Map(abc.ABC):
             Tuple should be ordered as (I_lon, I_lat, I_0, ..., I_n)
             for WCS maps and (I_hpx, I_0, ..., I_n) for HEALPix maps.
             Pixel indices can be either float or integer type.
+        fill_value : float
+            Value which is returned if the position is outside of the projection
+            footprint
 
         Returns
         -------
@@ -696,9 +831,8 @@ class Map(abc.ABC):
         mask = self.geom.contains_pix(pix)
 
         if not mask.all():
-            invalid = INVALID_VALUE[self.data.dtype]
-            vals = vals.astype(type(invalid))
-            vals[~mask] = invalid
+            vals = vals.astype(type(fill_value))
+            vals[~mask] = fill_value
 
         return vals
 
@@ -722,7 +856,7 @@ class Map(abc.ABC):
         pass
 
     @abc.abstractmethod
-    def interp_by_coord(self, coords, interp=None, fill_value=None):
+    def interp_by_coord(self, coords, method="linear", fill_value=None):
         """Interpolate map values at the given map coordinates.
 
         Parameters
@@ -731,16 +865,9 @@ class Map(abc.ABC):
             Coordinate arrays for each dimension of the map.  Tuple
             should be ordered as (lon, lat, x_0, ..., x_n) where x_i
             are coordinates for non-spatial dimensions of the map.
-
-        interp : {None, 'nearest', 'linear', 'cubic', 0, 1, 2, 3}
-            Method to interpolate data values.  By default no
-            interpolation is performed and the return value will be
-            the amplitude of the pixel encompassing the given
-            coordinate.  Integer values can be used in lieu of strings
-            to choose the interpolation method of the given order
-            (0='nearest', 1='linear', 2='quadratic', 3='cubic').  Note
-            that only 'nearest' and 'linear' methods are supported for
-            all map types.
+        method : {"linear", "nearest"}
+            Method to interpolate data values. By default linear
+            interpolation is performed.
         fill_value : None or float value
             The value to use for points outside of the interpolation domain.
             If None, values outside the domain are extrapolated.
@@ -753,7 +880,7 @@ class Map(abc.ABC):
         pass
 
     @abc.abstractmethod
-    def interp_by_pix(self, pix, interp=None, fill_value=None):
+    def interp_by_pix(self, pix, method="linear", fill_value=None):
         """Interpolate map values at the given pixel coordinates.
 
         Parameters
@@ -763,16 +890,9 @@ class Map(abc.ABC):
             map.  Tuple should be ordered as (p_lon, p_lat, p_0, ...,
             p_n) where p_i are pixel coordinates for non-spatial
             dimensions of the map.
-
-        interp : {None, 'nearest', 'linear', 'cubic', 0, 1, 2, 3}
-            Method to interpolate data values.  By default no
-            interpolation is performed and the return value will be
-            the amplitude of the pixel encompassing the given
-            coordinate.  Integer values can be used in lieu of strings
-            to choose the interpolation method of the given order
-            (0='nearest', 1='linear', 2='quadratic', 3='cubic').  Note
-            that only 'nearest' and 'linear' methods are supported for
-            all map types.
+        method : {"linear", "nearest"}
+            Method to interpolate data values. By default linear
+            interpolation is performed.
         fill_value : None or float value
             The value to use for points outside of the interpolation domain.
             If None, values outside the domain are extrapolated.
@@ -784,7 +904,7 @@ class Map(abc.ABC):
         """
         pass
 
-    def interp_to_geom(self, geom, preserve_counts=False, **kwargs):
+    def interp_to_geom(self, geom, preserve_counts=False, fill_value=0, **kwargs):
         """Interpolate map to input geometry.
 
         Parameters
@@ -803,27 +923,67 @@ class Map(abc.ABC):
         interp_map : `Map`
             Interpolated Map
         """
-
         coords = geom.get_coord()
-
-        # set nearest neighbour interpolation for mask as default
-        if self.data.dtype == bool:
-            kwargs.setdefault("interp", 0)
+        map_copy = self.copy()
 
         if preserve_counts:
-            if geom.ndim > 2:
-                assert self.geom.axes[0] == geom.axes[0]  # Energy axis has to match
-            old_map_copy = self.copy()
-            old_map_copy.data /= self.geom.solid_angle().to_value("deg2")
-            data = old_map_copy.interp_by_coord(coords, **kwargs)
+            if geom.ndim > 2 and geom.axes[0] != self.geom.axes[0]:
+                raise ValueError(
+                    f"Energy axis do not match: expected {self.geom.axes[0]}, but got {geom.axes[0]}."
+                )
+            map_copy.data /= map_copy.geom.solid_angle().to_value("deg2")
+
+        if map_copy.is_mask:
+            # TODO: check this NaN handling is needed
+            data = map_copy.get_by_coord(coords)
+            data = np.nan_to_num(data, nan=fill_value).astype(bool)
+        else:
+            data = map_copy.interp_by_coord(coords, fill_value=fill_value, **kwargs)
+
+        if preserve_counts:
             data *= geom.solid_angle().to_value("deg2")
-        else:
-            data = self.interp_by_coord(coords, **kwargs)
 
-        if self.data.dtype == bool:
-            data = data.astype(bool)
         return Map.from_geom(geom, data=data, unit=self.unit)
 
+    def resample(self, geom, weights=None, preserve_counts=True):
+        """Resample pixels to ``geom`` with given ``weights``.
+
+        Parameters
+        ----------
+        geom : `~gammapy.maps.Geom`
+            Target Map geometry
+        weights : `~numpy.ndarray`
+            Weights vector. Default is weight of one.
+        preserve_counts : bool
+            Preserve the integral over each bin.  This should be true
+            if the map is an integral quantity (e.g. counts) and false if
+            the map is a differential quantity (e.g. intensity)
+        """
+        coords = geom.get_coord()
+        idx = self.geom.coord_to_idx(coords)
+        return self._resample_by_idx(
+            idx, weights=weights, preserve_counts=preserve_counts
+        )
+
+    @abc.abstractmethod
+    def _resample_by_idx(self, idx, weights=None, preserve_counts=False):
+        """Resample pixels at ``idx`` with given ``weights``.
+
+        Parameters
+        ----------
+        idx : tuple
+            Tuple of pixel index arrays for each dimension of the map.
+            Tuple should be ordered as (I_lon, I_lat, I_0, ..., I_n)
+            for WCS maps and (I_hpx, I_0, ..., I_n) for HEALPix maps.
+        weights : `~numpy.ndarray`
+            Weights vector. Default is weight of one.
+        preserve_counts : bool
+            Preserve the integral over each bin.  This should be true
+            if the map is an integral quantity (e.g. counts) and false if
+            the map is a differential quantity (e.g. intensity)
+        """
+        pass
+
     def fill_events(self, events):
         """Fill event coordinates (`~gammapy.data.EventList`)."""
         self.fill_by_coord(events.map_coord(self.geom))
@@ -939,8 +1099,6 @@ class Map(abc.ABC):
         axes : `~numpy.ndarray` of `~matplotlib.pyplot.Axes`
             Axes grid
         """
-        import matplotlib.pyplot as plt
-
         if len(self.geom.axes) > 1:
             raise ValueError("Grid plotting is only supported for one non spatial axis")
 
@@ -976,17 +1134,26 @@ class Map(abc.ABC):
                 continue
 
             if image.geom.is_hpx:
-                image_wcs = (image.to_wcs(normalize=False, proj="AIT", oversample=2,),)
+                image_wcs = image.to_wcs(
+                    normalize=False,
+                    proj="AIT",
+                    oversample=2,
+                )
             else:
                 image_wcs = image
 
             image_wcs.plot(ax=ax, **kwargs)
 
             if axis.node_type == "center":
-                info = f"{axis.center[idx]:.1f}"
+                if axis.name == "energy" or axis.name == "energy_true":
+                    info = energy_unit_format(axis.center[idx])
+                else:
+                    info = f"{axis.center[idx]:.1f}"
             else:
-                info = f"{axis.edges[idx]:.1f} - {axis.edges[idx + 1]:.1f} "
-
+                if axis.name == "energy" or axis.name == "energy_true":
+                    info = f"{energy_unit_format(axis.edges[idx])} - {energy_unit_format(axis.edges[idx+1])}"
+                else:
+                    info = f"{axis.edges[idx]:.1f} - {axis.edges[idx + 1]:.1f} "
             ax.set_title(f"{axis.name.capitalize()} " + info)
             lon, lat = ax.coords[0], ax.coords[1]
             lon.set_ticks_position("b")
@@ -1030,9 +1197,8 @@ class Map(abc.ABC):
             m.plot_interactive(rc_params=rc_params)
         """
         import matplotlib as mpl
-        import matplotlib.pyplot as plt
-        from ipywidgets.widgets.interaction import interact, fixed
-        from ipywidgets import SelectionSlider, RadioButtons
+        from ipywidgets import RadioButtons, SelectionSlider
+        from ipywidgets.widgets.interaction import fixed, interact
 
         if self.geom.is_image:
             raise TypeError("Use .plot() for 2D Maps")
@@ -1047,14 +1213,16 @@ class Map(abc.ABC):
         interact_kwargs = {}
 
         for axis in self.geom.axes:
-            if axis.node_type == "edges":
-                options = [
-                    f"{val_min:.2e} - {val_max:.2e} {axis.unit}"
-                    for val_min, val_max in zip(axis.edges[:-1], axis.edges[1:])
-                ]
+            if axis.node_type == "center":
+                if axis.name == "energy" or axis.name == "energy_true":
+                    options = energy_unit_format(axis.center)
+                else:
+                    options = axis.as_plot_labels
+            elif axis.name == "energy" or axis.name == "energy_true":
+                E = energy_unit_format(axis.edges)
+                options = [f"{E[i]} - {E[i+1]}" for i in range(len(E) - 1)]
             else:
-                options = [f"{val:.2e} {axis.unit}" for val in axis.center]
-
+                options = axis.as_plot_labels
             interact_kwargs[axis.name] = SelectionSlider(
                 options=options,
                 description=f"Select {axis.name}:",
@@ -1080,7 +1248,7 @@ class Map(abc.ABC):
             img = self.get_image_by_idx(idx)
             stretch = ikwargs["stretch"]
             with mpl.rc_context(rc=rc_params):
-                fig, ax, cbar = img.plot(stretch=stretch, **kwargs)
+                img.plot(stretch=stretch, **kwargs)
                 plt.show()
 
     def copy(self, **kwargs):
@@ -1097,7 +1265,14 @@ class Map(abc.ABC):
             Copied Map.
         """
         if "geom" in kwargs:
-            raise ValueError("Can't copy and change geometry of the map.")
+            geom = kwargs["geom"]
+            if not geom.data_shape == self.geom.data_shape:
+                raise ValueError(
+                    "Can't copy and change data size of the map. "
+                    f" Current shape {self.geom.data_shape},"
+                    f" requested shape {geom.data_shape}"
+                )
+
         return self._init_copy(**kwargs)
 
     def apply_edisp(self, edisp):
@@ -1119,7 +1294,7 @@ class Map(abc.ABC):
             data = np.rollaxis(self.data, loc, len(self.data.shape))
             data = np.dot(data, edisp.pdf_matrix)
             data = np.rollaxis(data, -1, loc)
-            energy_axis = edisp.energy_axis.copy(name="energy")
+            energy_axis = edisp.axes["energy"].copy(name="energy")
         else:
             data = self.data
             energy_axis = self.geom.axes["energy_true"].copy(name="energy")
@@ -1127,6 +1302,30 @@ class Map(abc.ABC):
         geom = self.geom.to_image().to_cube(axes=[energy_axis])
         return self._init_copy(geom=geom, data=data)
 
+    def mask_nearest_position(self, position):
+        """Given a sky coordinate return nearest valid position in the mask
+
+        If the mask contains additional axes, the mask is reduced over those.
+
+        Parameters
+        ----------
+        position : `~astropy.coordinates.SkyCoord`
+            Test position
+
+        Returns
+        -------
+        position : `~astropy.coordinates.SkyCoord`
+            Nearest position in the mask
+        """
+        if not self.geom.is_image:
+            raise ValueError("Method only supported for 2D images")
+
+        coords = self.geom.to_image().get_coord().skycoord
+        separation = coords.separation(position)
+        separation[~self.data] = np.inf
+        idx = np.argmin(separation)
+        return coords.flatten()[idx]
+
     def sum_over_axes(self, axes_names=None, keepdims=True, weights=None):
         """To sum map values over all non-spatial axes.
 
@@ -1218,48 +1417,152 @@ class Map(abc.ABC):
         data = func.reduce(data, axis=idx, keepdims=keepdims, where=~np.isnan(data))
         return self._init_copy(geom=geom, data=data)
 
-    @classmethod
-    def from_images(cls, images, axis=None):
-        """Create Map from list of images and a non-spatial axis.
-
-        If the images have a non-spatial axis of length 1 a new axes is generated
-        from by merging the individual axes. The image geometries must be aligned.
+    def cumsum(self, axis_name):
+        """Compute cumulative sum along a given axis
 
         Parameters
         ----------
-        images : list of `Map` objects
-            Images
+        axis_name : str
+            Along which axis to integrate.
+
+        Returns
+        -------
+        cumsum : `Map`
+            Map with cumulative sum
+        """
+        axis = self.geom.axes[axis_name]
+        axis_idx = self.geom.axes.index_data(axis_name)
+
+        # TODO: the broadcasting should be done by axis.center, axis.bin_width etc.
+        shape = [1] * len(self.geom.data_shape)
+        shape[axis_idx] = -1
+
+        values = self.quantity * axis.bin_width.reshape(shape)
+
+        if axis_name == "rad":
+            # take Jacobian into account
+            values = 2 * np.pi * axis.center.reshape(shape) * values
+
+        data = np.insert(values.cumsum(axis=axis_idx), 0, 0, axis=axis_idx)
+
+        axis_shifted = MapAxis.from_nodes(
+            axis.edges, name=axis.name, interp=axis.interp
+        )
+        axes = self.geom.axes.replace(axis_shifted)
+        geom = self.geom.to_image().to_cube(axes)
+        return self.__class__(geom=geom, data=data.value, unit=data.unit)
+
+    def integral(self, axis_name, coords, **kwargs):
+        """Compute integral along a given axis
+
+        This method uses interpolation of the cumulative sum.
+
+        Parameters
+        ----------
+        axis_name : str
+            Along which axis to integrate.
+        coords : dict or `MapCoord`
+            Map coordinates
+
+        **kwargs : dict
+            Coordinates at which to evaluate the IRF
+
+        Returns
+        -------
+        array : `~astropy.units.Quantity`
+            Returns 2D array with axes offset
+        """
+        cumsum = self.cumsum(axis_name=axis_name)
+        cumsum = cumsum.pad(pad_width=1, axis_name=axis_name, mode="edge")
+        return u.Quantity(
+            cumsum.interp_by_coord(coords, **kwargs), cumsum.unit, copy=False
+        )
+
+    def normalize(self, axis_name=None):
+        """Normalise data in place along a given axis.
+
+        Parameters
+        ----------
+        axis_name : str
+            Along which axis to normalize.
+
+        """
+        cumsum = self.cumsum(axis_name=axis_name).quantity
+
+        with np.errstate(invalid="ignore", divide="ignore"):
+            axis = self.geom.axes.index_data(axis_name=axis_name)
+            normed = self.quantity / cumsum.max(axis=axis, keepdims=True)
+
+        self.quantity = np.nan_to_num(normed)
+
+    @classmethod
+    def from_stack(cls, maps, axis=None, axis_name=None):
+        """Create Map from list of images and a non-spatial axis.
+
+        The image geometries must be aligned, except for the axis that is stacked.
+
+        Parameters
+        ----------
+        maps : list of `Map` objects
+            List of maps
         axis : `MapAxis`
-            Map axis
+            If a `MapAxis` is provided the maps are stacked along the last data
+            axis and the new axis is introduced.
+        axis_name : str
+            If an axis name is as string the given the maps are stacked along
+            the given axis name.
 
         Returns
         -------
         map : `Map`
             Map with additional non-spatial axis.
-
         """
-        geom_ref = images[0].geom.to_image()
+        geom = maps[0].geom
+
+        if axis_name is None and axis is None:
+            axis_name = geom.axes.names[-1]
+
+        if axis_name:
+            axis = MapAxis.from_stack(axes=[m.geom.axes[axis_name] for m in maps])
+            geom = geom.drop(axis_name=axis_name)
 
         data = []
 
-        for image in images:
-            if not image.geom.to_image() == geom_ref:
-                raise ValueError("Image geometries not aligned")
-            data.append(image.data)
+        for m in maps:
+            if axis_name:
+                m_geom = m.geom.drop(axis_name=axis_name)
+            else:
+                m_geom = m.geom
 
-        if axis is None:
-            try:
-                axis = MapAxis.from_stack(axes=[image.geom.axes[0] for image in images])
-            except IndexError:
-                ValueError(
-                    "Images don't have a non-spatial axis. Please provide"
-                    " the axis separately"
-                )
+            if not m_geom == geom:
+                raise ValueError(f"Image geometries not aligned: {m.geom} and {geom}")
+
+            data.append(m.quantity.to_value(maps[0].unit))
 
         return cls.from_geom(
-            data=np.stack(data), geom=geom_ref.to_cube(axes=[axis]), unit=images[0].unit
+            data=np.stack(data), geom=geom.to_cube(axes=[axis]), unit=maps[0].unit
         )
 
+    def split_by_axis(self, axis_name):
+        """Split a Map along an axis into multiple maps.
+
+        Parameters
+        ----------
+        axis_name : str
+            Name of the axis to split
+
+        Returns
+        -------
+        maps : list
+            A list of `~gammapy.maps.Map`
+        """
+        maps = []
+        axis = self.geom.axes[axis_name]
+        for idx in range(axis.nbin):
+            m = self.slice_by_idx({axis_name: idx})
+            maps.append(m)
+        return maps
+
     def to_cube(self, axes):
         """Append non-spatial axes to create a higher-dimensional Map.
 
@@ -1286,6 +1589,78 @@ class Map(abc.ABC):
         data = self.data.reshape((1,) * len(axes) + self.data.shape)
         return self.from_geom(data=data, geom=geom, unit=self.unit)
 
+    def get_spectrum(self, region=None, func=np.nansum, weights=None):
+        """Extract spectrum in a given region.
+
+        The spectrum can be computed by summing (or, more generally, applying ``func``)
+        along the spatial axes in each energy bin. This occurs only inside the ``region``,
+        which by default is assumed to be the whole spatial extension of the map.
+
+        Parameters
+        ----------
+        region: `~regions.Region`
+             Region (pixel or sky regions accepted).
+        func : numpy.func
+            Function to reduce the data. Default is np.nansum.
+            For a boolean Map, use np.any or np.all.
+        weights : `WcsNDMap`
+            Array to be used as weights. The geometry must be equivalent.
+
+        Returns
+        -------
+        spectrum : `~gammapy.maps.RegionNDMap`
+            Spectrum in the given region.
+        """
+        if not self.geom.has_energy_axis:
+            raise ValueError("Energy axis required")
+
+        return self.to_region_nd_map(region=region, func=func, weights=weights)
+
+    def to_unit(self, unit):
+        """Convert map to different unit
+
+        Parameters
+        ----------
+        unit : `~astropy.unit.Unit` or str
+            New unit
+
+        Returns
+        -------
+        map : `Map`
+            Map with new unit and converted data
+        """
+        data = self.quantity.to_value(unit)
+        return self.from_geom(self.geom, data=data, unit=unit)
+
+    def is_allclose(self, other, rtol_axes=1e-3, atol_axes=1e-6, **kwargs):
+        """Compare two Maps for close equivalency
+
+        Parameters
+        ----------
+        other : `gammapy.maps.Map`
+            The Map to compare against
+        rtol_axes : float
+            Relative tolerance for the axes comparison.
+        atol_axes : float
+            Relative tolerance for the axes comparison.
+        **kwargs : dict
+                keywords passed to `numpy.allclose`
+
+        Returns
+        -------
+        is_allclose : bool
+            Whether the Map is all close.
+        """
+        if not isinstance(other, self.__class__):
+            return TypeError(f"Cannot compare {type(self)} and {type(other)}")
+
+        if self.data.shape != other.data.shape:
+            return False
+
+        axes_eq = self.axes.is_allclose(other.axes, rtol=rtol_axes, atol=atol_axes)
+        data_eq = np.allclose(self.quantity, other.quantity, **kwargs)
+        return axes_eq and data_eq
+
     def __repr__(self):
         geom = self.geom.__class__.__name__
         axes = ["skycoord"] if self.geom.is_hpx else ["lon", "lat"]
@@ -1302,12 +1677,12 @@ class Map(abc.ABC):
         )
 
     def _arithmetics(self, operator, other, copy):
-        """Perform arithmetics on maps after checking geometry consistency."""
+        """Perform arithmetic on maps after checking geometry consistency."""
         if isinstance(other, Map):
             if self.geom == other.geom:
                 q = other.quantity
             else:
-                raise ValueError("Map Arithmetics: Inconsistent geometries.")
+                raise ValueError("Map Arithmetic: Inconsistent geometries.")
         else:
             q = u.Quantity(other, copy=False)
 
@@ -1316,7 +1691,7 @@ class Map(abc.ABC):
         return out
 
     def _boolean_arithmetics(self, operator, other, copy):
-        """Perform arithmetics on maps after checking geometry consistency."""
+        """Perform arithmetic on maps after checking geometry consistency."""
         if operator == np.logical_not:
             out = self.copy()
             out.data = operator(out.data)
@@ -1326,7 +1701,7 @@ class Map(abc.ABC):
             if self.geom == other.geom:
                 other = other.data
             else:
-                raise ValueError("Map Arithmetics: Inconsistent geometries.")
+                raise ValueError("Map Arithmetic: Inconsistent geometries.")
 
         out = self.copy() if copy else self
         out.data = operator(out.data, other)
