@@ -5,10 +5,12 @@ from numpy.testing import assert_allclose, assert_equal
 import astropy.units as u
 from astropy.table import Table
 from astropy.time import Time
+from astropy.visualization import quantity_support
+import matplotlib.pyplot as plt
 from gammapy.data import GTI
 from gammapy.maps import LabelMapAxis, MapAxes, MapAxis, RegionNDMap, TimeMapAxis
 from gammapy.utils.scripts import make_path
-from gammapy.utils.testing import assert_time_allclose, requires_data
+from gammapy.utils.testing import assert_time_allclose, mpl_plot_check, requires_data
 from gammapy.utils.time import time_ref_to_dict
 
 MAP_AXIS_INTERP = [
@@ -174,6 +176,15 @@ def test_up_downsample_consistency(factor):
     assert_allclose(axis.edges, axis_new.edges)
 
 
+def test_one_bin_nodes():
+    axis = MapAxis.from_nodes([1], name="test", unit="deg")
+
+    assert_allclose(axis.center, 1 * u.deg)
+    assert_allclose(axis.coord_to_pix(1 * u.deg), 0)
+    assert_allclose(axis.coord_to_pix(2 * u.deg), 0)
+    assert_allclose(axis.pix_to_coord(0), 1 * u.deg)
+
+
 def test_group_table_basic(energy_axis_ref):
     energy_edges = [1, 2, 10] * u.TeV
 
@@ -241,11 +252,6 @@ def test_group_table_outside_range(energy_axis_ref):
         energy_axis_ref.group_table(energy_edges)
 
 
-def test_map_axis_single_bin():
-    with pytest.raises(ValueError):
-        _ = MapAxis.from_nodes([1])
-
-
 def test_map_axis_aligned():
     ax1 = MapAxis([1, 2, 3], interp="lin", node_type="edges")
     ax2 = MapAxis([1.5, 2.5], interp="log", node_type="center")
@@ -600,6 +606,31 @@ def test_axes_basics():
 
     assert axes.primary_axis.name == "time"
 
+    new_axes = axes.copy()
+    assert new_axes[0] == new_axes[0]
+    assert new_axes[1] == new_axes[1]
+    assert new_axes == axes
+
+    energy_axis = MapAxis.from_energy_edges([1, 4] * u.TeV)
+    new_axes = MapAxes([energy_axis, time_axis.copy()])
+    assert new_axes != axes
+
+
+def test_axes_getitem():
+    axis1 = MapAxis.from_bounds(1, 4, 3, name="a1")
+    axis2 = axis1.copy(name="a2")
+    axis3 = axis1.copy(name="a3")
+    axes = MapAxes([axis1, axis2, axis3])
+
+    assert isinstance(axes[0], MapAxis)
+    assert axes[-1].name == "a3"
+    assert isinstance(axes[1:], MapAxes)
+    assert len(axes[1:]) == 2
+    assert isinstance(axes[0:1], MapAxes)
+    assert len(axes[0:1]) == 1
+    assert isinstance(axes[["a3", "a1"]], MapAxes)
+    assert axes[["a3", "a1"]][0].name == "a3"
+
 
 def test_label_map_axis_basics():
     axis = LabelMapAxis(labels=["label-1", "label-2"], name="label-axis")
@@ -675,5 +706,36 @@ def test_mixed_axes():
 
     table = Table.read(hdu)
 
-    assert table["LABEL"].dtype == np.dtype("<U7")
+    assert table["LABEL"].dtype == np.dtype("U7")
     assert len(table) == 24
+
+
+def test_map_axis_format_plot_xaxis():
+    axis = MapAxis.from_energy_bounds(
+        "0.03 TeV", "300 TeV", nbin=20, per_decade=True, name="energy_true"
+    )
+
+    with mpl_plot_check():
+        ax = plt.gca()
+        with quantity_support():
+            ax.plot(axis.center, np.ones_like(axis.center))
+
+    ax1 = axis.format_plot_xaxis(ax=ax)
+    assert ax1.xaxis.label.properties()["text"] == "True Energy [TeV]"
+
+
+def test_time_map_axis_format_plot_xaxis(time_intervals):
+    axis = TimeMapAxis(
+        time_intervals["t_min"],
+        time_intervals["t_max"],
+        time_intervals["t_ref"],
+        name="time",
+    )
+
+    with mpl_plot_check():
+        ax = plt.gca()
+        with quantity_support():
+            ax.plot(axis.center, np.ones_like(axis.center))
+
+    ax1 = axis.format_plot_xaxis(ax=ax)
+    assert ax1.xaxis.label.properties()["text"] == "Time [iso]"
