@@ -17,6 +17,7 @@ from regions import (
     RectangleSkyRegion,
     SkyRegion,
 )
+import matplotlib.pyplot as plt
 from gammapy.utils.interpolation import ScaledRegularGridInterpolator
 from gammapy.utils.random import InverseCDFSampler, get_random_state
 from gammapy.utils.units import unit_from_fits_image_hdu
@@ -128,7 +129,7 @@ class WcsNDMap(WcsMap):
         idx = pix_tuple_to_idx(idx)
         return self.data.T[idx]
 
-    def interp_by_coord(self, coords, method="nearest", fill_value=None):
+    def interp_by_coord(self, coords, method="linear", fill_value=None):
 
         if self.geom.is_regular:
             pix = self.geom.coord_to_pix(coords)
@@ -136,7 +137,7 @@ class WcsNDMap(WcsMap):
         else:
             return self._interp_by_coord_griddata(coords, method=method)
 
-    def interp_by_pix(self, pix, method="nearest", fill_value=None):
+    def interp_by_pix(self, pix, method="linear", fill_value=None):
         if not self.geom.is_regular:
             raise ValueError("interp_by_pix only supported for regular geom.")
 
@@ -149,9 +150,20 @@ class WcsNDMap(WcsMap):
             data = self.data.T
 
         fn = ScaledRegularGridInterpolator(
-            grid_pix, data, fill_value=fill_value, bounds_error=False, method=method
+            grid_pix, data, fill_value=None, bounds_error=False, method=method
         )
-        return fn(tuple(pix), clip=False)
+        interp_data = fn(tuple(pix), clip=False)
+
+        if fill_value is not None:
+            idxs = self.geom.pix_to_idx(pix, clip=False)
+            invalid = np.broadcast_arrays(*[idx == -1 for idx in idxs])
+            mask = np.any(invalid, axis=0)
+            if not interp_data.shape:
+                mask = mask.squeeze()
+            interp_data[mask] = fill_value
+            interp_data[~np.isfinite(interp_data)] = fill_value
+
+        return interp_data
 
     def _interp_by_coord_griddata(self, coords, method="linear"):
         grid_coords = self.geom.get_coord()
@@ -173,7 +185,7 @@ class WcsNDMap(WcsMap):
 
         return vals
 
-    def fill_by_idx(self, idx, weights=None):
+    def _resample_by_idx(self, idx, weights=None, preserve_counts=False):
         idx = pix_tuple_to_idx(idx)
         msk = np.all(np.stack([t != INVALID_INDEX.int for t in idx]), axis=0)
         idx = [t[msk] for t in idx]
@@ -186,8 +198,13 @@ class WcsNDMap(WcsMap):
         idx = np.ravel_multi_index(idx, self.data.T.shape)
         idx, idx_inv = np.unique(idx, return_inverse=True)
         weights = np.bincount(idx_inv, weights=weights).astype(self.data.dtype)
+        if not preserve_counts:
+            weights /= np.bincount(idx_inv).astype(self.data.dtype)
         self.data.T.flat[idx] += weights
 
+    def fill_by_idx(self, idx, weights=None):
+        return self._resample_by_idx(idx, weights=weights, preserve_counts=True)
+
     def set_by_idx(self, idx, vals):
         idx = pix_tuple_to_idx(idx)
         self.data.T[idx] = vals
@@ -342,7 +359,6 @@ class WcsNDMap(WcsMap):
             WCS axis object
         """
         from astropy.visualization import simple_norm
-        import matplotlib.pyplot as plt
 
         if not self.geom.is_flat:
             raise TypeError("Use .plot_interactive() for Map dimension > 2")
@@ -365,7 +381,7 @@ class WcsNDMap(WcsMap):
         mask = np.isfinite(data)
 
         if mask.any():
-            min_cut, max_cut = kwargs.pop("vmin", None), kwargs.pop("vmin", None)
+            min_cut, max_cut = kwargs.pop("vmin", None), kwargs.pop("vmax", None)
             norm = simple_norm(data[mask], stretch, min_cut=min_cut, max_cut=max_cut)
             kwargs.setdefault("norm", norm)
 
@@ -426,7 +442,6 @@ class WcsNDMap(WcsMap):
 
     def _plot_default_axes(self, ax):
         from astropy.visualization.wcsaxes.frame import EllipticalFrame
-        import matplotlib.pyplot as plt
 
         if ax is None:
             fig = plt.gcf()
@@ -711,7 +726,7 @@ class WcsNDMap(WcsMap):
                     self.data.astype(np.float32), kernel[idx], method=method, mode=mode
                 )
         else:
-            for img, idx in self.iter_by_image():
+            for img, idx in self.iter_by_image_data():
                 ikern = Ellipsis if kernel.ndim == 2 else idx
                 data[idx] = convolve(
                     img.astype(np.float32), kernel[ikern], method=method, mode=mode
@@ -750,7 +765,7 @@ class WcsNDMap(WcsMap):
 
         smoothed_data = np.empty(self.data.shape, dtype=float)
 
-        for img, idx in self.iter_by_image():
+        for img, idx in self.iter_by_image_data():
             img = img.astype(float)
             if kernel == "gauss":
                 data = ndi.gaussian_filter(img, width, **kwargs)
