@@ -76,7 +76,7 @@ class Fit:
         Global backend used for fitting, default : minuit
     optimize_opts : dict
         Keyword arguments passed to the optimizer. For the `"minuit"` backend
-        see https://iminuit.readthedocs.io/en/latest/api.html#iminuit.Minuit
+        see https://iminuit.readthedocs.io/en/stable/reference.html#iminuit.Minuit
         for a detailed description of the available options. If there is an entry
         'migrad_opts', those options will be passed to `iminuit.Minuit.migrad()`.
 
@@ -159,7 +159,7 @@ class Fit:
 
         if self.backend not in registry.register["covariance"]:
             log.warning("No covariance estimate - not supported by this backend.")
-            return optimize_result
+            return FitResult(optimize_result=optimize_result)
 
         covariance_result = self.covariance(datasets=datasets)
 
@@ -184,6 +184,9 @@ class Fit:
         datasets, parameters = self._parse_datasets(datasets=datasets)
         datasets.parameters.check_limits()
 
+        if len(parameters.free_parameters.names) == 0:
+            raise ValueError("No free parameters for fitting")
+
         parameters.autoscale()
 
         kwargs = self.optimize_opts.copy()
@@ -217,8 +220,9 @@ class Fit:
         # Copy final results into the parameters object
         parameters.set_parameter_factors(factors)
         parameters.check_limits()
+
         return OptimizeResult(
-            parameters=parameters,
+            models=datasets.models.copy(),
             total_stat=datasets.stat_sum(),
             backend=backend,
             method=kwargs.get("method", backend),
@@ -241,21 +245,22 @@ class Fit:
         result : `CovarianceResult`
             Results
         """
-        datasets, parameters = self._parse_datasets(datasets=datasets)
+        datasets, unique_pars = self._parse_datasets(datasets=datasets)
+        parameters = datasets.models.parameters
 
         kwargs = self.covariance_opts.copy()
         kwargs["minuit"] = self.minuit
         backend = kwargs.pop("backend", self.backend)
         compute = registry.get("covariance", backend)
 
-        with parameters.restore_status():
+        with unique_pars.restore_status():
             if self.backend == "minuit":
                 method = "hesse"
             else:
                 method = ""
 
             factor_matrix, info = compute(
-                parameters=parameters, function=datasets.stat_sum, **kwargs
+                parameters=unique_pars, function=datasets.stat_sum, **kwargs
             )
 
             datasets.models.covariance = Covariance.from_factor_matrix(
@@ -268,6 +273,7 @@ class Fit:
             method=method,
             success=info["success"],
             message=info["message"],
+            matrix=datasets.models.covariance.data.copy(),
         )
 
     def confidence(self, datasets, parameter, sigma=1, reoptimize=True):
@@ -524,14 +530,21 @@ class FitStepResult:
 class CovarianceResult(FitStepResult):
     """Covariance result object."""
 
-    pass
+    def __init__(self, matrix=None, **kwargs):
+        self._matrix = matrix
+        super().__init__(**kwargs)
+
+    @property
+    def matrix(self):
+        """Covariance matrix (`~numpy.ndarray`)"""
+        return self._matrix
 
 
 class OptimizeResult(FitStepResult):
     """Optimize result object."""
 
-    def __init__(self, parameters, nfev, total_stat, trace, **kwargs):
-        self._parameters = parameters
+    def __init__(self, models, nfev, total_stat, trace, **kwargs):
+        self._models = models
         self._nfev = nfev
         self._total_stat = total_stat
         self._trace = trace
@@ -540,7 +553,12 @@ class OptimizeResult(FitStepResult):
     @property
     def parameters(self):
         """Best fit parameters"""
-        return self._parameters
+        return self.models.parameters
+
+    @property
+    def models(self):
+        """Best fit models"""
+        return self._models
 
     @property
     def trace(self):
@@ -577,6 +595,10 @@ class FitResult:
 
     def __init__(self, optimize_result=None, covariance_result=None):
         self._optimize_result = optimize_result
+
+        if covariance_result:
+            self.optimize_result.models.covariance = covariance_result.matrix
+
         self._covariance_result = covariance_result
 
     # TODO: is the convenience access needed?
@@ -585,6 +607,12 @@ class FitResult:
         """Best fit parameters of the optimization step"""
         return self.optimize_result.parameters
 
+    # TODO: is the convenience access needed?
+    @property
+    def models(self):
+        """Best fit parameters of the optimization step"""
+        return self.optimize_result.models
+
     # TODO: is the convenience access needed?
     @property
     def total_stat(self):
@@ -624,7 +652,11 @@ class FitResult:
     @property
     def success(self):
         """Total success flag"""
-        success = self.optimize_result.success and self.covariance_result.success
+        success = self.optimize_result.success
+
+        if self.covariance_result:
+            success &= self.covariance_result.success
+
         return success
 
     @property
@@ -635,7 +667,7 @@ class FitResult:
     @property
     def covariance_result(self):
         """Optimize result"""
-        return self._optimize_result
+        return self._covariance_result
 
     def __repr__(self):
         str_ = ""
