@@ -3,15 +3,16 @@ import pytest
 import numpy as np
 from numpy.testing import assert_allclose
 import astropy.units as u
+from astropy.coordinates import SkyCoord
 from astropy.coordinates.angle_utilities import angular_separation
 from astropy.time import Time
+from regions import CircleSkyRegion
 from gammapy.data.gti import GTI
 from gammapy.datasets.map import MapEvaluator
 from gammapy.irf import EDispKernel, PSFKernel
-from gammapy.maps import Map, MapAxis, WcsGeom
+from gammapy.maps import Map, MapAxis, RegionGeom, WcsGeom
 from gammapy.modeling import Parameter
 from gammapy.modeling.models import (
-    BackgroundModel,
     CompoundSpectralModel,
     ConstantSpectralModel,
     ConstantTemporalModel,
@@ -22,10 +23,11 @@ from gammapy.modeling.models import (
     PowerLawSpectralModel,
     SkyModel,
     SpatialModel,
+    TemplateNPredModel,
     TemplateSpatialModel,
     create_fermi_isotropic_diffuse_model,
 )
-from gammapy.utils.testing import mpl_plot_check, requires_data, requires_dependency
+from gammapy.utils.testing import mpl_plot_check, requires_data
 
 
 @pytest.fixture(scope="session")
@@ -36,6 +38,9 @@ def sky_model():
     spectral_model = PowerLawSpectralModel(
         index=2, amplitude="1e-11 cm-2 s-1 TeV-1", reference="1 TeV"
     )
+    spectral_model.index.error = 0.1
+    spectral_model.amplitude.error = "1e-12 cm-2 s-1 TeV-1"
+
     temporal_model = ConstantTemporalModel()
     return SkyModel(
         spatial_model=spatial_model,
@@ -56,12 +61,16 @@ def gti():
 
 @pytest.fixture(scope="session")
 def diffuse_model():
-    axis = MapAxis.from_nodes([0.1, 100], name="energy_true", unit="TeV", interp="log")
+    axis = MapAxis.from_edges(
+        [0.1, 1, 100], name="energy_true", unit="TeV", interp="log"
+    )
     m = Map.create(
         npix=(4, 3), binsz=2, axes=[axis], unit="cm-2 s-1 MeV-1 sr-1", frame="galactic"
     )
     m.data += 42
-    spatial_model = TemplateSpatialModel(m, normalize=False)
+    spatial_model = TemplateSpatialModel(
+        m, normalize=False, filename="diffuse_test.fits"
+    )
     return SkyModel(PowerLawNormSpectralModel(), spatial_model)
 
 
@@ -94,9 +103,11 @@ def background(geom):
 
 @pytest.fixture(scope="session")
 def edisp(geom, geom_true):
-    e_reco = geom.axes["energy"].edges
-    e_true = geom_true.axes["energy_true"].edges
-    return EDispKernel.from_diagonal_response(energy_true=e_true, energy=e_reco)
+    e_reco = geom.axes["energy"]
+    e_true = geom_true.axes["energy_true"]
+    return EDispKernel.from_diagonal_response(
+        energy_axis_true=e_true, energy_axis=e_reco
+    )
 
 
 @pytest.fixture(scope="session")
@@ -188,13 +199,13 @@ def test_skymodel_addition(sky_model, sky_models, sky_models_2, diffuse_model):
 
 
 def test_background_model(background):
-    bkg1 = BackgroundModel(background)
+    bkg1 = TemplateNPredModel(background)
     bkg1.spectral_model.norm.value = 2.0
     npred1 = bkg1.evaluate()
     assert_allclose(npred1.data[0][0][0], background.data[0][0][0] * 2.0, rtol=1e-3)
     assert_allclose(npred1.data.sum(), background.data.sum() * 2.0, rtol=1e-3)
 
-    bkg2 = BackgroundModel(background)
+    bkg2 = TemplateNPredModel(background)
     bkg2.spectral_model.norm.value = 2.0
     bkg2.spectral_model.tilt.value = 0.2
     bkg2.spectral_model.reference.quantity = "1000 GeV"
@@ -206,7 +217,7 @@ def test_background_model(background):
 
 def test_background_model_io(tmpdir, background):
     filename = str(tmpdir / "test-bkg-file.fits")
-    bkg = BackgroundModel(background, filename=filename)
+    bkg = TemplateNPredModel(background, filename=filename)
     bkg.spectral_model.norm.value = 2.0
     bkg.map.write(filename, overwrite=True)
     bkg_dict = bkg.to_dict()
@@ -218,45 +229,58 @@ def test_background_model_io(tmpdir, background):
     assert bkg_read.filename == filename
 
 
-class TestSkyModels:
-    @staticmethod
-    def test_parameters(sky_models):
-        parnames = [
-            "index",
-            "amplitude",
-            "reference",
-            "lon_0",
-            "lat_0",
-            "sigma",
-            "e",
-            "phi",
-        ] * 2
-        assert sky_models.parameters.names == parnames
+def test_background_model_copy(background):
+    background_copy = background.copy()
+    bkg = TemplateNPredModel(background_copy)
+    bkg.map.data += 1.0
+    assert np.all(
+        background_copy.data == background.data
+    )  # Check that the original map is unchanged
 
-        # Check that model parameters are references to the parts
-        p1 = sky_models.parameters["lon_0"]
-        p2 = sky_models[0].parameters["lon_0"]
-        assert p1 is p2
+    bkg_copy = bkg.copy()
+    bkg_copy.map.data += 1.0
+    assert np.all(
+        bkg_copy.map.data == bkg.map.data
+    )  # Check that the map has now changed
 
-    @staticmethod
-    def test_str(sky_models):
-        assert "Component 0" in str(sky_models)
-        assert "Component 1" in str(sky_models)
 
-    @staticmethod
-    def test_get_item(sky_models):
-        model = sky_models["source-2"]
-        assert model.name == "source-2"
+def test_parameters(sky_models):
+    parnames = [
+        "index",
+        "amplitude",
+        "reference",
+        "lon_0",
+        "lat_0",
+        "sigma",
+        "e",
+        "phi",
+    ] * 2
+    assert sky_models.parameters.names == parnames
 
-        model = sky_models["source-3"]
-        assert model.name == "source-3"
+    # Check that model parameters are references to the parts
+    p1 = sky_models.parameters["lon_0"]
+    p2 = sky_models[0].parameters["lon_0"]
+    assert p1 is p2
 
-        with pytest.raises(ValueError):
-            sky_models["spam"]
 
-    @staticmethod
-    def test_names(sky_models):
-        assert sky_models.names == ["source-2", "source-3"]
+def test_str(sky_models):
+    assert "Component 0" in str(sky_models)
+    assert "Component 1" in str(sky_models)
+
+
+def test_get_item(sky_models):
+    model = sky_models["source-2"]
+    assert model.name == "source-2"
+
+    model = sky_models["source-3"]
+    assert model.name == "source-3"
+
+    with pytest.raises(ValueError):
+        sky_models["spam"]
+
+
+def test_names(sky_models):
+    assert sky_models.names == ["source-2", "source-3"]
 
 
 @requires_data()
@@ -294,7 +318,10 @@ class TestSkyModel:
 
     @staticmethod
     def test_str(sky_model):
-        assert "SkyModel" in str(sky_model)
+        string_model = str(sky_model)
+        model_lines = string_model.splitlines()
+        assert "SkyModel" in string_model
+        assert "2.000   +/-    0.10" in model_lines[8]
 
     @staticmethod
     def test_parameters(sky_model):
@@ -370,6 +397,21 @@ class Test_Template_with_cube:
         assert q.shape == (5, 3, 4)
         assert_allclose(q.value.mean(), 42)
 
+    @staticmethod
+    def test_write(tmpdir, diffuse_model):
+        filename = tmpdir / diffuse_model.spatial_model.filename
+
+        diffuse_model.spatial_model.filename = None
+        with pytest.raises(IOError):
+            diffuse_model.spatial_model.write()
+
+        with pytest.raises(IOError):
+            Models(diffuse_model).to_dict()
+
+        diffuse_model.spatial_model.filename = filename
+        diffuse_model.spatial_model.write(overwrite=False)
+        TemplateSpatialModel.read(filename)
+
     @staticmethod
     @requires_data()
     def test_read():
@@ -433,8 +475,8 @@ class Test_template_cube_MapEvaluator:
         out = diffuse_evaluator.compute_flux()
         assert out.data.shape == (3, 4, 5)
         out = out.quantity.to("cm-2 s-1")
-        assert_allclose(out.value.sum(), 633263.444803, rtol=1e-5)
-        assert_allclose(out.value[0, 0, 0], 1164.656176, rtol=1e-5)
+        assert_allclose(out.value.sum(), 633263.444803, rtol=5e-3)
+        assert_allclose(out.value[0, 0, 0], 1164.656176, rtol=5e-3)
 
     @staticmethod
     def test_apply_psf(diffuse_evaluator):
@@ -442,8 +484,8 @@ class Test_template_cube_MapEvaluator:
         npred = diffuse_evaluator.apply_exposure(flux)
         out = diffuse_evaluator.apply_psf(npred)
         assert out.data.shape == (3, 4, 5)
-        assert_allclose(out.data.sum(), 1.106404e12, rtol=1e-5)
-        assert_allclose(out.data[0, 0, 0], 5.586508e08, rtol=1e-5)
+        assert_allclose(out.data.sum(), 1.106404e12, rtol=5e-3)
+        assert_allclose(out.data[0, 0, 0], 5.586508e08, rtol=5e-3)
 
     @staticmethod
     def test_apply_edisp(diffuse_evaluator):
@@ -451,15 +493,15 @@ class Test_template_cube_MapEvaluator:
         npred = diffuse_evaluator.apply_exposure(flux)
         out = diffuse_evaluator.apply_edisp(npred)
         assert out.data.shape == (2, 4, 5)
-        assert_allclose(out.data.sum(), 1.606345e12, rtol=1e-5)
-        assert_allclose(out.data[0, 0, 0], 1.83018e10, rtol=1e-5)
+        assert_allclose(out.data.sum(), 1.606345e12, rtol=5e-3)
+        assert_allclose(out.data[0, 0, 0], 1.83018e10, rtol=5e-3)
 
     @staticmethod
     def test_compute_npred(diffuse_evaluator):
         out = diffuse_evaluator.compute_npred()
         assert out.data.shape == (2, 4, 5)
-        assert_allclose(out.data.sum(), 1.106403e12, rtol=1e-5)
-        assert_allclose(out.data[0, 0, 0], 8.778828e09, rtol=1e-5)
+        assert_allclose(out.data.sum(), 1.106403e12, rtol=5e-3)
+        assert_allclose(out.data[0, 0, 0], 8.778828e09, rtol=5e-3)
 
 
 class TestSkyModelMapEvaluator:
@@ -493,8 +535,8 @@ class TestSkyModelMapEvaluator:
         npred = evaluator.apply_exposure(flux)
         out = evaluator.apply_psf(npred)
         assert out.data.shape == (3, 4, 5)
-        assert_allclose(out.data.sum(), 3.862314e-06, rtol=1e-5)
-        assert_allclose(out.data[0, 0, 0], 4.126612e-08, rtol=1e-5)
+        assert_allclose(out.data.sum(), 3.862314e-06, rtol=5e-3)
+        assert_allclose(out.data[0, 0, 0], 4.126612e-08, rtol=5e-3)
 
     @staticmethod
     def test_apply_edisp(evaluator):
@@ -509,8 +551,8 @@ class TestSkyModelMapEvaluator:
     def test_compute_npred(evaluator, gti):
         out = evaluator.compute_npred()
         assert out.data.shape == (2, 4, 5)
-        assert_allclose(out.data.sum(), 3.862314e-06, rtol=1e-5)
-        assert_allclose(out.data[0, 0, 0], 6.94503e-08, rtol=1e-5)
+        assert_allclose(out.data.sum(), 3.862314e-06, rtol=5e-3)
+        assert_allclose(out.data[0, 0, 0], 6.94503e-08, rtol=5e-3)
 
 
 def test_sky_point_source():
@@ -581,8 +623,8 @@ class MyCustomGaussianModel(SpatialModel):
     lon_0 = Parameter("lon_0", "0 deg")
     lat_0 = Parameter("lat_0", "0 deg", min=-90, max=90)
 
-    sigma_1TeV = Parameter("sigma_1TeV", "1 deg", min=0)
-    sigma_10TeV = Parameter("sigma_10TeV", "0.5 deg", min=0)
+    sigma_1TeV = Parameter("sigma_1TeV", "0.5 deg", min=0)
+    sigma_10TeV = Parameter("sigma_10TeV", "0.1 deg", min=0)
 
     @staticmethod
     def evaluate(lon, lat, energy, lon_0, lat_0, sigma_1TeV, sigma_10TeV):
@@ -590,11 +632,12 @@ class MyCustomGaussianModel(SpatialModel):
         sigmas = u.Quantity([sigma_1TeV, sigma_10TeV])
         energy_nodes = [1, 10] * u.TeV
         sigma = np.interp(energy, energy_nodes, sigmas)
+        sigma = sigma.to("rad")
 
         sep = angular_separation(lon, lat, lon_0, lat_0)
 
         exponent = -0.5 * (sep / sigma) ** 2
-        norm = 1 / (2 * np.pi * sigma ** 2)
+        norm = 1 / (2 * np.pi * sigma**2)
         return norm * np.exp(exponent)
 
     @property
@@ -603,16 +646,20 @@ class MyCustomGaussianModel(SpatialModel):
         return 5 * self.sigma_1TeV.quantity
 
 
-def test_energy_dependent_model(geom_true):
+def test_energy_dependent_model():
+    axis = MapAxis.from_edges(np.logspace(-1, 1, 4), unit=u.TeV, name="energy_true")
+    geom_true = WcsGeom.create(
+        skydir=(0, 0), binsz="0.1 deg", npix=(50, 50), frame="galactic", axes=[axis]
+    )
+
     spectral_model = PowerLawSpectralModel(amplitude="1e-11 cm-2 s-1 TeV-1")
     spatial_model = MyCustomGaussianModel(frame="galactic")
     sky_model = SkyModel(spectral_model=spectral_model, spatial_model=spatial_model)
     model = sky_model.integrate_geom(geom_true)
 
-    assert_allclose(model.data.sum(), 1.678314e-14, rtol=1e-3)
+    assert_allclose(model.data.sum(), 9.9e-11, rtol=1e-3)
 
 
-@requires_dependency("matplotlib")
 def test_plot_grid(geom_true):
     spatial_model = MyCustomGaussianModel(frame="galactic")
     with mpl_plot_check():
@@ -624,3 +671,20 @@ def test_sky_model_create():
     assert isinstance(m.spatial_model, PointSpatialModel)
     assert isinstance(m.spectral_model, PowerLawSpectralModel)
     assert m.name == "my-source"
+
+
+def test_integrate_geom():
+    model = GaussianSpatialModel(lon="0d", lat="0d", sigma=0.1 * u.deg, frame="icrs")
+    spectral_model = PowerLawSpectralModel(amplitude="1e-11 cm-2 s-1 TeV-1")
+    sky_model = SkyModel(spectral_model=spectral_model, spatial_model=model)
+
+    center = SkyCoord("0d", "0d", frame="icrs")
+    radius = 0.3 * u.deg
+    square = CircleSkyRegion(center, radius)
+
+    axis = MapAxis.from_energy_bounds("1 TeV", "10 TeV", nbin=3, name="energy_true")
+    geom = RegionGeom(region=square, axes=[axis], binsz_wcs="0.01deg")
+
+    integral = sky_model.integrate_geom(geom).data
+
+    assert_allclose(integral / 1e-12, [[[5.299]], [[2.460]], [[1.142]]], rtol=1e-3)
