@@ -4,7 +4,8 @@ import pytest
 import numpy as np
 from numpy.testing import assert_allclose
 import astropy.units as u
-from gammapy.maps import MapAxis
+from astropy.table import Table
+from gammapy.maps import MapAxis, RegionNDMap
 from gammapy.modeling.models import (
     SPECTRAL_MODEL_REGISTRY,
     BrokenPowerLawSpectralModel,
@@ -17,15 +18,20 @@ from gammapy.modeling.models import (
     GaussianSpectralModel,
     LogParabolaNormSpectralModel,
     LogParabolaSpectralModel,
+    Model,
     NaimaSpectralModel,
     PiecewiseNormSpectralModel,
     PowerLaw2SpectralModel,
     PowerLawNormSpectralModel,
     PowerLawSpectralModel,
+    SkyModel,
     SmoothBrokenPowerLawSpectralModel,
+    SuperExpCutoffPowerLaw4FGLDR3SpectralModel,
     SuperExpCutoffPowerLaw4FGLSpectralModel,
+    TemplateNDSpectralModel,
     TemplateSpectralModel,
 )
+from gammapy.utils.scripts import make_path
 from gammapy.utils.testing import (
     assert_quantity_allclose,
     mpl_plot_check,
@@ -46,11 +52,18 @@ def table_model():
 
 
 TEST_MODELS = [
+    dict(
+        name="constant",
+        model=ConstantSpectralModel(const=4 / u.cm**2 / u.s / u.TeV),
+        val_at_2TeV=u.Quantity(4, "cm-2 s-1 TeV-1"),
+        integral_1_10TeV=u.Quantity(35.9999999999999, "cm-2 s-1"),
+        eflux_1_10TeV=u.Quantity(198.00000000000006, "TeV cm-2 s-1"),
+    ),
     dict(
         name="powerlaw",
         model=PowerLawSpectralModel(
             index=2.3 * u.Unit(""),
-            amplitude=4 / u.cm ** 2 / u.s / u.TeV,
+            amplitude=4 / u.cm**2 / u.s / u.TeV,
             reference=1 * u.TeV,
         ),
         val_at_2TeV=u.Quantity(4 * 2.0 ** (-2.3), "cm-2 s-1 TeV-1"),
@@ -61,7 +74,7 @@ TEST_MODELS = [
         name="powerlaw",
         model=PowerLawSpectralModel(
             index=2 * u.Unit(""),
-            amplitude=4 / u.cm ** 2 / u.s / u.TeV,
+            amplitude=4 / u.cm**2 / u.s / u.TeV,
             reference=1 * u.TeV,
         ),
         val_at_2TeV=u.Quantity(1.0, "cm-2 s-1 TeV-1"),
@@ -71,7 +84,9 @@ TEST_MODELS = [
     dict(
         name="norm-powerlaw",
         model=PowerLawNormSpectralModel(
-            tilt=2 * u.Unit(""), norm=4.0 * u.Unit(""), reference=1 * u.TeV,
+            tilt=2 * u.Unit(""),
+            norm=4.0 * u.Unit(""),
+            reference=1 * u.TeV,
         ),
         val_at_2TeV=u.Quantity(1.0, ""),
         integral_1_10TeV=u.Quantity(3.6, "TeV"),
@@ -93,7 +108,7 @@ TEST_MODELS = [
         name="ecpl",
         model=ExpCutoffPowerLawSpectralModel(
             index=1.6 * u.Unit(""),
-            amplitude=4 / u.cm ** 2 / u.s / u.TeV,
+            amplitude=4 / u.cm**2 / u.s / u.TeV,
             reference=1 * u.TeV,
             lambda_=0.1 / u.TeV,
         ),
@@ -118,7 +133,7 @@ TEST_MODELS = [
         name="ecpl_3fgl",
         model=ExpCutoffPowerLaw3FGLSpectralModel(
             index=2.3 * u.Unit(""),
-            amplitude=4 / u.cm ** 2 / u.s / u.TeV,
+            amplitude=4 / u.cm**2 / u.s / u.TeV,
             reference=1 * u.TeV,
             ecut=10 * u.TeV,
         ),
@@ -127,11 +142,11 @@ TEST_MODELS = [
         eflux_1_10TeV=u.Quantity(5.340285560055799, "TeV cm-2 s-1"),
     ),
     dict(
-        name="plsec_4fgl",
+        name="plsec_4fgl_dr1",
         model=SuperExpCutoffPowerLaw4FGLSpectralModel(
             index_1=1.5,
             index_2=2,
-            amplitude=1 / u.cm ** 2 / u.s / u.TeV,
+            amplitude=1 / u.cm**2 / u.s / u.TeV,
             reference=1 * u.TeV,
             expfactor=1e-2,
         ),
@@ -139,11 +154,24 @@ TEST_MODELS = [
         integral_1_10TeV=u.Quantity(1.2125247, "cm-2 s-1"),
         eflux_1_10TeV=u.Quantity(3.38072082, "TeV cm-2 s-1"),
     ),
+    dict(
+        name="plsec_4fgl",
+        model=SuperExpCutoffPowerLaw4FGLDR3SpectralModel(
+            index_1=1.5,
+            index_2=2,
+            amplitude=1 / u.cm**2 / u.s / u.TeV,
+            reference=1 * u.TeV,
+            expfactor=1e-2,
+        ),
+        val_at_2TeV=u.Quantity(0.35212994, "cm-2 s-1 TeV-1"),
+        integral_1_10TeV=u.Quantity(1.328499, "cm-2 s-1"),
+        eflux_1_10TeV=u.Quantity(4.067067, "TeV cm-2 s-1"),
+    ),
     dict(
         name="logpar",
         model=LogParabolaSpectralModel(
             alpha=2.3 * u.Unit(""),
-            amplitude=4 / u.cm ** 2 / u.s / u.TeV,
+            amplitude=4 / u.cm**2 / u.s / u.TeV,
             reference=1 * u.TeV,
             beta=0.5 * u.Unit(""),
         ),
@@ -168,7 +196,7 @@ TEST_MODELS = [
         name="logpar10",
         model=LogParabolaSpectralModel.from_log10(
             alpha=2.3 * u.Unit(""),
-            amplitude=4 / u.cm ** 2 / u.s / u.TeV,
+            amplitude=4 / u.cm**2 / u.s / u.TeV,
             reference=1 * u.TeV,
             beta=1.151292546497023 * u.Unit(""),
         ),
@@ -177,18 +205,11 @@ TEST_MODELS = [
         eflux_1_10TeV=u.Quantity(3.9586515834989267, "TeV cm-2 s-1"),
         e_peak=0.74082 * u.TeV,
     ),
-    dict(
-        name="constant",
-        model=ConstantSpectralModel(const=4 / u.cm ** 2 / u.s / u.TeV),
-        val_at_2TeV=u.Quantity(4, "cm-2 s-1 TeV-1"),
-        integral_1_10TeV=u.Quantity(35.9999999999999, "cm-2 s-1"),
-        eflux_1_10TeV=u.Quantity(198.00000000000006, "TeV cm-2 s-1"),
-    ),
     dict(
         name="powerlaw_index1",
         model=PowerLawSpectralModel(
             index=1 * u.Unit(""),
-            amplitude=2 / u.cm ** 2 / u.s / u.TeV,
+            amplitude=2 / u.cm**2 / u.s / u.TeV,
             reference=1 * u.TeV,
         ),
         val_at_2TeV=u.Quantity(1.0, "cm-2 s-1 TeV-1"),
@@ -199,7 +220,7 @@ TEST_MODELS = [
         name="ecpl_2",
         model=ExpCutoffPowerLawSpectralModel(
             index=2.0 * u.Unit(""),
-            amplitude=4 / u.cm ** 2 / u.s / u.TeV,
+            amplitude=4 / u.cm**2 / u.s / u.TeV,
             reference=1 * u.TeV,
             lambda_=0.1 / u.TeV,
         ),
@@ -211,7 +232,7 @@ TEST_MODELS = [
     dict(
         name="GaussianSpectralModel",
         model=GaussianSpectralModel(
-            norm=4 / u.cm ** 2 / u.s, mean=2 * u.TeV, sigma=0.2 * u.TeV
+            amplitude=4 / u.cm**2 / u.s, mean=2 * u.TeV, sigma=0.2 * u.TeV
         ),
         val_at_2TeV=u.Quantity(7.978845608028654, "cm-2 s-1 TeV-1"),
         val_at_3TeV=u.Quantity(2.973439029468601e-05, "cm-2 s-1 TeV-1"),
@@ -223,7 +244,7 @@ TEST_MODELS = [
         name="ecpl",
         model=ExpCutoffPowerLawSpectralModel(
             index=1.8 * u.Unit(""),
-            amplitude=4 / u.cm ** 2 / u.s / u.TeV,
+            amplitude=4 / u.cm**2 / u.s / u.TeV,
             reference=1 * u.TeV,
             lambda_=0.1 / u.TeV,
             alpha=0.8,
@@ -238,7 +259,7 @@ TEST_MODELS = [
         model=BrokenPowerLawSpectralModel(
             index1=1.5 * u.Unit(""),
             index2=2.5 * u.Unit(""),
-            amplitude=4 / u.cm ** 2 / u.s / u.TeV,
+            amplitude=4 / u.cm**2 / u.s / u.TeV,
             ebreak=0.5 * u.TeV,
         ),
         val_at_2TeV=u.Quantity(0.125, "cm-2 s-1 TeV-1"),
@@ -250,7 +271,7 @@ TEST_MODELS = [
         model=SmoothBrokenPowerLawSpectralModel(
             index1=1.5 * u.Unit(""),
             index2=2.5 * u.Unit(""),
-            amplitude=4 / u.cm ** 2 / u.s / u.TeV,
+            amplitude=4 / u.cm**2 / u.s / u.TeV,
             ebreak=0.5 * u.TeV,
             reference=1 * u.TeV,
             beta=1,
@@ -264,7 +285,7 @@ TEST_MODELS = [
         model=SmoothBrokenPowerLawSpectralModel(
             index1=2.5 * u.Unit(""),
             index2=1.5 * u.Unit(""),
-            amplitude=4 / u.cm ** 2 / u.s / u.TeV,
+            amplitude=4 / u.cm**2 / u.s / u.TeV,
             ebreak=0.5 * u.TeV,
             reference=1 * u.TeV,
             beta=1,
@@ -276,7 +297,8 @@ TEST_MODELS = [
     dict(
         name="pbpl",
         model=PiecewiseNormSpectralModel(
-            energy=[1, 3, 7, 10] * u.TeV, norms=[1, 5, 3, 0.5] * u.Unit(""),
+            energy=[1, 3, 7, 10] * u.TeV,
+            norms=[1, 5, 3, 0.5] * u.Unit(""),
         ),
         val_at_2TeV=u.Quantity(2.76058404, ""),
         integral_1_10TeV=u.Quantity(24.758255, "TeV"),
@@ -288,8 +310,8 @@ TEST_MODELS = [
 
 TEST_MODELS.append(
     dict(
-        name="compound3",
-        model=TEST_MODELS[0]["model"] + TEST_MODELS[0]["model"],
+        name="compound6",
+        model=TEST_MODELS[0]["model"] + u.Quantity(4, "cm-2 s-1 TeV-1"),
         val_at_2TeV=TEST_MODELS[0]["val_at_2TeV"] * 2,
         integral_1_10TeV=TEST_MODELS[0]["integral_1_10TeV"] * 2,
         eflux_1_10TeV=TEST_MODELS[0]["eflux_1_10TeV"] * 2,
@@ -298,14 +320,15 @@ TEST_MODELS.append(
 
 TEST_MODELS.append(
     dict(
-        name="compound6",
-        model=TEST_MODELS[11]["model"] + u.Quantity(4, "cm-2 s-1 TeV-1"),
-        val_at_2TeV=TEST_MODELS[11]["val_at_2TeV"] * 2,
-        integral_1_10TeV=TEST_MODELS[11]["integral_1_10TeV"] * 2,
-        eflux_1_10TeV=TEST_MODELS[11]["eflux_1_10TeV"] * 2,
+        name="compound3",
+        model=TEST_MODELS[1]["model"] + TEST_MODELS[1]["model"],
+        val_at_2TeV=TEST_MODELS[1]["val_at_2TeV"] * 2,
+        integral_1_10TeV=TEST_MODELS[1]["integral_1_10TeV"] * 2,
+        eflux_1_10TeV=TEST_MODELS[1]["eflux_1_10TeV"] * 2,
     )
 )
 
+
 TEST_MODELS.append(
     dict(
         name="table_model",
@@ -322,8 +345,14 @@ TEST_MODELS.append(
 @pytest.mark.parametrize("spectrum", TEST_MODELS, ids=lambda _: _["name"])
 def test_models(spectrum):
     model = spectrum["model"]
+
+    for p in model.parameters:
+        assert p.type == "spectral"
+
     energy = 2 * u.TeV
     value = model(energy)
+    energies = [2, 3] * u.TeV
+    values = model(energies)
     assert_quantity_allclose(value, spectrum["val_at_2TeV"], rtol=1e-7)
     if "val_at_3TeV" in spectrum:
         energy = 3 * u.TeV
@@ -354,7 +383,10 @@ def test_models(spectrum):
         or spectrum["name"] == "GaussianSpectralModel"
         or spectrum["name"] == "pbpl"
     ):
-        assert_quantity_allclose(model.inverse(value), 2 * u.TeV, rtol=0.01)
+        assert_quantity_allclose(model.inverse(value), energy, rtol=0.01)
+        inverse = model.inverse_all(values)
+        for ke, ener in enumerate(energies):
+            assert_quantity_allclose(inverse[ke], energies[ke], rtol=0.01)
 
     if "integral_infinity" in spectrum:
         energy_min = 0 * u.TeV
@@ -382,7 +414,6 @@ def test_model_unit():
     assert value.unit == "cm-2 s-1 TeV-1"
 
 
-@requires_dependency("matplotlib")
 def test_model_plot():
     pwl = PowerLawSpectralModel(
         amplitude=1e-12 * u.Unit("TeV-1 cm-2 s-1"), reference=1 * u.Unit("TeV"), index=2
@@ -396,12 +427,46 @@ def test_model_plot():
         pwl.plot_error((1 * u.TeV, 10 * u.TeV))
 
 
+def test_model_plot_sed_type():
+    pwl = PowerLawSpectralModel(
+        amplitude=1e-12 * u.Unit("TeV-1 cm-2 s-1"), reference=1 * u.Unit("TeV"), index=2
+    )
+    pwl.amplitude.error = 0.1e-12 * u.Unit("TeV-1 cm-2 s-1")
+
+    with mpl_plot_check():
+        ax1 = pwl.plot((1 * u.TeV, 100 * u.TeV), sed_type="dnde")
+        ax2 = pwl.plot_error((1 * u.TeV, 100 * u.TeV), sed_type="dnde")
+        assert ax1.axes.axes.get_ylabel() == "dnde [1 / (cm2 s TeV)]"
+        assert ax2.axes.axes.get_ylabel() == "dnde [1 / (cm2 s TeV)]"
+
+    with mpl_plot_check():
+        ax1 = pwl.plot((1 * u.TeV, 100 * u.TeV), sed_type="e2dnde")
+        ax2 = pwl.plot_error((1 * u.TeV, 100 * u.TeV), sed_type="e2dnde")
+        assert ax1.axes.axes.get_ylabel() == "e2dnde [erg / (cm2 s)]"
+        assert ax2.axes.axes.get_ylabel() == "e2dnde [erg / (cm2 s)]"
+
+    with mpl_plot_check():
+        ax1 = pwl.plot((1 * u.TeV, 100 * u.TeV), sed_type="flux")
+        ax2 = pwl.plot_error((1 * u.TeV, 100 * u.TeV), sed_type="flux")
+        assert ax1.axes.axes.get_ylabel() == "flux [1 / (cm2 s)]"
+        assert ax2.axes.axes.get_ylabel() == "flux [1 / (cm2 s)]"
+
+    with mpl_plot_check():
+        ax1 = pwl.plot((1 * u.TeV, 100 * u.TeV), sed_type="eflux")
+        ax2 = pwl.plot_error((1 * u.TeV, 100 * u.TeV), sed_type="eflux")
+        assert ax1.axes.axes.get_ylabel() == "eflux [erg / (cm2 s)]"
+        assert ax2.axes.axes.get_ylabel() == "eflux [erg / (cm2 s)]"
+
+
 def test_to_from_dict():
-    spectrum = TEST_MODELS[0]
+    spectrum = TEST_MODELS[1]
     model = spectrum["model"]
 
     model_dict = model.to_dict()
-    model_class = SPECTRAL_MODEL_REGISTRY.get_cls(model_dict["type"])
+    # Here we reverse the order of parameters list to ensure assignment is correct
+    model_dict["spectral"]["parameters"].reverse()
+
+    model_class = SPECTRAL_MODEL_REGISTRY.get_cls(model_dict["spectral"]["type"])
     new_model = model_class.from_dict(model_dict)
 
     assert isinstance(new_model, PowerLawSpectralModel)
@@ -410,14 +475,57 @@ def test_to_from_dict():
     desired = [par.value for par in model.parameters]
     assert_quantity_allclose(actual, desired)
 
+    actual = [par.frozen for par in new_model.parameters]
+    desired = [par.frozen for par in model.parameters]
+    assert_allclose(actual, desired)
+
+    new_model = Model.from_dict(model_dict)
+
+    assert isinstance(new_model, PowerLawSpectralModel)
+
+    actual = [par.value for par in new_model.parameters]
+    desired = [par.value for par in model.parameters]
+    assert_quantity_allclose(actual, desired)
+
+    actual = [par.frozen for par in new_model.parameters]
+    desired = [par.frozen for par in model.parameters]
+    assert_allclose(actual, desired)
+
+
+def test_to_from_dict_partial_input(caplog):
+    spectrum = TEST_MODELS[1]
+    model = spectrum["model"]
+
+    model_dict = model.to_dict()
+    # Here we remove the reference energy
+    model_dict["spectral"]["parameters"].remove(model_dict["spectral"]["parameters"][2])
+
+    model_class = SPECTRAL_MODEL_REGISTRY.get_cls(model_dict["spectral"]["type"])
+    new_model = model_class.from_dict(model_dict)
+
+    assert isinstance(new_model, PowerLawSpectralModel)
+
+    actual = [par.value for par in new_model.parameters]
+    desired = [par.value for par in model.parameters]
+    assert_quantity_allclose(actual, desired)
+
+    actual = [par.frozen for par in new_model.parameters]
+    desired = [par.frozen for par in model.parameters]
+    assert_allclose(actual, desired)
+    assert "WARNING" in [_.levelname for _ in caplog.records]
+    assert (
+        "Parameter 'reference' not defined in YAML file. Using default value: 1.0 TeV"
+        in [_.message for _ in caplog.records]
+    )
+
 
 def test_to_from_dict_compound():
-    spectrum = TEST_MODELS[-2]
+    spectrum = TEST_MODELS[-3]
     model = spectrum["model"]
     assert spectrum["name"] == "compound6"
     model_dict = model.to_dict()
-    assert model_dict["operator"] == "add"
-    model_class = SPECTRAL_MODEL_REGISTRY.get_cls(model_dict["type"])
+    assert model_dict["spectral"]["operator"] == "add"
+    model_class = SPECTRAL_MODEL_REGISTRY.get_cls(model_dict["spectral"]["type"])
     new_model = model_class.from_dict(model_dict)
 
     assert isinstance(new_model, CompoundSpectralModel)
@@ -427,15 +535,14 @@ def test_to_from_dict_compound():
     assert_quantity_allclose(actual, desired)
 
 
-@requires_dependency("matplotlib")
 @requires_data()
 def test_table_model_from_file():
     filename = "$GAMMAPY_DATA/ebl/ebl_franceschini.fits.gz"
     absorption_z03 = TemplateSpectralModel.read_xspec_model(
         filename=filename, param=0.3
     )
-    with mpl_plot_check():
-        absorption_z03.plot(energy_range=(0.03, 10), energy_unit=u.TeV, flux_unit="")
+    value = absorption_z03(1 * u.TeV)
+    assert_allclose(value, 1)
 
 
 @requires_data()
@@ -500,7 +607,7 @@ def test_pwl_pivot_energy():
     pwl = PowerLawSpectralModel(amplitude="5.35510540e-11 cm-2 s-1 TeV-1")
 
     pwl.covariance = [
-        [0.0318377 ** 2, 6.56889442e-14, 0],
+        [0.0318377**2, 6.56889442e-14, 0],
         [6.56889442e-14, 0, 0],
         [0, 0, 0],
     ]
@@ -508,14 +615,14 @@ def test_pwl_pivot_energy():
     assert_quantity_allclose(pwl.pivot_energy, 3.3540034240210987 * u.TeV)
 
 
-def test_TemplateSpectralModel_evaluate_tiny():
+def test_template_spectral_model_evaluate_tiny():
     energy = np.array([1.00000000e06, 1.25892541e06, 1.58489319e06, 1.99526231e06])
     values = np.array([4.39150790e-38, 1.96639562e-38, 8.80497507e-39, 3.94262401e-39])
 
     model = TemplateSpectralModel(
         energy=energy, values=values * u.Unit("MeV-1 s-1 sr-1")
     )
-    result = model.evaluate(energy)
+    result = model(energy)
     tiny = np.finfo(np.float32).tiny
     mask = abs(values) - tiny > tiny
     np.testing.assert_allclose(
@@ -525,7 +632,17 @@ def test_TemplateSpectralModel_evaluate_tiny():
     assert np.all(result[mask] == 0.0)
 
 
-def test_TemplateSpectralModel_compound():
+def test_template_spectral_model_single_value():
+    energy = [1] * u.TeV
+    values = [1e-12] * u.Unit("TeV-1 s-1 cm-2")
+
+    model = TemplateSpectralModel(energy=energy, values=values)
+    result = model(energy=[0.5, 2] * u.TeV)
+
+    assert_allclose(result.data, 1e-12)
+
+
+def test_template_spectral_model_compound():
     energy = [1.00e06, 1.25e06, 1.58e06, 1.99e06] * u.MeV
     values = [4.39e-7, 1.96e-7, 8.80e-7, 3.94e-7] * u.Unit("MeV-1 s-1 sr-1")
 
@@ -539,8 +656,8 @@ def test_TemplateSpectralModel_compound():
     assert np.allclose(model_mul(energy), 2 * values)
 
     model_dict = model.to_dict()
-    assert model_dict["operator"] == "mul"
-    model_class = SPECTRAL_MODEL_REGISTRY.get_cls(model_dict["type"])
+    assert model_dict["spectral"]["operator"] == "mul"
+    model_class = SPECTRAL_MODEL_REGISTRY.get_cls(model_dict["spectral"]["type"])
     new_model = model_class.from_dict(model_dict)
     assert isinstance(new_model, CompoundSpectralModel)
     assert np.allclose(new_model(energy), 2 * values)
@@ -566,9 +683,11 @@ class TestNaimaModel:
             amplitude=2e33 / u.eV, e_0=10 * u.TeV, alpha=2.5
         )
         radiative_model = naima.radiative.PionDecay(
-            particle_distribution, nh=1 * u.cm ** -3
+            particle_distribution, nh=1 * u.cm**-3
         )
         model = NaimaSpectralModel(radiative_model)
+        for p in model.parameters:
+            assert p._type == "spectral"
 
         val_at_2TeV = 9.725347355450884e-14 * u.Unit("cm-2 s-1 TeV-1")
         integral_1_10TeV = 3.530537143620737e-13 * u.Unit("cm-2 s-1")
@@ -608,6 +727,8 @@ class TestNaimaModel:
         )
 
         model = NaimaSpectralModel(radiative_model)
+        for p in model.parameters:
+            assert p._type == "spectral"
 
         val_at_2TeV = 4.347836316893546e-12 * u.Unit("cm-2 s-1 TeV-1")
         integral_1_10TeV = 1.595813e-11 * u.Unit("cm-2 s-1")
@@ -637,6 +758,8 @@ class TestNaimaModel:
         radiative_model = naima.radiative.Synchrotron(particle_distribution, B=2 * u.G)
 
         model = NaimaSpectralModel(radiative_model)
+        for p in model.parameters:
+            assert p._type == "spectral"
 
         val_at_2TeV = 1.0565840392550432e-24 * u.Unit("cm-2 s-1 TeV-1")
         integral_1_10TeV = 4.449186e-13 * u.Unit("cm-2 s-1")
@@ -679,8 +802,8 @@ class TestNaimaModel:
             ECBPL,
             seed_photon_fields=[
                 "CMB",
-                ["FIR", 70 * u.K, 0.5 * u.eV / u.cm ** 3],
-                ["NIR", 5000 * u.K, 1 * u.eV / u.cm ** 3],
+                ["FIR", 70 * u.K, 0.5 * u.eV / u.cm**3],
+                ["NIR", 5000 * u.K, 1 * u.eV / u.cm**3],
             ],
             Eemax=50 * u.PeV,
             Eemin=0.1 * u.GeV,
@@ -707,7 +830,7 @@ class TestNaimaModel:
             amplitude=2e33 / u.eV, e_0=10 * u.TeV, alpha=2.5
         )
         radiative_model = naima.radiative.PionDecay(
-            particle_distribution, nh=1 * u.cm ** -3
+            particle_distribution, nh=1 * u.cm**-3
         )
         model = NaimaSpectralModel(radiative_model)
 
@@ -716,6 +839,35 @@ class TestNaimaModel:
         with pytest.raises(NotImplementedError):
             NaimaSpectralModel.from_parameters(model.parameters)
 
+    def test_cache(self):
+        import naima
+
+        particle_distribution = naima.models.ExponentialCutoffPowerLaw(
+            1e30 / u.eV, 10 * u.TeV, 3.0, 30 * u.TeV
+        )
+        radiative_model = naima.radiative.InverseCompton(
+            particle_distribution,
+            seed_photon_fields=["CMB", ["FIR", 26.5 * u.K, 0.415 * u.eV / u.cm**3]],
+            Eemin=100 * u.GeV,
+        )
+        model = NaimaSpectralModel(radiative_model, distance=1.5 * u.kpc)
+
+        opts = {
+            "energy_bounds": [10 * u.GeV, 80 * u.TeV],
+            "sed_type": "e2dnde",
+        }
+        model.plot(label="IC (total)", **opts)
+        for seed, ls in zip(["CMB", "FIR"], ["-", "--"]):
+            model = NaimaSpectralModel(radiative_model, seed=seed, distance=1.5 * u.kpc)
+            model.plot(label=f"IC ({seed})", ls=ls, color="gray", **opts)
+
+        skymodel = SkyModel(model)
+        # fail if cache is on :
+        skymodel.spectral_model.plot(
+            energy_bounds=[10 * u.GeV, 80 * u.TeV], energy_power=2
+        )
+        assert radiative_model._memoize == False
+
 
 class TestSpectralModelErrorPropagation:
     """Test spectral model error propagation.
@@ -757,45 +909,44 @@ class TestSpectralModelErrorPropagation:
         assert out.unit == "cm-2 s-1 TeV-1"
         assert_allclose(out.data, [3.760e-11, 3.6193e-12], rtol=1e-3)
 
-    @pytest.mark.xfail(reason="FIXME, do we need this method?")
     def test_integral_error(self):
         out = self.model.integral_error(1 * u.TeV, 10 * u.TeV)
         assert out.unit == "cm-2 s-1"
         assert out.shape == (2,)
         assert_allclose(out.data, [2.197e-11, 2.796e-12], rtol=1e-3)
 
-    @pytest.mark.xfail(reason="FIXME, do we need this method?")
     def test_energy_flux_error(self):
         out = self.model.energy_flux_error(1 * u.TeV, 10 * u.TeV)
         assert out.unit == "TeV cm-2 s-1"
         assert out.shape == (2,)
         assert_allclose(out.data, [4.119e-11, 8.157e-12], rtol=1e-3)
 
-    def test_ecpl_model(self):
-        # Regression test for ECPL model
-        # https://github.com/gammapy/gammapy/issues/2007
-        model = ExpCutoffPowerLawSpectralModel(
-            amplitude=2.076183759227292e-12 * u.Unit("cm-2 s-1 TeV-1"),
-            index=1.8763343736076483,
-            lambda_=0.08703226432146616 * u.Unit("TeV-1"),
-            reference=1 * u.TeV,
-        )
-        model.covariance = [
-            [0.00204191498, -1.507724e-14, 0.0, -0.001834819, 0.0],
-            [-1.507724e-14, 1.6864740e-25, 0.0, 1.854251e-14, 0.0],
-            [0.0, 0.0, 0.0, 0.0, 0.0],
-            [-0.001834819175, 1.8542517e-14, 0.0, 0.0032559101, 0.0],
-            [0.0, 0.0, 0.0, 0.0, 0.0],
-        ]
 
-        out = model.evaluate_error(1 * u.TeV)
-        assert_allclose(out.data, [1.903129e-12, 2.979976e-13], rtol=1e-3)
+def test_dnde_error_ecpl_model():
+    # Regression test for ECPL model
+    # https://github.com/gammapy/gammapy/issues/2007
+    model = ExpCutoffPowerLawSpectralModel(
+        amplitude=2.076183759227292e-12 * u.Unit("cm-2 s-1 TeV-1"),
+        index=1.8763343736076483,
+        lambda_=0.08703226432146616 * u.Unit("TeV-1"),
+        reference=1 * u.TeV,
+    )
+    model.covariance = [
+        [0.00204191498, -1.507724e-14, 0.0, -0.001834819, 0.0],
+        [-1.507724e-14, 1.6864740e-25, 0.0, 1.854251e-14, 0.0],
+        [0.0, 0.0, 0.0, 0.0, 0.0],
+        [-0.001834819175, 1.8542517e-14, 0.0, 0.0032559101, 0.0],
+        [0.0, 0.0, 0.0, 0.0, 0.0],
+    ]
 
-        out = model.evaluate_error(0.1 * u.TeV)
-        assert_allclose(out.data, [1.548176e-10, 1.933612e-11], rtol=1e-3)
+    out = model.evaluate_error(1 * u.TeV)
+    assert_allclose(out.data, [1.903129e-12, 2.979976e-13], rtol=1e-3)
+
+    out = model.evaluate_error(0.1 * u.TeV)
+    assert_allclose(out.data, [1.548176e-10, 1.933612e-11], rtol=1e-3)
 
 
-def test_integral_error_PowerLaw():
+def test_integral_error_power_law():
     energy = np.linspace(1 * u.TeV, 10 * u.TeV, 10)
     energy_min = energy[:-1]
     energy_max = energy[1:]
@@ -806,11 +957,11 @@ def test_integral_error_PowerLaw():
 
     flux, flux_error = powerlaw.integral_error(energy_min, energy_max)
 
-    assert_allclose(flux.value[0] / 1e-13, 5.0, rtol=0.1)
-    assert_allclose(flux_error.value[0] / 1e-14, 8.546615432273905, rtol=0.01)
+    assert_allclose(flux.value[0] / 1e-13, 5.0, rtol=1e-3)
+    assert_allclose(flux_error.value[0] / 1e-14, 7.915984, rtol=1e-3)
 
 
-def test_integral_error_ExpCutOffPowerLaw():
+def test_integral_error_exp_cut_off_power_law():
     energy = np.linspace(1 * u.TeV, 10 * u.TeV, 10)
     energy_min = energy[:-1]
     energy_max = energy[1:]
@@ -823,10 +974,10 @@ def test_integral_error_ExpCutOffPowerLaw():
     flux, flux_error = exppowerlaw.integral_error(energy_min, energy_max)
 
     assert_allclose(flux.value[0] / 1e-13, 5.05855622, rtol=0.01)
-    assert_allclose(flux_error.value[0] / 1e-14, 8.9598, rtol=0.01)
+    assert_allclose(flux_error.value[0] / 1e-14, 8.552617, rtol=0.01)
 
 
-def test_energy_flux_error_power_Law():
+def test_energy_flux_error_power_law():
     energy_min = 1 * u.TeV
     energy_max = 10 * u.TeV
 
@@ -852,3 +1003,119 @@ def test_energy_flux_error_exp_cutoff_power_law():
 
     assert_allclose(enrg_flux.value / 1e-12, 2.788, rtol=0.001)
     assert_allclose(enrg_flux_error.value / 1e-12, 1.419, rtol=0.001)
+
+
+def test_integral_exp_cut_off_power_law_large_number_of_bins():
+    energy = np.geomspace(1, 10, 100) * u.TeV
+    energy_min = energy[:-1]
+    energy_max = energy[1:]
+
+    exppowerlaw = ExpCutoffPowerLawSpectralModel(
+        amplitude="1e-11 TeV-1 cm-2 s-1", index=2
+    )
+    exppowerlaw.parameters["lambda_"].value = 1e-3
+    powerlaw = PowerLawSpectralModel(amplitude="1e-11 TeV-1 cm-2 s-1", index=2)
+    expected_flux = powerlaw.integral(energy_min, energy_max)
+
+    flux = exppowerlaw.integral(energy_min, energy_max)
+
+    assert_allclose(flux.value, expected_flux.value, rtol=0.01)
+
+
+def test_template_ND(tmpdir):
+    energy_axis = MapAxis.from_bounds(
+        1.0, 100, 10, interp="log", name="energy_true", unit="GeV"
+    )
+    norm = MapAxis.from_bounds(0, 10, 10, interp="lin", name="norm", unit="")
+    tilt = MapAxis.from_bounds(-1.0, 1, 5, interp="lin", name="tilt", unit="")
+    region_map = RegionNDMap.create(
+        region="icrs;point(83.63, 22.01)", axes=[energy_axis, norm, tilt]
+    )
+    region_map.data[:, :, :5, 0, 0] = 1
+    region_map.data[:, :, 5:, 0, 0] = 2
+
+    template = TemplateNDSpectralModel(region_map)
+    assert len(template.parameters) == 2
+    assert template.parameters["norm"].value == 5
+    assert template.parameters["tilt"].value == 0
+    assert_allclose(template([1, 100, 1000] * u.GeV), [1.0, 2.0, 2.0])
+
+    template.parameters["norm"].value = 1
+    template.filename = str(tmpdir / "template_ND.fits")
+    template.write()
+    dict_ = template.to_dict()
+    template_new = TemplateNDSpectralModel.from_dict(dict_)
+    assert_allclose(template_new.map.data, region_map.data)
+    assert len(template_new.parameters) == 2
+    assert template_new.parameters["norm"].value == 1
+    assert template_new.parameters["tilt"].value == 0
+
+
+def test_template_ND_no_energy(tmpdir):
+    norm = MapAxis.from_bounds(0, 10, 10, interp="lin", name="norm", unit="")
+    tilt = MapAxis.from_bounds(-1.0, 1, 5, interp="lin", name="tilt", unit="")
+    region_map = RegionNDMap.create(
+        region="icrs;point(83.63, 22.01)", axes=[norm, tilt]
+    )
+    region_map.data[:, :5, 0, 0] = 1
+    region_map.data[:, 5:, 0, 0] = 2
+
+    with pytest.raises(ValueError):
+        TemplateNDSpectralModel(region_map)
+
+
+@requires_data()
+def test_template_ND_EBL(tmpdir):
+
+    # TODO: add RegionNDMap.read(format="xspec")
+    # Create EBL data array
+    filename = "$GAMMAPY_DATA/ebl/ebl_franceschini.fits.gz"
+    filename = make_path(filename)
+    table_param = Table.read(filename, hdu="PARAMETERS")
+    npar = len(table_param)
+    par_axes = []
+    idx_data = []
+    for k in range(npar):
+        name = table_param["NAME"][k].lower().strip()
+        param, idx = np.unique(table_param[0]["VALUE"], return_index=True)
+        par_axes.append(
+            MapAxis(param, node_type="center", interp="lin", name=name, unit="")
+        )
+        idx_data.append(idx)
+    idx_data.append(Ellipsis)
+    idx_data = tuple(idx_data)
+
+    # Get energy values
+    table_energy = Table.read(filename, hdu="ENERGIES")
+    energy_lo = u.Quantity(
+        table_energy["ENERG_LO"], "keV", copy=False
+    )  # unit not stored in file
+    energy_hi = u.Quantity(
+        table_energy["ENERG_HI"], "keV", copy=False
+    )  # unit not stored in file
+    energy = np.sqrt(energy_lo * energy_hi)
+
+    # Get spectrum values
+    table_spectra = Table.read(filename, hdu="SPECTRA")
+
+    energy_axis = MapAxis(energy, node_type="center", interp="log", name="energy_true")
+    region_map = RegionNDMap.create(
+        region="galactic;point(0, 0)", axes=[energy_axis] + par_axes
+    )
+    # TODO: here we use a fake position, is it possible to allow region=None ?
+    data = table_spectra["INTPSPEC"].data[idx_data]
+    region_map.data[:, :, 0, 0] = data
+
+    template = TemplateNDSpectralModel(region_map)
+    assert len(template.parameters) == 1
+    assert_allclose(template.parameters["redshift"].value, 1.001, rtol=1e-3)
+    expected = [9.950501e-01, 4.953951e-01, 1.588062e-06]
+    assert_allclose(template([1, 100, 1000] * u.GeV), expected, rtol=1e-3)
+    template.parameters["redshift"].value = 0.1
+    template.filename = str(tmpdir / "template_ND_ebl_franceschini.fits")
+    template.write()
+    dict_ = template.to_dict()
+    template_new = TemplateNDSpectralModel.from_dict(dict_)
+    assert_allclose(template_new.map.data, region_map.data)
+    assert len(template.parameters) == 1
+    assert_allclose(template.parameters["redshift"].value, 0.1)
