@@ -6,6 +6,7 @@ import itertools
 import logging
 import numpy as np
 from astropy import units as u
+from gammapy.utils.interpolation import interpolation_scale
 from gammapy.utils.table import table_from_row_data
 
 __all__ = ["Parameter", "Parameters"]
@@ -18,17 +19,26 @@ def _get_parameters_str(parameters):
 
     for par in parameters:
         if par.name == "amplitude":
-            line = "\t{:12} {:11}: {:10.2e} {} {:<12s}\n"
+            value_format, error_format = "{:10.2e}", "{:7.1e}"
         else:
-            line = "\t{:12} {:11}: {:7.3f} {} {:<12s}\n"
+            value_format, error_format = "{:10.3f}", "{:7.2f}"
 
-        frozen = "(frozen)" if par.frozen else ""
-        try:
-            error = "+/- {:7.2f}".format(parameters.get_error(par))
-        except AttributeError:
-            error = ""
+        line = "\t{:21} {:8}: " + value_format + "\t {} {:<12s}\n"
 
-        str_ += line.format(par.name, frozen, par.value, error, par.unit)
+        if par._link_label_io is not None:
+            name = par._link_label_io
+        else:
+            name = par.name
+
+        if par.frozen:
+            frozen, error = "(frozen)", "\t\t"
+        else:
+            frozen = ""
+            try:
+                error = "+/- " + error_format.format(par.error)
+            except AttributeError:
+                error = ""
+        str_ += line.format(name, frozen, par.value, error, par.unit)
     return str_.expandtabs(tabsize=2)
 
 
@@ -65,6 +75,24 @@ class Parameter:
         Maximum (sometimes used in fitting)
     frozen : bool, optional
         Frozen? (used in fitting)
+    error : float
+        Parameter error
+    scan_min : float
+        Minimum value for the parameter scan. Overwrites scan_n_sigma.
+    scan_max : float
+        Minimum value for the parameter scan. Overwrites scan_n_sigma.
+    scan_n_values: int
+        Number of values to be used for the parameter scan.
+    scan_n_sigma : int
+        Number of sigmas to scan.
+    scan_values: `numpy.array`
+        Scan values. Overwrites all of the scan keywords before.
+    scale_method : {'scale10', 'factor1', None}
+        Method used to set ``factor`` and ``scale``
+    interp : {"lin", "sqrt", "log"}
+        Parameter scaling to use for the scan.
+    is_norm : bool
+        Whether the parameter represents the flux norm of the model.
     """
 
     def __init__(
@@ -77,14 +105,26 @@ class Parameter:
         max=np.nan,
         frozen=False,
         error=0,
+        scan_min=None,
+        scan_max=None,
+        scan_n_values=11,
+        scan_n_sigma=2,
+        scan_values=None,
+        scale_method="scale10",
+        interp="lin",
+        is_norm=False,
     ):
-        self.name = name
+        if not isinstance(name, str):
+            raise TypeError(f"Name must be string, got '{type(name)}' instead")
+
+        self._name = name
         self._link_label_io = None
         self.scale = scale
         self.min = min
         self.max = max
         self.frozen = frozen
         self._error = error
+        self._is_norm = is_norm
         self._type = None
 
         # TODO: move this to a setter method that can be called from `__set__` also!
@@ -97,9 +137,18 @@ class Parameter:
             self.factor = value
             self.unit = unit
 
+        self.scan_min = scan_min
+        self.scan_max = scan_max
+        self.scan_values = scan_values
+        self.scan_n_values = scan_n_values
+        self.scan_n_sigma = scan_n_sigma
+        self.interp = interp
+        self.scale_method = scale_method
+
     def __get__(self, instance, owner):
         if instance is None:
             return self
+
         par = instance.__dict__[self.name]
         par._type = getattr(instance, "type", None)
         return par
@@ -111,6 +160,15 @@ class Parameter:
             par = instance.__dict__[self.name]
             raise TypeError(f"Cannot assign {value!r} to parameter {par!r}")
 
+    def __set_name__(self, owner, name):
+        if not self._name == name:
+            raise ValueError(f"Expected parameter name '{name}', got {self._name}")
+
+    @property
+    def is_norm(self):
+        """Whether the parameter represents the norm of the model"""
+        return self._is_norm
+
     @property
     def type(self):
         return self._type
@@ -128,12 +186,6 @@ class Parameter:
         """Name (str)."""
         return self._name
 
-    @name.setter
-    def name(self, val):
-        if not isinstance(val, str):
-            raise TypeError(f"Invalid type: {val}, {type(val)}")
-        self._name = val
-
     @property
     def factor(self):
         """Factor (float)."""
@@ -168,7 +220,11 @@ class Parameter:
 
     @min.setter
     def min(self, val):
-        self._min = float(val)
+        "Astropy Table has masked values for NaN. Replacing with np.nan."
+        if isinstance(val, np.ma.core.MaskedConstant):
+            self._min = np.nan
+        else:
+            self._min = float(val)
 
     @property
     def factor_min(self):
@@ -185,7 +241,11 @@ class Parameter:
 
     @max.setter
     def max(self, val):
-        self._max = float(val)
+        "Astropy Table has masked values for NaN. Replacing with np.nan."
+        if isinstance(val, np.ma.core.MaskedConstant):
+            self._max = np.nan
+        else:
+            self._max = float(val)
 
     @property
     def factor_max(self):
@@ -195,6 +255,17 @@ class Parameter:
         """
         return self.max / self.scale
 
+    @property
+    def scale_method(self):
+        """Method used to set ``factor`` and ``scale``"""
+        return self._scale_method
+
+    @scale_method.setter
+    def scale_method(self, val):
+        if val not in ["scale10", "factor1"] and val is not None:
+            raise ValueError(f"Invalid method: {val}")
+        self._scale_method = val
+
     @property
     def frozen(self):
         """Frozen? (used in fitting) (bool)."""
@@ -202,7 +273,9 @@ class Parameter:
 
     @frozen.setter
     def frozen(self, val):
-        if not isinstance(val, bool):
+        if val in ["True", "False"]:
+            val = bool(val)
+        if not isinstance(val, bool) and not isinstance(val, np.bool_):
             raise TypeError(f"Invalid type: {val}, {type(val)}")
         self._frozen = val
 
@@ -222,10 +295,92 @@ class Parameter:
 
     @quantity.setter
     def quantity(self, val):
-        val = u.Quantity(val, unit=self.unit)
+        val = u.Quantity(val)
+
+        if not val.unit.is_equivalent(self.unit):
+            raise u.UnitConversionError(
+                f"Unit must be equivalent to {self.unit} for parameter {self.name}"
+            )
+
         self.value = val.value
         self.unit = val.unit
 
+    # TODO: possibly allow to set this independently
+    @property
+    def conf_min(self):
+        """Confidence min value (`float`)
+
+        Returns parameter minimum if defined else the scan_min
+        """
+        if not np.isnan(self.min):
+            return self.min
+        else:
+            return self.scan_min
+
+    # TODO: possibly allow to set this independently
+    @property
+    def conf_max(self):
+        """Confidence max value (`float`)
+
+        Returns parameter maximum if defined else the scan_max
+        """
+        if not np.isnan(self.max):
+            return self.max
+        else:
+            return self.scan_max
+
+    @property
+    def scan_min(self):
+        """Stat scan min"""
+        if self._scan_min is None:
+            return self.value - self.error * self.scan_n_sigma
+
+        return self._scan_min
+
+    @property
+    def scan_max(self):
+        """Stat scan max"""
+        if self._scan_max is None:
+            return self.value + self.error * self.scan_n_sigma
+
+        return self._scan_max
+
+    @scan_min.setter
+    def scan_min(self, value):
+        """Stat scan min setter"""
+        self._scan_min = value
+
+    @scan_max.setter
+    def scan_max(self, value):
+        """Stat scan max setter"""
+        self._scan_max = value
+
+    @property
+    def scan_n_sigma(self):
+        """Stat scan n sigma"""
+        return self._scan_n_sigma
+
+    @scan_n_sigma.setter
+    def scan_n_sigma(self, n_sigma):
+        """Stat scan n sigma"""
+        self._scan_n_sigma = int(n_sigma)
+
+    @property
+    def scan_values(self):
+        """Stat scan values (`~numpy.ndarray`)"""
+        if self._scan_values is None:
+            scale = interpolation_scale(self.interp)
+            parmin, parmax = scale([self.scan_min, self.scan_max])
+            values = np.linspace(parmin, parmax, self.scan_n_values)
+            return scale.inverse(values)
+
+        return self._scan_values
+
+    @scan_values.setter
+    def scan_values(self, values):
+        """Set scan values"""
+        self._scan_values = values
+
     def check_limits(self):
         """Emit a warning or error if value is outside the min/max range"""
         if not self.frozen:
@@ -247,28 +402,39 @@ class Parameter:
         """A deep copy"""
         return copy.deepcopy(self)
 
+    def update_from_dict(self, data):
+        """Update parameters from a dict.
+        Protection against changing parameter model, type, name."""
+        keys = ["value", "unit", "min", "max", "frozen"]
+        for k in keys:
+            setattr(self, k, data[k])
+
     def to_dict(self):
         """Convert to dict."""
         output = {
             "name": self.name,
             "value": self.value,
             "unit": self.unit.to_string("fits"),
+            "error": self.error,
             "min": self.min,
             "max": self.max,
             "frozen": self.frozen,
-            "error": self.error,
+            "interp": self.interp,
+            "scale_method": self.scale_method,
+            "is_norm": self.is_norm,
         }
 
         if self._link_label_io is not None:
             output["link"] = self._link_label_io
+
         return output
 
-    def autoscale(self, method="scale10"):
+    def autoscale(self):
         """Autoscale the parameters.
 
-        Set ``factor`` and ``scale`` according to ``method``
+        Set ``factor`` and ``scale`` according to ``scale_method`` attribute
 
-        Available methods:
+        Available ``scale_method``
 
         * ``scale10`` sets ``scale`` to power of 10,
           so that abs(factor) is in the range 1 to 10
@@ -276,23 +442,19 @@ class Parameter:
 
         In both cases the sign of value is stored in ``factor``,
         i.e. the ``scale`` is always positive.
+        If ``scale_method`` is None the scaling is ignored.
 
-        Parameters
-        ----------
-        method : {'factor1', 'scale10'}
-            Method to apply
         """
-        if method == "scale10":
+        if self.scale_method == "scale10":
             value = self.value
             if value != 0:
                 exponent = np.floor(np.log10(np.abs(value)))
                 scale = np.power(10.0, exponent)
                 self.factor = value / scale
                 self.scale = scale
-        elif method == "factor1":
+
+        elif self.scale_method == "factor1":
             self.factor, self.scale = 1, self.value
-        else:
-            raise ValueError(f"Invalid method: {method}")
 
 
 class Parameters(collections.abc.Sequence):
@@ -326,12 +488,40 @@ class Parameters(collections.abc.Sequence):
         return [par.type for par in self]
 
     @property
-    def values(self):
+    def min(self):
+        """Parameter mins (`numpy.ndarray`)."""
+        return np.array([_.min for _ in self._parameters], dtype=np.float64)
+
+    @min.setter
+    def min(self, min_array):
+        """Parameter minima (`numpy.ndarray`)."""
+        if not len(self) == len(min_array):
+            raise ValueError("Minima must have same length as parameter list")
+
+        for min_, par in zip(min_array, self):
+            par.min = min_
+
+    @property
+    def max(self):
+        """Parameter maxima (`numpy.ndarray`)."""
+        return np.array([_.max for _ in self._parameters], dtype=np.float64)
+
+    @max.setter
+    def max(self, max_array):
+        """Parameter maxima (`numpy.ndarray`)."""
+        if not len(self) == len(max_array):
+            raise ValueError("Maxima must have same length as parameter list")
+
+        for max_, par in zip(max_array, self):
+            par.max = max_
+
+    @property
+    def value(self):
         """Parameter values (`numpy.ndarray`)."""
         return np.array([_.value for _ in self._parameters], dtype=np.float64)
 
-    @values.setter
-    def values(self, values):
+    @value.setter
+    def value(self, values):
         """Parameter values (`numpy.ndarray`)."""
         if not len(self) == len(values):
             raise ValueError("Values must have same length as parameter list")
@@ -388,10 +578,13 @@ class Parameters(collections.abc.Sequence):
         else:
             raise TypeError(f"Invalid type: {type(val)!r}")
 
-    def __getitem__(self, name):
-        """Access parameter by name or index"""
-        idx = self.index(name)
-        return self._parameters[idx]
+    def __getitem__(self, key):
+        """Access parameter by name, index or boolean mask"""
+        if isinstance(key, np.ndarray) and key.dtype == bool:
+            return self.__class__(list(np.array(self._parameters)[key]))
+        else:
+            idx = self.index(key)
+            return self._parameters[idx]
 
     def __len__(self):
         return len(self._parameters)
@@ -412,7 +605,15 @@ class Parameters(collections.abc.Sequence):
 
     def to_table(self):
         """Convert parameter attributes to `~astropy.table.Table`."""
-        rows = [p.to_dict() for p in self._parameters]
+        rows = []
+        for p in self._parameters:
+            d = p.to_dict()
+            if "link" not in d:
+                d["link"] = ""
+            for key in ["scale_method", "interp"]:
+                if key in d:
+                    del d[key]
+            rows.append({**dict(type=p.type), **d})
         table = table_from_row_data(rows)
 
         table["value"].format = ".4e"
@@ -428,11 +629,13 @@ class Parameters(collections.abc.Sequence):
     @classmethod
     def from_dict(cls, data):
         parameters = []
+
         for par in data:
             link_label = par.pop("link", None)
             parameter = Parameter(**par)
             parameter._link_label_io = link_label
             parameters.append(parameter)
+
         return cls(parameters=parameters)
 
     def set_parameter_factors(self, factors):
@@ -446,46 +649,94 @@ class Parameters(collections.abc.Sequence):
                 parameter.factor = factors[idx]
                 idx += 1
 
-    def autoscale(self, method="scale10"):
+    def autoscale(self):
         """Autoscale all parameters.
 
         See :func:`~gammapy.modeling.Parameter.autoscale`
 
-        Parameters
-        ----------
-        method : {'factor1', 'scale10'}
-            Method to apply
         """
         for par in self._parameters:
-            par.autoscale(method)
+            par.autoscale()
 
-    @property
-    def restore_values(self):
-        """Context manager to restore values.
+    def select(
+        self,
+        name=None,
+        type=None,
+        frozen=None,
+    ):
+        """Create a mask of models, true if all conditions are verified
+
+        Parameters
+        ----------
+        name : str or list
+            Name of the parameter
+        type : {None, spatial, spectral, temporal}
+           type of models
+        frozen : bool
+            Select frozen parameters if True, exclude them if False.
+
+        Returns
+        -------
+        parameters : `Parameters`
+           Selected parameters
+        """
+        selection = np.ones(len(self), dtype=bool)
+
+        if name and not isinstance(name, list):
+            name = [name]
+
+        for idx, par in enumerate(self):
+            if name:
+                selection[idx] &= np.any([_ == par.name for _ in name])
+
+            if type:
+                selection[idx] &= type == par.type
+
+            if frozen is not None:
+                if frozen:
+                    selection[idx] &= par.frozen
+                else:
+                    selection[idx] &= ~par.frozen
+
+        return self[selection]
+
+    def freeze_all(self):
+        """Freeze all parameters"""
+        for par in self._parameters:
+            par.frozen = True
+
+    def unfreeze_all(self):
+        """Unfreeze all parameters (even those frozen by default)"""
+        for par in self._parameters:
+            par.frozen = False
+
+    def restore_status(self, restore_values=True):
+        """Context manager to restore status.
 
         A copy of the values is made on enter,
         and those values are restored on exit.
 
+        Parameters
+        ----------
+        restore_values : bool
+            Restore values if True, otherwise restore only frozen status.
+
         Examples
         --------
         ::
 
             from gammapy.modeling.models import PowerLawSpectralModel
             pwl = PowerLawSpectralModel(index=2)
-            with pwl.parameters.restore_values:
+            with pwl.parameters.restore_status():
                 pwl.parameters["index"].value = 3
             print(pwl.parameters["index"].value)
         """
-        return restore_parameters_values(self)
-
-    def freeze_all(self):
-        """Freeze all parameters"""
-        for par in self._parameters:
-            par.frozen = True
+        return restore_parameters_status(self, restore_values)
 
 
-class restore_parameters_values:
-    def __init__(self, parameters):
+class restore_parameters_status:
+    def __init__(self, parameters, restore_values=True):
+        self.restore_values = restore_values
         self._parameters = parameters
         self.values = [_.value for _ in parameters]
         self.frozen = [_.frozen for _ in parameters]
@@ -495,5 +746,6 @@ class restore_parameters_values:
 
     def __exit__(self, type, value, traceback):
         for value, par, frozen in zip(self.values, self._parameters, self.frozen):
-            par.value = value
+            if self.restore_values:
+                par.value = value
             par.frozen = frozen
