@@ -52,7 +52,6 @@ def members(tf):
 def extract_bundle(bundle, destination):
     with tarfile.open(bundle) as tar:
         tar.extractall(path=destination, members=members(tar))
-    Path(bundle).unlink()
 
 
 def show_info_notebooks(outfolder, release):
@@ -87,12 +86,18 @@ def cli_download_notebooks(release, out):
     """Download notebooks"""
     localfolder = Path(out) / release
     url_file_env = f"{ENVS_BASE_URL}/gammapy-{release}-environment.yml"
+    if release == "dev":
+        url_file_env = "https://raw.githubusercontent.com/gammapy/gammapy/master/environment-dev.yml"
+
     yaml_destination_file = localfolder / f"gammapy-{release}-environment.yml"
     progress_download(url_file_env, yaml_destination_file)
     url_tar_notebooks = f"{NBTAR_BASE_URL}/{release}/_downloads/notebooks-{release}.tar"
     tar_destination_file = localfolder / f"notebooks_{release}.tar"
     progress_download(url_tar_notebooks, tar_destination_file)
-    extract_bundle(tar_destination_file, localfolder)
+    my_tar = tarfile.open(tar_destination_file)
+    my_tar.extractall(localfolder)
+    my_tar.close()
+    Path(tar_destination_file).unlink()
     show_info_notebooks(localfolder, release)
 
 
@@ -111,4 +116,5 @@ def cli_download_datasets(out):
     progress_download(TAR_DATASETS, tar_destination_file)
     log.info(f"Extracting {tar_destination_file}")
     extract_bundle(tar_destination_file, localfolder)
+    Path(tar_destination_file).unlink()
     show_info_datasets(localfolder)
