@@ -7,7 +7,43 @@ from pathlib import Path
 from uuid import uuid4
 import yaml
 
-__all__ = ["read_yaml", "write_yaml", "make_path", "recursive_merge_dicts"]
+__all__ = [
+    "get_images_paths",
+    "get_notebooks_paths",
+    "make_path",
+    "read_yaml",
+    "recursive_merge_dicts",
+    "write_yaml",
+]
+
+PATH_DOCS = Path(__file__).resolve().parent / ".." / ".." / "docs"
+SKIP = ["_static", "_build", "_checkpoints", "docs/user-guide/model-gallery/"]
+
+
+def get_notebooks_paths(folder=PATH_DOCS):
+    """Generator yields a Path for each notebook.
+
+    Parameters
+    ----------
+    folder : str
+        Folder where to search
+    """
+    for i in Path(folder).rglob("*.ipynb"):
+        if not any(s in str(i) for s in SKIP):
+            yield i.resolve()
+
+
+def get_images_paths(folder=PATH_DOCS):
+    """Generator yields a Path for each image used in notebook.
+
+    Parameters
+    ----------
+    folder : str
+        Folder where to search
+    """
+    for i in Path(folder).rglob("images/*"):
+        if not any(s in str(i) for s in SKIP):
+            yield i.resolve()
 
 
 def read_yaml(filename, logger=None):
@@ -74,7 +110,10 @@ def make_path(path):
     # TODO: raise error or warning if environment variables that don't resolve are used
     # e.g. "spam/$DAMN/ham" where `$DAMN` is not defined
     # Otherwise this can result in cryptic errors later on
-    return Path(os.path.expandvars(path))
+    if path is None:
+        return None
+    else:
+        return Path(os.path.expandvars(path))
 
 
 def recursive_merge_dicts(a, b):
